#!/usr/bin/env python
'''Main backend server object'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"
__version__ = "0.1"

import gobject

from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger

# Entertainer backend core
from entertainerlib.backend.core.message import Message
from entertainerlib.backend.core.message_bus import MessageBus
from entertainerlib.backend.core.message_scheduler import MessageScheduler
from entertainerlib.backend.core.connection_server import ConnectionServer
from entertainerlib.backend.core.message_type_priority import (
    MessageType, MessagePriority)

# Entertainer backend components
from entertainerlib.backend.components.feeds.feed_manager import FeedManager
from entertainerlib.backend.components.mediacache.media_cache_manager import (
    MediaCacheManager)

class BackendServer:
    """
    Entertainer backend server.

    This is a backend of Entertainer media center. Backend is responsible of
    many vital things like updateing media library cache, recording scheduled
    TV-shows, fetching RSS-feeds etc.
    """

    def __init__(self):
        """
        Initialize backend

        This method creates and initializes all backend components.
        """
        gobject.threads_init()

        self.config = Configuration()
        self.logger = Logger().getLogger('backend.BackendServer')
        self.message_bus = MessageBus()
        self._port = self.config.get_port()

        # Connection server - Thread that listens incoming socket connections
        self.connection_server = None

        # Config files
        self.content_config = None
        self.preferences = None

        self.scheduler = None
        self.feed_manager = None
        self.guide_updater = None
        self.media_manager = None

        # The order of the initialize method calls is significant! Don't change
        # the order unless you know what your doing!
        self.initialize_configuration()
        self.initialize_feed_manager()
        self.initialize_media_cache_manager()
        self.initialize_connection_server()
        self.initialize_scheduler()

    def initialize_configuration(self):
        """Initialize configuration"""
        cfg_dict = {
            MessageType.CONTENT_CONF_UPDATED : MessagePriority.VERY_HIGH,
            MessageType.PREFERENCES_CONF_UPDATED : MessagePriority.VERY_HIGH
            }
        self.message_bus.registerMessageHandler(self.config, cfg_dict)
        self.logger.debug("Configuration intialized successfully")

    def initialize_connection_server(self):
        """Initialize connection server."""
        self.connection_server = ConnectionServer(self._port,
                                                  self.message_bus)
        # Start listening incoming connections
        self.connection_server.start()

    def initialize_scheduler(self):
        """Initialize message scheduler."""
        self.scheduler = MessageScheduler(self.message_bus)
        self.scheduler.addMessage(Message(MessageType.UPDATE_FEEDS),
            60 * self.config.get_feed_fetch_interval())
        self.logger.debug("Message scheduler intialized successfully")

    def initialize_feed_manager(self):
        """Initialize feed manager."""
        self.feed_manager = FeedManager(self.message_bus)
        feed_mgr_dict = { MessageType.UPDATE_FEEDS : MessagePriority.HIGH,
                        MessageType.REBUILD_FEED_CACHE : MessagePriority.HIGH }
        self.message_bus.registerMessageHandler(self.feed_manager,
                                                feed_mgr_dict)
        self.logger.debug("Feed Manager intialized successfully")

    def initialize_media_cache_manager(self):
        '''Initialize the media cache manager'''
        self.media_manager = MediaCacheManager()
        media_dict = {
            MessageType.CONTENT_CONF_UPDATED : MessagePriority.VERY_LOW,
            MessageType.REBUILD_IMAGE_CACHE : MessagePriority.HIGH,
            MessageType.REBUILD_MUSIC_CACHE : MessagePriority.HIGH,
            MessageType.REBUILD_VIDEO_CACHE : MessagePriority.HIGH
            }
        self.message_bus.registerMessageHandler(self.media_manager, media_dict)
        self.logger.debug("Media Manager intialized successfully")

    def quitBackend(self):
        '''Close the backend server'''
        self.message_bus.unregisterAllMessageHandlers()

