'''FeedManager - Manage feeds'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
from pysqlite2 import dbapi2 as sqlite
from entertainerlib.backend.components.feeds.feed_fetcher import FeedFetcher

from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger

# Messaging system
from entertainerlib.backend.core.message_type_priority import MessageType
from entertainerlib.backend.core.message_handler import MessageHandler

class FeedManager(MessageHandler):
    """Backend feed manager keeps feeds cache database updated."""

    def __init__(self, message_bus):
        """
        Create a new FeedManager object
        @param message_bus: MessageBus object
        """
        self.message_bus = message_bus
        self.logger = Logger().getLogger(
            'backend.components.feeds.FeedManager')
        self.config = Configuration()

        if not os.path.exists(self.config.FEED_DB):
            self.createFeedCacheDatabase()
        MessageHandler.__init__(self)
        self.updateFeedCache()

    def updateFeedCache(self):
        """
        Update all feeds to cache in a new thread and after
        that emit FEED_DB_UPDATED message to the messagebus.
        """
        fetch_thread = FeedFetcher(self.message_bus,
                                   self.config.get_feeds())
        fetch_thread.start()

    def createFeedCacheDatabase(self):
        """Create a feed cache database file and tables."""
        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""CREATE TABLE feed(
                                url TEXT,
                                title TEXT,
                                subtitle TEXT,
                                description TEXT,
                                time TIME,
                                date DATE,
                                PRIMARY KEY(url))""")

        db_cursor.execute("""CREATE TABLE entry(
                                feed_url TEXT,
                                title TEXT,
                                description TEXT,
                                isread TEXT,
                                time TIME,
                                date DATE,
                                id TEXT,
                                PRIMARY KEY(id))""")
        db_conn.commit()
        db_conn.close()

    def wipeFeedCacheDatabase(self):
        """Wipes the feed cache database file and tables."""
        # all we need to do to wipe the cache database is to drop the two
        # tables containing the data: feed and entry
        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""DROP TABLE IF EXISTS feed""")
        db_cursor.execute("""DROP TABLE IF EXISTS entry""")
        db_conn.commit()
        db_conn.close()

    # Implementes MessageHandler interface
    def handleMessage(self, message):
        """
        Handle the received message
        @param message: Received Message object
        """
        if message.get_type() == MessageType.UPDATE_FEEDS:
            self.updateFeedCache()
        elif message.get_type() == MessageType.REBUILD_FEED_CACHE:
            #rebuild feed cache
            self.rebuildFeedCacheDatabase()

    def rebuildFeedCacheDatabase(self):
        """Destroy all current data and index everything from the scratch."""
        self.logger.info("Feed cache rebuilding requested")
        # we remove the tables from the database the create them again,
        # this wipes the tables
        self.wipeFeedCacheDatabase()
        self.createFeedCacheDatabase()
        # there will be no data in the tables so we need to populate
        # the feed cache
        self.updateFeedCache()
        self.logger.info("Feed cache rebuilt")

