'''Entertainer Frontend - This is a client side of Entertainer.'''

__licence__ = 'GPLv2'
__copyright__ = ('2007, Lauri Taimila', '2009, Matt Layman')
__author__ = ('Lauri Taimila <lauri@taimila.com>',
              'Matt Layman <laymansterms.dev@gmail.com>')

import sys

import gtk

from entertainerlib.frontend.backend_connection import BackendConnection
from entertainerlib.frontend.gui.user_interface import UserInterface
from entertainerlib.frontend.medialibrary.feeds import FeedLibrary
from entertainerlib.frontend.medialibrary.music import MusicLibrary
from entertainerlib.frontend.medialibrary.images import ImageLibrary
from entertainerlib.frontend.medialibrary.videos import VideoLibrary
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger
from entertainerlib.utils.system_tray_icon import SystemTrayIcon

class FrontendClient:
    '''
    Entertainer frontend

    This is a frontend application of the Entertainer. Frontend is a GUI part
    that user sees on the screen. This class is a core of the frontend. Frontend
    connects to the backend's messagebus at startup.
    '''

    def __init__(self):
        '''
        Create a new frontend.

        This initializes all frontend stuff like media librarys, remote
        control receiver and GUI. After this we just wait user actions.
        '''
        config = Configuration()
        self.logger = Logger().getLogger('frontend.FrontendClient')
        self.backend_connection = self.initialize_backend_connection()
        feed_library = FeedLibrary(self.backend_connection)
        music_library = MusicLibrary(self.backend_connection)
        image_library = ImageLibrary(self.backend_connection)
        video_library = VideoLibrary(self.backend_connection)
        self.ui = UserInterface(
            feed_library, image_library, music_library, video_library,
            self.quit_frontend)

        if config.tray_icon_enabled():
            SystemTrayIcon(
                self.quit_frontend, self.toggle_interface_visibility)

    def start(self):
        '''Start the necessary main loop.'''
        self.ui.start_up()
        self.interface_visible = True
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()

    def initialize_backend_connection(self):
        '''Connect to the backend server.'''
        backend_connection = BackendConnection()
        self.logger.debug('Connected to the Entertainer backend server.')

        return backend_connection

    def quit_frontend(self):
        '''Clean up the connection to the backend then close the frontend.'''
        self.backend_connection.close_connection()

        gtk.main_quit()
        sys.exit(0)

    def toggle_interface_visibility(self):
        '''Toggle between showing and hiding the interface's visibility.'''
        if self.interface_visible:
            self.ui.hide()
            self.interface_visible = False
        else:
            self.ui.show()
            self.interface_visible = True

