'''Disc - Screen allows user to browse and play tracks from CD album.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os

import cdrom
import gtk
import gobject
import pango
import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.frontend.medialibrary.playlist import Playlist
from entertainerlib.utils.albumart_downloader import AlbumArtDownloader

class Disc(Screen):
    """
    Disc

    Screen that allows user to browse and play tracks from the current Audio CD.
    """

    def __init__(self, music_library, media_player):
        Screen.__init__(self)

        self.theme = self.config.theme
        self.music_library = music_library
        self.media_player = media_player
        # When album info is loaded we create Playlist object
        self.playlist = None

        self.art = None
        self.art2 = None
        self.in_behaviour = None
        self.out_behaviour = None
        self.li = None
        self.track_menu = None

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Audio Disc"),
            "screen_title")
        self.add(screen_title)

        # Display throbber animation while loading CD metadata
        self.throbber = LoadingAnimation(40, clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        self.throbber.set_position(self.get_abs_x(0.5), self.get_abs_y(0.5))
        self.throbber.show()
        self.add(self.throbber)

        # Create and initialize screen items
        self.has_disc = True
        gobject.timeout_add(500, self._get_disc_information)

    def _get_disc_information(self):
        """
        Fetch album information from the Internet and create widgets based
        on received data.
        """
        try:
            disc = self.music_library.get_compact_disc_information()

            title = disc.get_title()
            artist = disc.get_artist()
            tracks = disc.get_tracks()

            self.playlist = Playlist(tracks)
            self._create_album_info(title, artist, tracks, disc.get_length())
            self._create_track_menu(tracks)
            self._create_album_cover_texture(artist, title)
            self._create_list_indicator()

            art_file = os.path.join(self.config.ALBUM_ART_DIR,
                artist + " - " + title + ".jpg")
            if artist != "Unknown artist" and not os.path.exists(art_file):
                art_search = AlbumArtDownloader(title, artist,
                    self.config.ALBUM_ART_DIR, self._update_albumart)
                art_search.start()

        except cdrom.error:
            # No disc in drive
            self.has_disc = False
            no_disc = Label(0.05, "title", 0.5, 0.5,
                _("No audio disc in drive"))
            no_disc.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            self.add(no_disc)

        # Remove loading animation
        self.throbber.hide()
        self.remove(self.throbber)
        del self.throbber

        # This function should be called only once (gobject timeout)
        return False

    def _update_albumart(self, artist, title):
        """
        Search album art for current audio disc. This function is called only
        if album art doesn't exist already. If album art is found then we
        replace current disc icon with the new album art.
        @param artist: Artist name
        @param title: Album title
        """
        art_file = os.path.join(self.config.ALBUM_ART_DIR,
            artist + " - " + title + ".jpg")

        if os.path.exists(art_file):
            clutter.threads_enter()
            self.art2 = Texture(art_file, 0.1, 0.165)
            clutter.threads_leave()

            self.art2.set_size(self.get_abs_x(0.3148), self.get_abs_y(0.5599))
            self.art2.set_opacity(0)
            self.add(self.art2)

            timeline_in = clutter.Timeline(35, 26)
            alpha_in = clutter.Alpha(timeline_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourOpacity(0, 255, alpha_in)
            self.in_behaviour.apply(self.art2)

            timeline_out = clutter.Timeline(35, 26)
            alpha_out = clutter.Alpha(timeline_out, clutter.smoothstep_inc_func)
            self.out_behaviour = clutter.BehaviourOpacity(255, 0, alpha_out)
            self.out_behaviour.apply(self.art)

            timeline_out.start()
            timeline_in.start()

    def _create_album_cover_texture(self, artist, title):
        """
        Create a texture that is displayed next to track list. This texture
        displays album cover art.
        @param artist: Artist
        @param title: Title of the album
        """
        coverfile = os.path.join(self.config.ALBUM_ART_DIR,
            artist + " - " + title + ".jpg")

        if(os.path.exists(coverfile)):
            pixbuf = gtk.gdk.pixbuf_new_from_file(coverfile)
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file(self.theme.getImage("disc"))
        self.art = EyeCandyTexture(pixbuf, self.get_abs_x(0.3148),
            self.get_abs_y(0.5599))
        self.art.set_position(self.get_abs_x(0.1), self.get_abs_y(0.13))
        self.art.set_rotation(clutter.Y_AXIS, 25, 0, 0, 0)
        self.add(self.art)

    def _create_album_info(self, title, artist_name, tracks, length):
        """
        Create album info labels.
        @param title: Album title
        @param artist_name: Artist
        @param tracks: List of CompactDisc objects
        """
        album = Label(0.04167, "text", 0.50146, 0.13,
           artist_name + " - " + title, font_weight="bold")
        album.set_size(0.4393, 0.06510)
        album.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(album)

        minutes = str(length / 60)

        num_of_tracks = Label(0.02604, "subtitle", 0.50146, 0.18,
            _("%(total)s tracks, %(time)s minutes") % \
            {'total': len(tracks), 'time': minutes}, font_weight="bold")
        self.add(num_of_tracks)

    def _create_track_menu(self, tracks):
        """
        Create a track menu. This menu contains list of all tracks on album.
        @param tracks: List of CompactDisc objects
        """
        self.track_menu = TextMenu(self.theme, self.config.show_effects())
        self.track_menu.set_row_count(1)
        self.track_menu.set_visible_column_count(7)
        self.track_menu.set_item_size(self.get_abs_x(0.4393),
            self.get_abs_y(0.0781))
        self.track_menu.set_position(self.get_abs_x(0.4978),
            self.get_abs_y(0.2344))

        for index, track in enumerate(tracks):
            length = str(track.get_length() / 60) + ":" + \
                str(track.get_length() % 60).zfill(2)
            item = TextMenuItem(0.4393, 0.0781, track.get_title(), length)
            item.set_userdata(index)
            self.track_menu.add_actor(item)
        self.track_menu.set_active(True)
        self.add(self.track_menu)

    def _create_list_indicator(self):
        """
        Create list indicator widget
        """
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.config.get_stage_width() - \
            self.li.get_width() - self.get_abs_x(0.06589),
            self.config.get_stage_height() - self.get_abs_y(0.1953))
        self.li.set_delimiter(" | ")
        self.li.set_maximum(self.track_menu.get_number_of_items())
        self.add(self.li)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readable)"""
        return "Disc"

    def _move_menu(self, menu_direction):
        '''Move the text menu in the provided direction.'''
        self.track_menu.move(menu_direction)
        self.li.set_current(self.track_menu.get_current_position() + 1)

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        self._move_menu(TextMenu.UP)

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        self._move_menu(TextMenu.DOWN)

    def _handle_select(self):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        track_index = self.track_menu.get_current_menuitem().get_userdata()
        self.playlist.set_current(track_index)
        self.media_player.set_playlist(self.playlist)
        self.media_player.play()

    def handle_user_event(self, event):
        '''Handle user events unless there is no disc.'''
        if self.has_disc:
            Screen.handle_user_event(self, event)

