'''Feed - Screen that contains a list of entries in one feed'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import pango

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.utils.feed_utils import FeedEntryParser

class Feed(Screen):
    """
    Feed

    This screen is displayed when headlines are accessed for a specific feed.
    """

    def __init__(self, feed, move_to_new_screen_callback):
        Screen.__init__(self, move_to_new_screen_callback)

        self.theme = self.config.theme
        self.feed = feed
        self.menu = None

        self._create_entry_menu()

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Feed"))
        self.add(screen_title)

        # RSS Icon
        icon = Texture(self.theme.getImage("rss_icon"), 0.05, 0.07)
        self.add(icon)

        # RSS Feed Title
        title_text = FeedEntryParser().convert(self.feed.get_title())
        title = Label(0.065, "title", 0.13, 0.07, title_text)
        title.set_use_markup(True)
        title.width = 0.6
        title.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(title)

        # RSS Feed description
        desc_text = FeedEntryParser().convert(self.feed.get_subtitle())
        desc = Label(0.03646, "subtitle", 0.13, 0.14, desc_text)
        desc.set_use_markup(True)
        desc.width = 0.6
        desc.set_ellipsize(pango.ELLIPSIZE_END)
        self.add(desc)

        # Last update
        update = Label(0.0313, "title", 0.78, 0.1, _("Updated"))
        self.add(update)
        date_text = self.feed.get_time()[:5]  + " - " +  self.feed.get_date()
        date = Label(0.0313, "subtitle", 0.78, 0.14, date_text)
        self.add(date)

        #List indicator
        self.li = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.li.set_position(self.get_abs_x(0.7562) - self.li.get_width(),
            self.config.get_stage_height() - self.li.get_height() - \
            self.get_abs_y(0.03906))
        self.li.set_delimiter(" | ")
        self.li.set_maximum(self.menu.get_number_of_items())
        self.li.set_current(1)
        self.add(self.li)

    def _create_entry_menu(self):
        """
        Create Entries-menu. This menu contains list of entries.
        """
        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_row_count(1)
        self.menu.set_item_size(self.get_abs_x(0.5124), self.get_abs_y(0.0781))
        self.menu.set_position(self.get_abs_x(0.2438), self.get_abs_y(0.2604))

        # Create menu items based on FeedLibrary
        entries = self.feed.get_entries()
        for entry in entries:
            item = TextMenuItem(0.5124, 0.0781,
                FeedEntryParser().strip_tags(entry.get_title()))
            item.set_userdata(entry)
            self.menu.add_actor(item)

        self.menu.set_active(True)
        self.add(self.menu)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "Feed"

    def _move_menu(self, menu_direction):
        '''Move the text menu in the given direction.'''
        self.menu.move(menu_direction)
        self.li.set_current(self.menu.get_current_position() + 1)

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        self._move_menu(TextMenu.UP)

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        self._move_menu(TextMenu.DOWN)

    def _handle_select(self):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        entry = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'feed' : self.feed, 'entry' : entry }
        self.callback("entry", kwargs)

