'''Main - Screen has a main menu'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.widgets.clock_label import ClockLabel
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.scroll_menu import ScrollMenu
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem
from entertainerlib.utils.cd_utils import eject_cd
from entertainerlib.utils.feed_utils import FeedEntryParser

class Main(Screen):
    """
    Main

    This screen is displayed when frontend is opened. Screen includes menu
    that allows user to navigate media center. This is main screen of the
    frontend.
    """

    # Size of the preview area
    PREVIEW_WIDTH = 830
    PREVIEW_HEIGHT = 580

    # Directions
    UP = 0
    DOWN = 1

    def __init__(self, feed_library, media_player, move_to_new_screen_callback):
        Screen.__init__(self, move_to_new_screen_callback)

        self.feed_library = feed_library
        self.media_player = media_player

        self.theme = self.config.theme

        self.preview = clutter.Group() # Group that contains preview actors
        self.preview.set_position(self.get_abs_x(0.07), self.get_abs_y(0.1))
        self.preview.show()
        self.add(self.preview)

        self.active_menu = "main"
        self.rss_preview_menu = None # Pointer to the current preview menu

        self.menu = None
        self.playing_item = None
        self.create_main_menu()

        clock = ClockLabel(0.13, "screentitle", 0, 0.87)
        self.add(clock)

    def create_main_menu(self):
        """Create main menu of the home screen"""
        self.menu = ScrollMenu(10, 60, self.config.show_effects())
        self.menu.set_name("mainmenu")

        # Common values for all ScrollMenu items
        size = 0.045
        color = "menuitem_active"

        item0 = Label(size, color, 0, 0, _("Play CD"), "disc")
        self.menu.add(item0)

        item2 = Label(size, color, 0, 0, _("Videos"), "videos")
        self.menu.add(item2)

        item3 = Label(size, color, 0, 0, _("Music"), "music")
        self.menu.add(item3)

        item4 = Label(size, color, 0, 0, _("Photographs"), "photo")
        self.menu.add(item4)

        item5 = Label(size, color, 0, 0, _("Headlines"), "rss")
        self.menu.add(item5)

        if self.config.display_weather_in_frontend():
            item6 = Label(size, color, 0, 0, _("Weather"), "weather")
            self.menu.add(item6)

        if self.config.display_cd_eject_in_frontend():
            item7 = Label(size, color, 0, 0, _("Eject CD"), "eject_cd")
            self.menu.add(item7)

        if self.media_player.has_media():
            playing = Label(size, color, 0, 0, _("Playing now..."), "playing")
            self.menu.add(playing)

        self.menu.set_number_of_visible_items(5)
        menu_clip = self.menu.get_number_of_visible_items() * 70

        # Menu position
        menu_y = int((self.config.get_stage_height() - menu_clip + 10) / 2)
        self.menu.set_position(self.get_abs_x(0.75), menu_y)

        self.add(self.menu)

    def show_playing_preview(self):
        """Create a group that displays information on current media."""
        # Video preview of current media
        video_texture = self.media_player.get_texture()
        if video_texture == None:
            video_texture = Texture(self.theme.getImage("default_album_art"))
        width, height = video_texture.get_size()
        x_ratio = (self.PREVIEW_WIDTH - 50) / float(width)
        y_ratio = (self.PREVIEW_HEIGHT - 50) / float(height)

        if x_ratio > y_ratio:
            video_texture.set_scale((self.PREVIEW_HEIGHT - 50) / float(height),
                                    (self.PREVIEW_HEIGHT - 50) / float(height))
            new_width = int(width * \
                ((self.PREVIEW_HEIGHT - 50) / float(height)))
            new_x = int(((self.PREVIEW_WIDTH - 50) - new_width) / 2.0)
            video_texture.set_position(int(new_x), 0)
            # Below are size and position calculations for border rectangle
            rect_x = new_x -3
            rect_y = -3
            new_width = (self.PREVIEW_HEIGHT - 50) / float(height) * width
            new_height = (self.PREVIEW_HEIGHT - 50) / float(height) * height
        else:
            video_texture.set_scale((self.PREVIEW_WIDTH - 50) / float(width),
                                    (self.PREVIEW_WIDTH - 50) / float(width))
            new_height = int(height * \
                ((self.PREVIEW_WIDTH - 50) / float(width)))
            new_y = int(((self.PREVIEW_HEIGHT - 50) - new_height) / 2.0)
            video_texture.set_position(0, int(new_y))
            rect_x = -3
            rect_y = new_y -3
            # Below are size and position calculations for border rectangle
            new_width = (self.PREVIEW_WIDTH - 50) / float(width) * width
            new_height = (self.PREVIEW_WIDTH - 50) / float(width) * height

        # Video frame
        rect = clutter.Rectangle()
        rect.set_size(int(new_width + 6), int(new_height + 6))
        rect.set_position(rect_x, rect_y)
        rect.set_color((128, 128, 128, 192))
        self.preview.add(rect)

        self.preview.add(video_texture)

        title_text = _("Now Playing: %(title)s") % \
            {'title': self.media_player.get_media_title()}
        title = Label(0.03, "text", 0.03, 0.74, title_text)
        self.preview.add(title)

    def show_rss_preview(self):
        """Rss preview"""

        if self.feed_library.is_empty() is False:
            # RSS Icon
            icon = Texture(self.theme.getImage("rss_icon"), 0, 0.04)
            icon.set_scale(0.6, 0.6)

            # RSS Feed Title
            title = Label(0.075, "title", 0.045, 0.03, _("Recent headlines"))

            menu = TextMenu(self.theme, self.config.show_effects())
            menu.set_row_count(1)
            menu.set_position(self.get_abs_x(0.035), self.get_abs_y(0.12))
            menu.set_item_size(self.get_abs_x(0.549), self.get_abs_y(0.078))

            # List of latest entries. pack = (Feed object, Entry object)
            entries = self.feed_library.get_latest_entries(5)
            for pack in entries:
                text = pack[0].get_title() + " - " + pack[1].get_title()
                item = TextMenuItem(0.549, 0.078,
                    FeedEntryParser().strip_tags(text),
                    pack[1].get_date())
                kwargs = { 'feed' : pack[0], 'entry' : pack[1] }
                item.set_userdata(kwargs)
                menu.add_actor(item)

            menu.set_active(False)

            # Fade in timeline
            fade_in = clutter.Timeline(20, 60)
            alpha_in = clutter.Alpha(fade_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourOpacity(0x00, 0xff, alpha_in)
            self.in_behaviour.apply(menu)
            self.in_behaviour.apply(title)
            self.in_behaviour.apply(icon)

            self.preview.add(icon)
            self.preview.add(title)
            self.rss_preview_menu = menu
            self.preview.add(menu)

            menu.set_opacity(0x00)
            icon.set_opacity(0x00)
            title.set_opacity(0x00)

            fade_in.start()

        else:
            # No headlines available in the library
            info = Label(0.07, "title", 0.1, 0.35, _("No headlines available"))
            self.preview.add(info)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "Main"

    def update(self):
        """
        Update screen widgets. This is called always when screen is poped from
        the screen history. Updates main menu widget.
        """
        selected_name = self.menu.get_selected().get_name()

        self.remove(self.menu)
        self.create_main_menu()

        index = self.menu.get_index(selected_name)
        if not index == -1:
            self.menu.select(index)

        self.menu.update()

        self.update_preview_area(self.menu.get_selected())

    def update_preview_area(self, item):
        """
        Update preview area. This area displayes information of currently
        selected menuitem.
        @param item: Menuitem (clutter.Actor)
        """
        self.preview.remove_all()
        if item.get_name() == "playing":
            self.show_playing_preview()
        #only show the rss menu if it has entries
        elif item.get_name() == "rss":
            self.show_rss_preview()

    def _move_menu(self, menu_direction):
        '''Move the menu in the given direction.'''

        if menu_direction == self.UP:
            scroll_direction = self.menu.scroll_down
            preview_direction = TextMenu.UP
        elif menu_direction == self.DOWN:
            scroll_direction = self.menu.scroll_up
            preview_direction = TextMenu.DOWN

        if self.active_menu == "main":
            scroll_direction()
            selected = self.menu.get_selected()
            if selected.get_name() == None:
                scroll_direction()
                selected = self.menu.get_selected()
            self.update_preview_area(selected)
        else:
            self.rss_preview_menu.move(preview_direction)

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        self._move_menu(self.UP)

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        self._move_menu(self.DOWN)

    def _handle_left(self):
        '''Handle UserEvent.NAVIGATE_LEFT.'''
        if self.feed_library.is_empty():
            return

        if self.active_menu == "main":
            self.active_menu = "preview"
            self.menu.set_active(False)
            self.rss_preview_menu.set_active(True)

    def _handle_right(self):
        '''Handle UserEvent.NAVIGATE_RIGHT.'''
        if self.feed_library.is_empty():
            return

        if self.active_menu == "preview":
            self.active_menu = "main"
            self.menu.set_active(True)
            self.rss_preview_menu.set_active(False)

    def _handle_select(self):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        item = self.menu.get_selected()

        if item.get_name() == "disc":
            self.callback("audio_cd")
        elif item.get_name() == "playing":
            self.callback("video_osd")
        elif item.get_name() == "music":
            self.callback("music")
        elif item.get_name() == "videos":
            self.callback("video")
        elif item.get_name() == "weather":
            self.callback("weather")
        elif item.get_name() == "eject_cd":
            eject_cd()
        elif item.get_name() == "photo":
            self.callback("photo_albums")
        elif item.get_name() == "rss":
            if self.active_menu == "main":
                self.callback("rss")
            else:
                menu_item = self.rss_preview_menu.get_current_menuitem()
                kwargs = menu_item.get_userdata()
                self.callback("entry", kwargs)

