'''Movie - Screen allows user to view movie information'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
import gtk
import pango

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.user_event import UserEvent
from entertainerlib.frontend.gui.widgets.eyecandy_texture import (
    EyeCandyTexture)
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.scroll_area import ScrollArea
from entertainerlib.frontend.gui.widgets.texture import Texture
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem

class Movie(Screen):
    """
    Movie

    This screen contains information of one Movie.
    """

    def __init__(self, media_player, movie, move_to_new_screen_callback):
        Screen.__init__(self, move_to_new_screen_callback)

        self.theme = self.config.theme
        self.media_player = media_player
        self.movie = movie

        # Screen Title (Displayed at the bottom left corner)
        screen_title = Label(0.13, "screentitle", 0, 0.87, _("Movie"))
        self.add(screen_title)

        # Add the additional actions that are needed but not handled by default
        self.event_handlers.update({
            UserEvent.NAVIGATE_FIRST_PAGE : self._handle_first_page,
            UserEvent.NAVIGATE_LAST_PAGE : self._handle_last_page,
            UserEvent.NAVIGATE_PREVIOUS_PAGE : self._handle_previous_page,
            UserEvent.NAVIGATE_NEXT_PAGE : self._handle_next_page
        })

        self.menu = None
        self.scroll_area = None
        self.create_menu()
        self.create_movie_information()

    def create_movie_information(self):
        '''Create clutter parts related to movie information'''

        # Movie art texture
        if self.movie.has_cover_art():
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.movie.get_cover_art_url())
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file(
                self.theme.getImage("default_movie_art"))
        movie_art = EyeCandyTexture(pixbuf, self.get_abs_x(0.1),
            self.get_abs_y(0.25))
        movie_art.set_position(self.get_abs_x(0.33), self.get_abs_y(0.1))
        self.add(movie_art)

        # Movie title
        title = Label(0.04, "title", 0.47, 0.1, self.movie.get_title(),
            font_weight="bold")
        title.set_ellipsize(pango.ELLIPSIZE_END)
        title.set_size(0.5124, 0.05208)
        self.add(title)

        # Movie release year
        year_text = _("Released in %(year)s") % {'year': self.movie.get_year()}
        year = Label(0.032, "subtitle", 0.47, 0.3, year_text)
        year.set_ellipsize(pango.ELLIPSIZE_END)
        year.set_size(0.5124, 0.05208)
        self.add(year)

        # Show only 2 genres (or one if there is only one)
        genres = self.movie.get_genres()
        if len(genres) > 1:
            genre = genres[0] + "/" + genres[1]
        else:
            genre = genres[0]

        # Runtime and genres
        info_text = _("%(runtime)s min, %(genre)s") % \
            {'runtime': self.movie.get_runtime(), 'genre': genre}
        info = Label(0.032, "subtitle", 0.47, 0.24, info_text)
        info.set_ellipsize(pango.ELLIPSIZE_END)
        info.set_size(0.5124, 0.05208)
        self.add(info)

        # Stars (rating)
        star = Texture(self.theme.getImage("star"))
        star2 = Texture(self.theme.getImage("star2"))

        for i in range(self.movie.get_rating()):
            tex = clutter.CloneTexture(star)
            tex.set_position(
                self.get_abs_x(0.47) + (self.get_abs_x(0.0366) * i),
                self.get_abs_y(0.17))
            tex.set_size(self.get_abs_x(0.024), self.get_abs_y(0.04))
            self.add(tex)

        dark_star = 5 - self.movie.get_rating()
        for i in range(dark_star):
            tex = clutter.CloneTexture(star2)
            tex.set_position(self.get_abs_x(0.47) + (self.get_abs_x(0.0366) * \
                (i+ self.movie.get_rating())), self.get_abs_y(0.17))
            tex.set_size(self.get_abs_x(0.024), self.get_abs_y(0.04))
            self.add(tex)

        # Plot
        plot = Label(0.029, "subtitle", 0, 0, self.movie.get_plot())
        plot.set_justify(True)
        plot.set_line_wrap_mode(pango.WRAP_WORD)
        plot.set_line_wrap(True)
        plot.width = 0.5124
        self.scroll_area = ScrollArea(0.5124, 0.3516, 0.33, 0.38, plot)
        self.add(self.scroll_area)

        # Actors
        act_title = Label(0.032, "title", 0.33, 0.8, _("Starring"))
        self.add(act_title)

        actors_text = ", ".join(self.movie.get_actors()[:3])
        actors = Label(0.032, "subtitle", 0.46, 0.8, actors_text)
        actors.set_ellipsize(pango.ELLIPSIZE_END)
        actors.set_size(0.5124, 0.05208)
        self.add(actors)

        # Directors
        dict_title = Label(0.032, "title", 0.33, 0.86, _("Directed by"))
        self.add(dict_title)

        dlist = self.movie.get_directors()
        if len(dlist) > 1:
            director = dlist[0] + ", " + dlist[1]
        else:
            director = dlist[0]
        directors = Label(0.032, "subtitle", 0.46, 0.86, director)
        directors.set_ellipsize(pango.ELLIPSIZE_END)
        directors.set_size(0.5124, 0.05208)
        self.add(directors)

        # Writers
        wri_title = Label(0.032, "title", 0.33, 0.92, _("Written by"))
        self.add(wri_title)

        wlist = self.movie.get_writers()
        if len(wlist) > 1:
            writer = wlist[0] + ", " + wlist[1]
        else:
            writer = wlist[0]
        writers = Label(0.032, "subtitle", 0.46, 0.92, writer)
        writers.set_ellipsize(pango.ELLIPSIZE_END)
        writers.set_size(0.5124, 0.05208)
        self.add(writers)

    def create_menu(self):
        """
        Create menu of the screen. This displayed the left side of the screen.
        """
        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_row_count(1)
        #self.track_menu.set_visible_column_count(10)
        self.menu.set_item_size(self.get_abs_x(0.2196), self.get_abs_y(0.0781))
        self.menu.set_position(self.get_abs_x(0.07), self.get_abs_y(0.1))

        item1 = TextMenuItem(0.2196, 0.0781, _("Watch"))
        item1.set_userdata("watch")
        self.menu.add_actor(item1)

        self.menu.set_active(True)
        self.add(self.menu)

    def get_type(self):
        """Return screen type."""
        return Screen.NORMAL

    def get_name(self):
        """Return screen name (human readble)"""
        return "Movie"

    def _handle_up(self):
        '''Handle UserEvent.NAVIGATE_UP.'''
        if self.menu.is_active():
            self.menu.move(self.menu.UP)
        else:
            self.scroll_area.scroll_up()

    def _handle_down(self):
        '''Handle UserEvent.NAVIGATE_DOWN.'''
        if self.menu.is_active():
            self.menu.move(self.menu.DOWN)
        else:
            self.scroll_area.scroll_down()

    def _handle_left(self):
        '''Handle UserEvent.NAVIGATE_LEFT.'''
        self.menu.set_active(True)
        self.scroll_area.set_active(False)

    def _handle_right(self):
        '''Handle UserEvent.NAVIGATE_RIGHT.'''
        self.menu.set_active(False)
        self.scroll_area.set_active(True)

    def _handle_select(self):
        '''Handle UserEvent.NAVIGATE_SELECT.'''
        if self.menu.is_active():
            item = self.menu.get_current_menuitem().get_userdata()
            if item == "watch":
                self.mediaplayer.set_media(self.movie)
                self.mediaplayer.play()
                self.callback("video_osd")

    def _handle_first_page(self):
        '''Handle UserEvent.NAVIGATE_FIRST_PAGE.'''
        self.scroll_area.scroll_to_top()

    def _handle_last_page(self):
        '''Handle UserEvent.NAVIGATE_LAST_PAGE.'''
        self.scroll_area.scroll_to_bottom()

    def _handle_previous_page(self):
        '''Handle UserEvent.NAVIGATE_PREVIOUS_PAGE.'''
        self.scroll_area.scroll_page_up()

    def _handle_next_page(self):
        '''Handle UserEvent.NAVIGATE_NEXT_PAGE.'''
        self.scroll_area.scroll_page_down()

