'''Albums tab which displays albums and allows users to select them'''
# pylint: disable-msg=W1001

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import clutter
import gobject
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.texture import Texture

class AlbumsTab(Tab):
    '''Tab to show album listings'''

    def __init__(self, albums, move_to_new_screen_callback, name="albums",
        tab_title=_("Albums")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)

        # Start the loading animation while the menu is loading
        self.throbber = LoadingAnimation(40,
            clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        self.throbber.set_position(self.get_abs_x(0.1), self.get_abs_y(0.1))
        self.throbber.show()
        self.add(self.throbber)

        if len(albums) < 4:
            # XXX: laymansterms - This is "less than elegant." I'm not fixing
            # ImageMenu right now so we can't get rid of tex_size_x and y yet.
            # We really need to eliminate the concept tex_size_y, but it will
            # have to wait for now.
            x_percent = 0.2928
            tex_size_x = self.get_abs_x(x_percent)
            tex_size_y = self.get_abs_y(0.5208)
            row_count = 1
            visible = 3
        elif len(albums) < 13:
            x_percent = 0.1464
            tex_size_x = self.get_abs_x(x_percent)
            tex_size_y = self.get_abs_y(0.2604)
            row_count = 2
            visible = 6
        else:
            x_percent = 0.1098
            tex_size_x = self.get_abs_x(x_percent)
            tex_size_y = self.get_abs_y(0.1953)
            row_count = 3
            visible = 8

        # Create albums menu
        self.menu = ImageMenu(self.theme)
        self.menu.set_row_count(row_count)
        self.menu.set_visible_column_count(visible)
        self.menu.set_item_size(tex_size_x, tex_size_y)

        # Default texture
        self.default_cover = Texture(self.theme.getImage("default_album_art"))

        gobject.timeout_add(50, self._async_menu_build, albums, x_percent)

        self.menu.set_position(self.get_abs_x(0.0586), self.get_abs_y(0.1823))
        self.menu.set_active(False)
        self.menu.show()

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.add(self.menu)

        self.li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.li.set_position(self.get_abs_x(0.9297) - self.li.get_width(),
            self.get_abs_y(0.7943))
        self.li.set_delimiter(" | ")
        self.li.set_maximum(len(albums))
        self.li.set_current(1)
        self.li.show()
        self.add(self.li)

        # Create album information (displays current menuitem information)
        self.album_title = Label(0.045, "title", 0.22, 0.79, "")
        self.album_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_title.width = 0.366
        self.add(self.album_title)

        self.album_artist = Label(0.037, "subtitle", 0.22, 0.86, "")
        self.album_artist.set_ellipsize(pango.ELLIPSIZE_END)
        self.album_artist.width = 0.366
        self.add(self.album_artist)

        self.album_tracks = Label(0.037, "subtitle", 0.22, 0.91, "")
        self.add(self.album_tracks)

        self._selected_album = None
        gobject.timeout_add(50, self._init_selected_album)

    def _async_menu_build(self, albums, item_size_percent):
        """
        This function is called as an async callback. It is also a recursive
        function. This allows us to load menu items in the background without
        blocking the user interface.
        """

        # Check if we should stop recursive calls (menu is complete)
        if len(albums) > 0:
            album = albums[0]
            if album.has_album_art():
                texture = Texture(album.get_album_art_url())
            else:
                texture = clutter.CloneTexture(self.default_cover)

            item = ImageMenuItem(item_size_percent, 1, texture)
            item.set_userdata(album)
            self.menu.add_actor(item)
            # Recursive call, remove first album from the list
            gobject.timeout_add(25, self._async_menu_build, albums[1:],
                item_size_percent)
            return False
        else:
            self.throbber.hide()
            # Done loading the albums, notify gobject to stop calling menu build
            return False

    def _init_selected_album(self):
        '''The menu is loaded dynamically so the current menu item may not be
        available initially. Keep trying until the value is available'''
        try:
            self._selected_album = \
                self.menu.get_current_menuitem().get_userdata()
            # Stop gobject
            return False
        except IndexError:
            # Item must not be available yet, continue to call
            return True

    def _get_selected_album(self):
        '''Get the currently selected album from the menu'''
        return self.menu.get_current_menuitem().get_userdata()

    selected_album = property(_get_selected_album)

    def can_activate(self):
        '''Albums tab will always be created from an existing artist with at
        least one album'''
        return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_album_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _update_album_info(self, album):
        '''Update the album information labels'''
        self.album_title.set_text(album.get_title())
        self.album_artist.set_text(album.get_artist())
        self.album_tracks.set_text(_("%(total)s tracks") % \
            {'total': album.get_number_of_tracks()})

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_album_info(self.menu.get_current_menuitem().get_userdata())
        self.li.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_cursor_position()[1] == 0:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        album = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'album' : album }
        self.callback("album", kwargs)
        return False

