'''Artists tab for the music screen'''
# pylint: disable-msg=W1001

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import clutter
import gobject
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.loading_animation import (
    LoadingAnimation)
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem

class ArtistsTab(Tab):
    '''Tab for the music screen to show artist listings'''

    def __init__(self, music_library, artists, move_to_new_screen_callback,
        name="artists", tab_title=_("Artists")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)
        self.library = music_library

        # Start the loading animation while the menu is loading
        self.throbber = LoadingAnimation(40,
            clutter.Color(255, 255, 255, 255),
            clutter.Color(128, 128, 128, 128))
        self.throbber.set_position(self.get_abs_x(0.1), self.get_abs_y(0.1))
        self.throbber.show()
        self.add(self.throbber)

        self.menu = TextMenu(self.config.theme, self.config.show_effects())
        self.menu.set_item_size(self.get_abs_x(0.293), self.get_abs_y(0.078))
        cursor = clutter.Rectangle()
        cursor.set_color((0, 0, 0, 0))
        self.menu.set_cursor(cursor)
        self.menu.set_orientation(TextMenu.HORIZONTAL)
        self.menu.set_row_count(7)
        self.menu.set_visible_column_count(3)
        self.menu.set_position(self.get_abs_x(0.057), self.get_abs_y(0.208))
        self.menu.set_active(False)
        if self.config.show_effects():
            self.menu.set_animate(True)
        self.add(self.menu)

        # Create artist label
        self.artist_title = Label(0.0416, "title", 0.22, 0.794, "")
        self.artist_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.artist_title.width = 0.366
        self.add(self.artist_title)

        self.artist_albums = Label(0.0365, "subtitle", 0.22, 0.86, "")
        self.add(self.artist_albums)

        self.artist_tracks = Label(0.0365, "subtitle", 0.22, 0.911, "")
        self.add(self.artist_tracks)

        # Create artis menu list indicator
        self.li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.li.set_position(self.get_abs_x(0.929) - self.li.get_width(),
            self.get_abs_y(0.794))
        self.li.set_delimiter(" | ")
        self.li.set_maximum(len(artists))
        self.li.set_current(1)
        self.add(self.li)

        gobject.timeout_add(25, self._async_menu_build, artists)

        self._selected_artist = None
        gobject.timeout_add(25, self._init_selected_artist)

    def _async_menu_build(self, artists):
        """
        This function is called as an async callback. It is also a recursive
        function. This allows us to load menu items in the background without
        blocking the user interface.
        """

        # Check if we should stop recursive calls (menu is complete)
        if len(artists) > 0:
            artist = artists[0]
            item = TextMenuItem(0.293, 0.104, artist)
            item.justified = True
            item.set_userdata(artist)
            self.menu.add_actor(item)
            # Recursive call, remove first artist from the list
            gobject.timeout_add(15, self._async_menu_build, artists[1:])
            return False
        else:
            self.throbber.hide()
            # Done loading artists, notify gobject to stop calling menu build
            return False

    def _init_selected_artist(self):
        '''The menu is loaded dynamically so the current menu item may not be
        available initially. Keep trying until the value is available'''
        try:
            self._selected_artist = \
                self.menu.get_current_menuitem().get_userdata()
            # Stop gobject
            return False
        except IndexError:
            # Item must not be available yet, continue to call
            return True

    def _get_selected_artist(self):
        '''Get the currently selected artist from the menu'''
        return self.menu.get_current_menuitem().get_userdata()

    selected_artist = property(_get_selected_artist)

    def can_activate(self):
        '''Albums tab will always be created from an existing artist with at
        least one album'''
        return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_artist_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _update_artist_info(self, artist):
        '''Update the artist information labels'''
        self.artist_title.set_text(artist)
        self.artist_albums.set_text(_("%(albums)d albums") %
            {'albums': self.library.number_of_albums_by_artist(artist)})
        self.artist_tracks.set_text(_("%(tracks)d tracks") %
            {'tracks': self.library.number_of_tracks_by_artist(artist)})

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_artist_info(
            self.menu.get_current_menuitem().get_userdata())
        self.li.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_cursor_position()[1] == 0:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        artist = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'artist' : artist }
        self.callback("artist", kwargs)
        return False

