'''MoviesTab - MoviesTab is group of objects that are displayed on movie tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
import gtk
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.rounded_texture import RoundedTexture
from entertainerlib.frontend.gui.widgets.texture import Texture

class MoviesTab(Tab):
    """
    Tab can be used as part of the TabGroup

    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """

    def __init__(self, video_library, move_to_new_screen_callback,
        name="movies", tab_title=_("Movies")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)
        self.video_library = video_library
        self.theme = self.config.theme
        self.list_indicator = None
        self.movie_info = None
        self.menu = None
        self.movie_plot = None
        self.movie_title = None

        if self.video_library.get_number_of_movies() == 0:
            self._create_empty_library_notice()
        else:
            self._create_menu()

    def can_activate(self):
        """
        Allow if we have some movies indexed.
        """
        if self.video_library.get_number_of_movies() == 0:
            return False
        else:
            return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_movie_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _create_empty_library_notice(self):
        """
        Create an information box that is displayed if there are no indexed
        movies.
        """
        message = _(
            "There are no indexed movies in Entertainer media library.  To "
            "add movies, click on 'content' button on toolbar and open "
            "'videos' tab. Now click on 'add' button and select some folders "
            "which contains movie files.")
        Tab.show_empty_tab_notice(self, _("No movies available!"), message)

    def _create_menu(self):
        """
        Create a view that is displayed when there is indexed movies in
        the video library.
        """
        #Create movie menu
        self.menu = ImageMenu(self.theme)
        self.menu.set_item_size(self.get_abs_x(0.1), self.get_abs_y(0.25))
        self.menu.set_item_spacing(self.get_abs_y(0.025))
        self.menu.set_visible_column_count(2)
        self.menu.set_row_count(7)
        self.menu.set_orientation(self.menu.VERTICAL)

        # Default texture
        default_cover = Texture(self.theme.getImage("default_movie_art"))

        movies = self.video_library.get_movies()
        for movie in movies:
            if movie.has_cover_art():
                pix_buffer = gtk.gdk.pixbuf_new_from_file(
                    movie.get_cover_art_url())
                texture = RoundedTexture(pix_buffer, self.get_abs_x(0.1),
                                            self.get_abs_y(0.25))
            else:
                texture = clutter.CloneTexture(default_cover)
            item = ImageMenuItem(0.1, 1.41, texture)
            item.set_userdata(movie)
            self.menu.add_actor(item)

        self.menu.set_position(self.get_abs_x(0.11), self.get_abs_y(0.2))
        self.menu.set_active(False)
        self.menu.show()

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.add(self.menu)

        # Create list indicator
        self.list_indicator = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.list_indicator.set_position(self.get_abs_x(0.75),
            self.get_abs_y(0.76))
        self.list_indicator.set_delimiter(" | ")
        self.list_indicator.set_maximum(self.menu.get_number_of_items())
        self.list_indicator.set_current(1)
        self.show()
        self.add(self.list_indicator)

        # Create information labels
        self.movie_title = Label(0.042, "title", 0.2, 0.75, "",
            font_weight="bold")
        self.movie_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.movie_title.width = 0.5
        self.add(self.movie_title)

        self.movie_info = Label(0.034, "subtitle", 0.2, 0.8, "")
        self.movie_info.set_ellipsize(pango.ELLIPSIZE_END)
        self.movie_info.width = 0.5
        self.add(self.movie_info)

        self.movie_plot = Label(0.025, "subtitle", 0.2, 0.85, "")
        self.movie_plot.width = 0.7
        self.add(self.movie_plot)

    def _update_movie_info(self, movie):
        """
        Update movie information. Updates movie specific labels.
        @param movie: Currently selected movie (Movie object)
        """
        genres = movie.get_genres()
        if len(genres) > 1:
            genre = genres[0] + "/" + genres[1]
        else:
            genre = genres[0]

        self.movie_title.set_text(movie.get_title() + \
            " (" + str(movie.get_year()) + ")")
        self.movie_info.set_text(str(movie.get_runtime()) + "min, " + genre)
        self.movie_plot.set_text(movie.get_short_plot())

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_movie_info(self.menu.get_current_menuitem().get_userdata())
        self.list_indicator.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_current_position() <= 6:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        movie = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'movie' : movie }
        self.callback("movie", kwargs)
        return False

