'''SeriesTab - Group of objects that are displayed on tv-series tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
import gtk
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.rounded_texture import RoundedTexture
from entertainerlib.frontend.gui.widgets.texture import Texture

class SeriesTab(Tab):
    """
    Tab can be used as part of the TabGroup

    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """

    def __init__(self, video_library, move_to_new_screen_callback,
        name="series", tab_title=_("TV-Series")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)
        self.video_library = video_library
        self.theme = self.config.theme
        self.list_indicator = None
        self.series_info = None
        self.menu = None
        self.series_title = None

        if self.video_library.get_number_of_tv_series() == 0:
            self._create_empty_library_notice()
        else:
            self._create_menu()

        self.hide_tab()

    def can_activate(self):
        """
        Allow if we have some TV-series indexed.
        """
        if self.video_library.get_number_of_tv_series() == 0:
            return False
        else:
            return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_series_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _create_empty_library_notice(self):
        """
        Create an information box that is displayed if there are no indexed
        tv-series.
        """
        message = _(
            "There are no indexed TV-series in Entertainer media library. To "
            "add TV-series, click on 'content' button on toolbar and open "
            "'videos' tab. Now click on 'add' button and select some folders "
            "which contains video files.")
        Tab.show_empty_tab_notice(self, _("No TV-series available!"), message)

    def _create_menu(self):
        """
        Create a view that is displayed when there is indexed TV-series in
        the video library.
        """
        #Create TV-series menu
        self.menu = ImageMenu(self.theme)
        self.menu.set_item_size(self.get_abs_x(0.1), self.get_abs_y(0.25))
        self.menu.set_item_spacing(self.get_abs_y(0.025))
        self.menu.set_visible_column_count(2)
        self.menu.set_row_count(7)
        self.menu.set_orientation(self.menu.VERTICAL)

        # Default texture
        default_cover = Texture(self.theme.getImage("default_movie_art"))

        all_series = self.video_library.get_tv_series()
        for series in all_series:
            if series.has_cover_art():
                pix_buffer = gtk.gdk.pixbuf_new_from_file(
                    series.get_cover_art_url())
                texture = RoundedTexture(pix_buffer, self.get_abs_x(0.1),
                    self.get_abs_y(0.25))
            else:
                texture = clutter.CloneTexture(default_cover)

            item = ImageMenuItem(0.1, 1.41, texture)
            item.set_userdata(series)
            self.menu.add_actor(item)

        self.menu.set_position(self.get_abs_x(0.11), self.get_abs_y(0.2))
        self.menu.set_active(False)
        self.menu.show()

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.add(self.menu)

        # Create list indicator
        self.list_indicator = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.list_indicator.set_position(self.get_abs_x(0.75),
            self.get_abs_y(0.76))
        self.list_indicator.set_delimiter(" | ")
        self.list_indicator.set_maximum(self.menu.get_number_of_items())
        self.list_indicator.set_current(1)
        self.list_indicator.show()
        self.add(self.list_indicator)

        # Create information labels
        self.series_title = Label(0.042, "title", 0.2, 0.75, "",
            font_weight="bold")
        self.series_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.series_title.width = 0.5
        self.add(self.series_title)

        self.series_info = Label(0.034, "subtitle", 0.2, 0.82, "")
        self.add(self.series_info)

    def _update_series_info(self, series):
        """
        Update TV-Serie information. Updates clutter labels on TV-series tab
        @param series: Currently selected TV-Serie (TVSeries object)
        """
        info = _("%(season)d Seasons\n%(episode)d Episodes") % {'season':
            series.get_number_of_seasons(), 'episode':
            series.get_number_of_episodes()}

        self.series_info.set_text(info)
        self.series_title.set_text(series.get_title())

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_series_info(
            self.menu.get_current_menuitem().get_userdata())
        self.list_indicator.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_current_position() <= 6:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        series = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'tv_series' : series }
        self.callback("tv_series", kwargs)
        return False

