'''Tab - Tab is group of objects that are displayed on one tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.user_event import UserEvent
from entertainerlib.frontend.gui.widgets.base import Base
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.texture import Texture

class Tab(Base, clutter.Group):
    """
    Tab can be used as part of the TabGroup

    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """

    def __init__(self, name, title="Untitled tab", callback=None):

        Base.__init__(self)
        clutter.Group.__init__(self)
        self._name = name
        self._title = title
        self.callback = callback
        self.active = False
        self.theme = self.config.theme

        self.dispatch = {
            UserEvent.NAVIGATE_UP : self._handle_up,
            UserEvent.NAVIGATE_DOWN : self._handle_down,
            UserEvent.NAVIGATE_LEFT : self._handle_left,
            UserEvent.NAVIGATE_RIGHT : self._handle_right,
            UserEvent.NAVIGATE_SELECT : self._handle_select,
        }

        self.timeline = clutter.Timeline(15, 60)
        self.alpha = clutter.Alpha(self.timeline, clutter.smoothstep_inc_func)
        self.behaviour = clutter.BehaviourOpacity(0, 255, self.alpha)
        self.behaviour.apply(self)

    @property
    def name(self):
        '''Internal string name that can be used as a key for the tab group'''
        return self._name

    @property
    def title(self):
        '''User string title displayed in the interface'''
        return self._title

    def can_activate(self):
        """
        Returns a boolean whether or not this tab can be activated. If tab
        doesn't have any controlling widgets it shouldn't activate ever.
        Default value is False. Override this method when implementing a new
        tab and add logic if needed.
        """
        return False

    def is_active(self):
        """
        Is this tab active
        @return: boolean
        """
        return self.active

    def set_active(self, boolean):
        """
        Set this tab to active or inactive state
        @param boolean: True to set active, False to set inactive
        """
        self.active = boolean

    def show_empty_tab_notice(self, title=_("Empty tab"),
            message_body=_("This tab doesn't contain any elements.")):
        """
        Create an information box that is displayed if there is no widgets in
        this tab. This method should be called only from child class as needed.
        """

        # Create warning icon
        info_icon = Texture(self.theme.getImage("warning_icon"), 0.28, 0.27)
        self.add(info_icon)

        # Create warning title
        info_title = Label(0.0625, "title", 0.33, 0.27, title)
        self.add(info_title)

        # Create warning help text
        info = Label(0.042, "menuitem_inactive", 0.28, 0.4, message_body)
        info.width = 0.57
        self.add(info)

    def show_tab(self):
        """
        Activate this tab. This is called when tab is changed and this tab
        is becoming active. This method should activate animations.
        """
        self.set_opacity(0)
        self.show()
        self.timeline.set_direction(clutter.TIMELINE_FORWARD)
        self.timeline.start()

    def hide_tab(self):
        """
        Deactivate this tab. This is called when tab is changed and this tab
        is becoming non-active. This method should disable animations.
        """
        self.hide()

    def _handle_up(self):
        '''Dummy method for unimplemented handlers in subclass'''
        return False

    def _handle_down(self):
        '''Dummy method for unimplemented handlers in subclass'''
        return False

    def _handle_left(self):
        '''Dummy method for unimplemented handlers in subclass'''
        return False

    def _handle_right(self):
        '''Dummy method for unimplemented handlers in subclass'''
        return False

    def _handle_select(self):
        '''Dummy method for unimplemented handlers in subclass'''
        return False

    def handle_user_event(self, event):
        '''Dispatch specific user events to the appropriate method

        @return: Boolean, Should control be moved back to tab bar from tab
        '''
        try:
            handle_method = self.dispatch[event.get_type()]
        except KeyError:
            # Must not be in the dispatch table, keep focus by returning False
            return False

        return handle_method()

