'''Tracks tab for the artist screen'''

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import clutter
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.text_menu import TextMenu
from entertainerlib.frontend.gui.widgets.text_menu_item import TextMenuItem

class TracksTab(Tab):
    '''Tab for the artist screen to show track listings'''

    def __init__(self, tracks, move_to_new_screen_callback, name="tracks",
        tab_title=_("Tracks")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)

        self.menu = TextMenu(self.theme, self.config.show_effects())
        self.menu.set_item_size(self.get_abs_x(0.2928), self.get_abs_y(0.0781))

        cursor = clutter.Rectangle()
        cursor.set_color((0, 0, 0, 0))
        self.menu.set_cursor(cursor)

        for track in tracks:
            item = TextMenuItem(0.2928, 0.104, track.get_title())
            item.justified = True
            item.set_userdata(track)
            self.menu.add_actor(item)

        self.menu.set_orientation(TextMenu.HORIZONTAL)
        self.menu.set_row_count(7)
        self.menu.set_visible_column_count(3)
        self.menu.set_position(self.get_abs_x(0.0586), self.get_abs_y(0.2083))
        self.menu.set_active(False)
        self.menu.show()

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.add(self.menu)

        self.track_title = Label(0.045, "title", 0.22, 0.79, "")
        self.track_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_title.width = 0.366
        self.add(self.track_title)

        self.track_number = Label(0.037, "subtitle", 0.22, 0.86, "")
        self.track_number.set_ellipsize(pango.ELLIPSIZE_END)
        self.track_number.width = 0.366
        self.add(self.track_number)

        self.track_length = Label(0.037, "subtitle", 0.22, 0.91, "")
        self.add(self.track_length)

        self.li = ListIndicator(0.18, 0.045, ListIndicator.HORIZONTAL)
        self.li.set_position(self.get_abs_x(0.9297) - self.li.get_width(),
            self.get_abs_y(0.7943))
        self.li.set_delimiter(" | ")
        self.li.set_maximum(self.menu.get_number_of_items())
        self.li.set_current(1)
        self.li.show()
        self.add(self.li)

    def can_activate(self):
        '''Tracks tab will always be created from an existing artist with at
        least one track'''
        return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_track_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _update_track_info(self, track):
        '''Update the track information labels'''
        self.track_title.set_text(track.get_title())
        self.track_length.set_text(str(track.get_length() / 60) + ":" + \
            str(track.get_length() % 60).zfill(2))
        self.track_number.set_text(_("Track %(track)s from %(album)s") % \
            {'track': track.get_tracknumber(),
             'album': track.get_album().get_title()})

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_track_info(self.menu.get_current_menuitem().get_userdata())
        self.li.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_cursor_position()[1] == 0:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        track = self.menu.get_current_menuitem().get_userdata()
        kwargs = { 'track' : track }
        self.callback("audio_play", kwargs)
        return False

