'''VideoClipsTab - Group of objects that are displayed on video clips tab'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os

import gtk
import pango

from entertainerlib.frontend.gui.tabs.tab import Tab
from entertainerlib.frontend.gui.widgets.image_menu import ImageMenu
from entertainerlib.frontend.gui.widgets.image_menu_item import ImageMenuItem
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator
from entertainerlib.frontend.gui.widgets.rounded_texture import RoundedTexture

class VideoClipsTab(Tab):
    """
    Tab can be used as part of the TabGroup

    Tab is a very simple container that contains all the widgets and logic
    of the tab page.
    """

    def __init__(self, media_player, video_library, move_to_new_screen_callback,
        name="clips", tab_title=_("Video clips")):
        Tab.__init__(self, name, tab_title, move_to_new_screen_callback)
        self.media_player = media_player
        self.video_library = video_library
        self.theme = self.config.theme
        self.list_indicator = None
        self.clip_info = None
        self.menu = None
        self.clip_title = None

        if self.video_library.get_number_of_video_clips() == 0:
            self._create_empty_library_notice()
        else:
            self._create_menu()

        self.hide_tab()

    def can_activate(self):
        """
        Allow if we have some movies indexed.
        """
        if self.video_library.get_number_of_video_clips() == 0:
            return False
        else:
            return True

    def set_active(self, boolean):
        """
        Override method from parent class.
        @param boolean: True to set this tab active, False to set inactive
        """
        Tab.set_active(self, boolean)
        if boolean:
            self._update_clip_info(
                self.menu.get_current_menuitem().get_userdata())
            self.menu.set_active(True)
        else:
            self.menu.set_active(False)

    def _create_empty_library_notice(self):
        """
        Create an information box that is displayed if there are no indexed
        movies.
        """
        message = _(
            "There are no indexed Video Clips in the Entertainer media "
            "library. Please add some folders containing video clips "
            "to the Library using the configuration tool.")
        Tab.show_empty_tab_notice(self, _("No video clips available!"), message)

    def _create_menu(self):
        """
        Create a view that is displayed when there are indexed clips in
        the video library.
        """
        self.menu = ImageMenu(self.theme)
        self.menu.set_item_size(self.get_abs_x(0.233), self.get_abs_y(0.25))
        self.menu.set_item_spacing(self.get_abs_y(0.025))
        self.menu.set_visible_column_count(2)
        self.menu.set_row_count(3)
        self.menu.set_orientation(self.menu.VERTICAL)

        clips = self.video_library.get_video_clips()
        for clip in clips:
            pix_buffer = gtk.gdk.pixbuf_new_from_file(clip.get_thumbnail_url())
            texture = RoundedTexture(pix_buffer, self.get_abs_x(0.233),
                self.get_abs_y(0.25))
            item = ImageMenuItem(0.233, 0.5625, texture)
            item.set_userdata(clip)
            self.menu.add_actor(item)

        self.menu.set_position(self.get_abs_x(0.15), self.get_abs_y(0.2))
        self.menu.set_active(False)

        if self.config.show_effects():
            self.menu.set_animate(True)

        self.add(self.menu)

        # Create list indicator
        self.list_indicator = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.list_indicator.set_position(self.get_abs_x(0.67),
            self.get_abs_y(0.76))
        self.list_indicator.set_delimiter(" | ")
        self.list_indicator.set_maximum(self.menu.get_number_of_items())
        self.list_indicator.set_current(1)
        self.list_indicator.show()
        self.add(self.list_indicator)

        # Create information labels
        self.clip_title = Label(0.042, "title", 0.15, 0.77, "",
            font_weight="bold")
        self.clip_title.set_ellipsize(pango.ELLIPSIZE_END)
        self.clip_title.width = 0.5
        self.add(self.clip_title)

        self.clip_info = Label(0.034, "subtitle", 0.15, 0.85, "")
        self.clip_info.set_ellipsize(pango.ELLIPSIZE_END)
        self.clip_info.width = 0.5
        self.add(self.clip_info)

    def _update_clip_info(self, clip):
        """
        Update VideoClip information. Updates clutter labels on Clips tab
        @param clip: Currently selected VideoClip (VideoClip object)
        """
        (folder, filename) = os.path.split(clip.get_filename())
        self.clip_title.set_text(filename)
        self.clip_info.set_text(folder)

    def _update_menu(self, direction):
        '''Update the menu based on the provided menu direction'''
        self.menu.move(direction)
        self._update_clip_info(self.menu.get_current_menuitem().get_userdata())
        self.list_indicator.set_current(self.menu.get_current_position() + 1)
        return False

    def _handle_up(self):
        '''Handle the up user event'''
        if self.menu.get_current_position() <= 2:
            return True # Move control back to tab bar
        else:
            return self._update_menu(self.menu.UP)

    def _handle_down(self):
        '''Handle the down user event'''
        return self._update_menu(self.menu.DOWN)

    def _handle_left(self):
        '''Handle the left user event'''
        return self._update_menu(self.menu.LEFT)

    def _handle_right(self):
        '''Handle the right user event'''
        return self._update_menu(self.menu.RIGHT)

    def _handle_select(self):
        '''Handle the select user event'''
        clip = self.menu.get_current_menuitem().get_userdata()
        self.media_player.set_media(clip)
        self.media_player.play()
        self.callback("video_osd")
        return False

