'''Slide - Transition to slide out old screen and slide in the new screen'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.screens.screen import Screen
from entertainerlib.frontend.gui.transitions.transition import Transition
from entertainerlib.utils.configuration import Configuration

class Slide(Transition):
    """
    Slide

    Slide all components from the "from_screen" away from the screen and slide 
    in all the components of the "to_screen". This class implements simple slide
    effect for screen changes.
    """

    def __init__(self, remove_from_stage_callback):
        Transition.__init__(self, remove_from_stage_callback)
        self.config = Configuration()
        self.out_behaviour = None
        self.out_behaviour2 = None
        self.in_behaviour = None
        self.in_behaviour2 = None

    def forward_effect(self, from_screen, to_screen):
        """
        Slide out all components from the "from_screen" and slide in all
        components of the "to_screen".
        """
        if to_screen.get_type() == Screen.DIALOG:
            self.forward_effect_for_dialog(from_screen, to_screen)
        else:
            width = self.config.get_stage_width()
            # Initialize to_screen for animation
            to_screen.set_position(-width, 0)
            to_screen.show()

            # Slide out timeline
            if from_screen is not None:
                slide_out = clutter.Timeline(20, 60)
                alpha_out = clutter.Alpha(slide_out,
                                            clutter.smoothstep_inc_func)
                self.out_behaviour = clutter.BehaviourPath(alpha_out,
                                            ((0, 0), (width, 0)))
                self.out_behaviour.apply(from_screen)

            # Slide in timeline
            slide_in = clutter.Timeline(20, 60)
            alpha_in = clutter.Alpha(slide_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourPath(alpha_in,
                                            ((-width, 0), (0, 0)))
            self.in_behaviour.apply(to_screen)

            # Start transition animating
            if from_screen is not None:
                slide_out.start()
            slide_in.start()

    def backward_effect(self, from_screen, to_screen):
        """
        Slide out all components from the "from_screen" and slide in all
        components of the "to_screen".
        """
        if from_screen.get_type() == Screen.DIALOG:
            self.backward_effect_for_dialog(from_screen, to_screen)
        else:
            width = self.config.get_stage_width()
            # Initialize to_screen for animation
            to_screen.set_position(width, 0)
            to_screen.show()

            # Slide out timeline
            slide_out = clutter.Timeline(20, 60)
            alpha_out = clutter.Alpha(slide_out, clutter.smoothstep_inc_func)
            self.out_behaviour = clutter.BehaviourPath(alpha_out,
                                            ((0, 0), (-width, 0)))
            self.out_behaviour.apply(from_screen)
            slide_out.connect('completed', self._remove_from_stage_callback,
                                            from_screen)

            # Slide in timeline
            slide_in = clutter.Timeline(20, 60)
            alpha_in = clutter.Alpha(slide_in, clutter.smoothstep_inc_func)
            self.in_behaviour = clutter.BehaviourPath(alpha_in,
                                            ((width, 0), (0, 0)))
            self.in_behaviour.apply(to_screen)

            # Start transition animating
            slide_out.start()
            slide_in.start()

    def forward_effect_for_dialog(self, from_screen, to_screen):
        """This is called when next screen is dialog."""
        #Fade current screen away
        slide_out = clutter.Timeline(20, 60)
        alpha_out = clutter.Alpha(slide_out, clutter.smoothstep_inc_func)

        self.out_behaviour = clutter.BehaviourOpacity(alpha_out, 254, 64)
        self.out_behaviour.apply(from_screen)

        self.out_behaviour2 = clutter.BehaviourScale(alpha_out, 1, 0.9,
                                            clutter.GRAVITY_CENTER)
        self.out_behaviour2.apply(from_screen)

        #Fade dialog in
        slide_in = clutter.Timeline(20, 60)
        alpha_in = clutter.Alpha(slide_in, clutter.smoothstep_inc_func)

        to_screen.set_opacity(0)
        self.in_behaviour = clutter.BehaviourOpacity(alpha_in, 0, 255)
        self.in_behaviour.apply(to_screen)

        self.in_behaviour2 = clutter.BehaviourDepth(alpha_in, 100, 0)
        self.in_behaviour2.apply(to_screen)

        # Start animations
        slide_out.start()
        slide_in.start()

    def backward_effect_for_dialog(self, from_screen, to_screen):
        """This is used when dialog is closed and control returns to screen."""
        #Fade current screen away
        slide_out = clutter.Timeline(20, 60)
        slide_out.connect('completed', self._remove_from_stage_callback,
                                            from_screen)
        alpha_out = clutter.Alpha(slide_out, clutter.smoothstep_inc_func)

        self.out_behaviour = clutter.BehaviourOpacity(alpha_out, 254, 0)
        self.out_behaviour.apply(from_screen)

        self.out_behaviour2 = clutter.BehaviourDepth(alpha_out, 0, 100)
        self.out_behaviour2.apply(from_screen)

        #Fade dialog in
        slide_in = clutter.Timeline(20, 60)
        alpha_in = clutter.Alpha(slide_in, clutter.smoothstep_inc_func)

        self.in_behaviour = clutter.BehaviourOpacity(alpha_in, 64, 255)
        self.in_behaviour.apply(to_screen)

        self.in_behaviour2 = clutter.BehaviourScale(alpha_in, 0.9, 1,
                                            clutter.GRAVITY_CENTER)
        self.in_behaviour2.apply(to_screen)

        # Start animations
        slide_out.start()
        slide_in.start()

