'''Displays text in rounded rectangle box'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
from cluttercairo import CairoTexture

class ArrowTexture(CairoTexture):
    """Clutter CairoTexture which represents an arrow."""

    # Arrow direction (One of these should be passed as a paramter)
    UP = 0
    DOWN = 1
    RIGHT = 2
    LEFT = 3
    ARC_TO_BEZIER = 0.55228475

    def __init__(self, size, fg_color, bg_color, direction):
        """
        Initialize ArrowTexture
        @param size: Width and height of the texture
        @param fg_color: Foreground color (arrows)
        @param bg_color: Background color (background)
        @param direction: Arrow direction UP,DOWN,RIGHT or LEFT (class
            constant)
        """
        CairoTexture.__init__(self, size, size)
        context = self.cairo_create()
        context.scale(size, size)
        fg_cairo = self._color_to_cairo_color(fg_color)
        bg_cairo = self._color_to_cairo_color(bg_color)

        # Draw background
        context.set_line_width (0.15)
        context.set_source_rgba(bg_cairo[0], bg_cairo[1],
                                bg_cairo[2], bg_cairo[3])
        self._roundedrec(context, 0, 0, 1, 1, 0.08, 0.1)
        context.fill()

        # Draw arrow
        context.set_source_rgba(fg_cairo[0], fg_cairo[1],
                                fg_cairo[2], fg_cairo[3])
        if direction == ArrowTexture.DOWN:
            context.move_to(0.25, 0.33)
            context.line_to(0.50, 0.66)
            context.line_to(0.75, 0.33)
            context.stroke()
        elif direction == ArrowTexture.UP:
            context.move_to(0.25, 0.66)
            context.line_to(0.50, 0.33)
            context.line_to(0.75, 0.66)
            context.stroke()
        elif direction == ArrowTexture.RIGHT:
            context.move_to(0.33, 0.25)
            context.line_to(0.66, 0.50)
            context.line_to(0.33, 0.75)
            context.stroke()
        elif direction == ArrowTexture.LEFT:
            context.move_to(0.66, 0.25)
            context.line_to(0.33, 0.50)
            context.line_to(0.66, 0.75)
            context.stroke()

        del(context) # Updates texture

        # Create bounce effect
        self.timeline = clutter.Timeline(30)
        self.alpha = clutter.Alpha(self.timeline, clutter.sine_func)
        self.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
        self.behaviour = clutter.BehaviourScale(1.0, 1.0, 1.5, 1.5, self.alpha)
        self.behaviour.apply(self)

    def bounce(self):
        """
        Execute bounce effect for this array.
        """
        if not self.timeline.is_playing():
            self.timeline.start()

    def _color_to_cairo_color(self, color):
        """
        Transform from 0-255 to 0-1.
        @param color: (r, g, b, a) integer tuple
        """
        (int_r, int_g, int_b, int_a) = color
        r = float(int_r) / 255.0
        g = float(int_g) / 255.0
        b = float(int_b) / 255.0
        a = float(int_a) / 255.0
        return r, g, b, a

    def _roundedrec(self, cr, x_pos, y_pos, width, height, radius_x=5,
        radius_y=5):
        """
        from mono moonlight aka mono silverlight
        test limits (without using multiplications)
        http://graphics.stanford.edu/courses/cs248-98-fall/Final/q1.html
        """

        if radius_x > width - radius_x:
            radius_x = width / 2
        if radius_y > height - radius_y:
            radius_y = height / 2

        #approximate (quite close) the arc using a bezier curve
        c1 = ArrowTexture.ARC_TO_BEZIER * radius_x
        c2 = ArrowTexture.ARC_TO_BEZIER * radius_y

        cr.new_path()
        cr.move_to(x_pos + radius_x, y_pos)
        cr.rel_line_to(width - 2 * radius_x, 0.0)
        cr.rel_curve_to(c1, 0.0, radius_x, c2, radius_x, radius_y)
        cr.rel_line_to(0, height - 2 * radius_y)
        cr.rel_curve_to(0.0, c2, c1 - radius_x, radius_y, -radius_x, radius_y)
        cr.rel_line_to(-width + 2 * radius_x, 0)
        cr.rel_curve_to(-c1, 0, -radius_x, -c2, -radius_x, -radius_y)
        cr.rel_line_to(0, -height + 2 * radius_y)
        cr.rel_curve_to(0.0, -c2, radius_x - c1, -radius_y, radius_x, -radius_y)
        cr.close_path()
