"""Label for widget"""

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterm.dev@gmail.com>"

import clutter

from entertainerlib.frontend.gui.widgets.base import Base

class Label(Base, clutter.Label):
    """Wrapper of a clutter label to encapsulate some label settings and have
    knowledge of the stage size"""

    def __init__(self, font_size, color_name, x_pos_percent, y_pos_percent,
        text=None, name=None, font_weight=""):
        """Initialize the Label object"""

        Base.__init__(self)
        clutter.Label.__init__(self)

        self.theme = self.config.theme

        self._font_weight = font_weight
        self._font_size = font_size
        self._set_font_size(font_size)

        self.set_color(self.theme.get_color(color_name))

        self._position = None
        self._set_position((x_pos_percent, y_pos_percent))

        if text:
            self.set_text(text)
        if name:
            self.set_name(name)

        self._width = self._init_width()
        self._height = self._init_height()

        self.set_line_wrap(True)

    def set_size(self, x_percent, y_percent):
        """Override clutter label set_size to calculate absolute size from a
        percentage"""
        self._width = x_percent
        self._height = y_percent
        clutter.Label.set_size(self,
            self.get_abs_x(x_percent),
            self.get_abs_y(y_percent))

    def _init_width(self):
        """Generate the width of the label initially before the width property
        is set to ensure width has a value"""
        return float(self.get_width()) / self.config.get_stage_width()

    def _get_width(self):
        """Return the width as a percent of the stage size"""
        return self._width

    def _set_width(self, x_percent):
        """Set the width based on a hozirontal percent of the stage size.
        Provide clutter label set_width the absolute size from the percentage"""
        self._width = x_percent
        clutter.Label.set_width(self, self.get_abs_x(x_percent))

    width = property(_get_width, _set_width)

    def _get_position(self):
        """Return the position as a tuple of percents of the stage size"""
        return self._position

    def _set_position(self, position_pair):
        """Translate a tuple of x and y percentages into an absolute position
        within the display area"""
        self._position = position_pair

        (x_pos_percent, y_pos_percent) = position_pair

        clutter.Label.set_position(self,
            self.get_abs_x_pos(x_pos_percent),
            self.get_abs_y_pos(y_pos_percent))

    position = property(_get_position, _set_position)

    def _init_height(self):
        """Generate the height of the label initially before the height property
        is set to ensure height has a value"""
        return float(self.get_height()) / self.config.get_stage_height()

    def _get_height(self):
        """Return the height as a percent of the stage size"""
        return self._height

    def _set_height(self, y_percent):
        """Set the height based on a vertical percent of the stage size.
        Provide clutter label set_height the absolute size from the percentage
        """
        self._height = y_percent
        clutter.Label.set_height(self, self.get_abs_y(y_percent))

    height = property(_get_height, _set_height)

    def _get_font_size(self):
        """Return the font size as a percent of the stage size"""
        return self._font_size

    def _set_font_size(self, y_percent):
        """Set the font size based on a vertical percent of the screen size"""
        self._font_size = y_percent
        self.set_font_name(self.theme.font + ", " + self._font_weight +
            " " + str(self.get_abs_y(y_percent)) + "px")

    font_size = property(_get_font_size, _set_font_size)

