'''Displays currently selected item and length of the list'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.widgets.arrow_texture import ArrowTexture
from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.utils.configuration import Configuration

class ListIndicator(clutter.Group):
    """
    ListIndicator displays 'current / maximum' value label and arrows.
    """

    # Direction
    # HORIZONTAL layout: ARROW_LEFT current / maximum ARROW_RIGHT
    # VERITCAL layout:   current / maximum ARROW_UP ARROW_DOWN
    HORIZONTAL = 0
    VERTICAL = 1

    def __init__(self, width, height, direction):
        """
        Initialize actor.
        @param width: Width of the widget in percentage (0-1)
        @param height: Height of the widget in percentage (0-1)
        @param direction: Direction (constant of this class)
        """
        clutter.Group.__init__(self)
        self.config = Configuration()

        # Size
        self.width = int(self.config.get_stage_width() * width)
        self.height = int(self.config.get_stage_height() * height)

        self.direction = direction
        self.delimiter = " / "
        self.current = 1
        self.maximum = 1

        self.theme = self.config.theme
        self.fg = self.theme.get_color("arrow_foreground")
        self.bg = self.theme.get_color("arrow_background")

        if direction == ListIndicator.VERTICAL:
            text_x_pos = width / 3
        else:
            text_x_pos = width / 2

        self.text = Label(height * 0.8, "text", text_x_pos, (height / 2),
            str(self.maximum) + self.delimiter + str(self.maximum))
        self.text.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
        self.add(self.text)

        # Create arrows and calculate positions on screen
        if direction == ListIndicator.VERTICAL:
            self.arrow1 = ArrowTexture(
                self.height, self.fg, self.bg, ArrowTexture.UP)
            self.arrow2 = ArrowTexture(
                self.height, self.fg, self.bg, ArrowTexture.DOWN)
            self.arrow1.set_position(
                int(self.width - self.height / 2.0), int(self.height / 2.0))
            self.arrow2.set_position(
                int(self.width - (self.height * 1.5 + self.height / 6.0)),
                int(self.height / 2.0))

        elif direction == ListIndicator.HORIZONTAL:
            self.arrow1 = ArrowTexture(
                self.height, self.fg, self.bg, ArrowTexture.LEFT)
            self.arrow2 = ArrowTexture(
                self.height, self.fg, self.bg, ArrowTexture.RIGHT)
            self.arrow1.set_position(
                int(self.height / 2.0),
                int(self.height / 2.0))
            self.arrow2.set_position(
                int(self.width - self.height / 2.0), int(self.height / 2.0))

        self.add(self.arrow1)
        self.add(self.arrow2)

    def set_current(self, value):
        """
        Set current value
        @param number: Current value (between 1 - maximum)
        """
        if value > 0 and value <= self.maximum:
            self.current = value
            self._update_text()

            # Bounce arrow if user has reached the limit
            if self.current == 1:
                self.arrow1.bounce()
            elif self.current == self.maximum:
                self.arrow2.bounce()

    def get_current(self):
        """
        Get current value
        @return Current value (integer)
        """
        return self.current

    def set_maximum(self, value):
        """
        Set maximum value of the indicator
        @param number: Set maximum value
        """
        self.maximum = value
        self._update_text()

    def get_maximum(self):
        """
        Get maximum value.
        @return Max value (integer)
        """
        return self.maximum

    def set_delimiter(self, delimiter):
        """
        Set delimiter text that is displayed between current and maximum value.
        Default delimiter text is ' / ' spaces included.
        @param delimiter: delimiter text
        """
        self.delimiter = delimiter
        self._update_text()

    def show_position(self):
        """
        Show position text that indicates the current position of the content.
        """
        self.text.show()

    def hide_position(self):
        """
        Hide position text that indicates the current position of the content.
        If this is called then indicator shows only arrows.
        """
        self.text.hide()

    def _update_text(self):
        """Update the text label"""

        self.text.set_text(
            str(self.current) + self.delimiter + str(self.maximum))

