'''Animation widget indicates that Entertainer is loading'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import math
import cairo
import clutter
import gobject
from cluttercairo import CairoTexture

class LoadingAnimation(CairoTexture):
    """
    Loading animation widget

    This widget is used when we want to tell user that Entertainer is loading.
    """

    def __init__(self, size, fg_color, bg_color, inner_radius = 0.25,
        outter_radius = 0.45, thickness = 0.08):
        """
        Initialize LoadingAnimation
        @param size: Width and height of the texture
        @param fg_color: Foreground color
        @param bg_color: Background color
        @param inner_radius: Distance from origo to line start point 0-1
        @param outter_radius: Distance from origo to line end point 0-1
        @param thinkness: Lines thickness 0-1
        """
        CairoTexture.__init__(self, size, size)
        self.set_anchor_point(size /2, size /2)

        c = self._clutterColorToCairoColor(fg_color)
        bg = self._clutterColorToCairoColor(bg_color)

        context = self.cairo_create()
        context.scale(size, size)
        context.set_line_width (thickness)
        context.set_line_cap(cairo.LINE_CAP_ROUND)

        # Draw the 12 lines
        for i in range(12):
            self._draw_line(context, bg, c, outter_radius, inner_radius, i)

        del(context) # Updates texture

        self.keep_going = False

    def _draw_line(self, context, bg_col, fg_col, radius, in_radius, number):
        """Draw one line"""
        if number < 2:
            context.set_source_rgba(bg_col[0], bg_col[1], bg_col[2], bg_col[3])
        else:
            context.set_source_rgba(fg_col[0], fg_col[1], fg_col[2], fg_col[3] \
                - (1 -(number - 1) * 0.1))

        # radius = radius of larger circle
        # in_radius = radius of smaller circle
        angle =  number * 30 * (math.pi / 180.0)
        context.arc(0.5, 0.5, in_radius, angle, angle)
        (x, y) = context.get_current_point()
        context.new_path()
        context.arc(0.5, 0.5, radius, angle, angle)
        context.line_to(x, y)
        context.stroke()

    def _clutterColorToCairoColor(self, color):
        """
        Transform from 0-255 to 0-1.
        @param color: clutter.Color
        """
        r = float(color.red) / 255.0
        g = float(color.green) / 255.0
        b = float(color.blue) / 255.0
        a = float(color.alpha) / 255.0
        return [r, g, b, a]

    def _rotate_throbber(self):
        """
        Update texture.
        """
        r = self.get_rotation(clutter.Z_AXIS)
        self.set_rotation(clutter.Z_AXIS, 30 + int(r[0]), 0, 0, 0)
        return self.keep_going

    def hide(self):
        """
        Hide throbber smoothly and stop animation.
        """
        timeline = clutter.Timeline(35, 26)
        alpha = clutter.Alpha(timeline, clutter.smoothstep_inc_func)
        self.behaviour = clutter.BehaviourOpacity(255, 0, alpha)
        self.behaviour.apply(self)
        timeline.start()

        gobject.timeout_add(5, self._stop_rotating, timeline)

    def _stop_rotating(self, timeline):
        '''Rotate throbber will be stopped when the hide timeline finishes'''
        if timeline.is_playing():
            return True
        else:
            self.keep_going = False
            CairoTexture.hide(self)
            return False

    def show(self):
        """
        Show throbber and activate animation
        """
        self.keep_going = True
        gobject.timeout_add(50, self._rotate_throbber)
        CairoTexture.show(self)

