'''Cairo based texture that rounds corners'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

from cluttercairo import CairoTexture
import gtk

class RoundedTexture(CairoTexture):
    """
    Texture with rounded corners

    This is a clutter.Texture that uses Cairo to draw image textures with
    rounded smooth corners.
    """

    def __init__(self, pixbuf, width, height, margin=1, radius=15):
        """
        Create a new cairo based texture from GDK pixel buffer. This texture is
        similar to clutter.Texture, but this is rendered with cairo and it adds
        rounded corners to pixmap texture.

        @param pixbuf: gtk.gdk.Pixbuf object. Graphics for texture
        @param margin: number of pixels to cut off from sides
        @param radius: Radius for rounded corners
        """
        CairoTexture.__init__(self, width, height)
        context = self.cairo_create()
        ct = gtk.gdk.CairoContext(context)

        # Round corners
        x = 0 + margin
        y = 0 + margin
        w1 = width - (margin * 2)
        h1 = height - (margin * 2)
        self.roundedCorners(context, x, y, w1, h1, radius, radius)

        # Scale context area
        wr = width / float(pixbuf.get_width())
        hr = height / float(pixbuf.get_height())
        context.scale(wr, hr)
        ct.set_source_pixbuf(pixbuf, 0, 0)
        context.paint()
        del context # Update texture
        del ct

    def roundedCorners(self, context, x, y, w, h, radius_x, radius_y):
        """
        Clip cairo texture with rounded rectangle.
        """
        ARC_TO_BEZIER = 0.55228475
        if radius_x > w - radius_x:
            radius_x = w / 2
        if radius_y > h - radius_y:
            radius_y = h / 2

        #approximate (quite close) the arc using a bezier curve
        c1 = ARC_TO_BEZIER * radius_x
        c2 = ARC_TO_BEZIER * radius_y

        context.new_path()
        context.move_to ( x + radius_x, y)
        context.rel_line_to ( w - 2 * radius_x, 0.0)
        context.rel_curve_to ( c1, 0.0, radius_x, c2, radius_x, radius_y)
        context.rel_line_to ( 0, h - 2 * radius_y)
        context.rel_curve_to ( 0.0, c2, c1 - radius_x, radius_y, -radius_x,
            radius_y)
        context.rel_line_to ( -w + 2 * radius_x, 0)
        context.rel_curve_to ( -c1, 0, -radius_x, -c2, -radius_x, -radius_y)
        context.rel_line_to (0, -h + 2 * radius_y)
        context.rel_curve_to (0.0, -c2, radius_x - c1, -radius_y, radius_x,
            -radius_y)
        context.close_path()
        context.clip()

