'''Container that has scrollbars and allows content to be scrolled'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

from entertainerlib.frontend.gui.widgets.base import Base
from entertainerlib.frontend.gui.widgets.list_indicator import ListIndicator

class ScrollArea(Base, clutter.Group):
    """Wrapper of a clutter Group that allows for scrolling. ScrollArea
    modifies the width of the content and it assumes that the content uses
    percent modification (read: not default clutter objects)."""

    def __init__(self, x_size_percent, y_size_percent, x_pos_percent,
        y_pos_percent, group, step_size_percent=0.04, active=False):
        """Initialize the ScrollArea object."""

        Base.__init__(self)
        clutter.Group.__init__(self)

        self.offset = 0 # Determines current position of the view
        self.step_size = self.get_abs_y(step_size_percent)
        self.area_width = self.get_abs_x(x_size_percent)
        self.area_height = self.get_abs_y(y_size_percent)
        self.active = active
        self.content_group = None

        self.set_position(
            self.get_abs_x_pos(x_pos_percent),
            self.get_abs_y_pos(y_pos_percent))

        # Create content position indicator
        self.indicator = ListIndicator(0.2, 0.045, ListIndicator.VERTICAL)
        self.indicator.hide_position()
        self.indicator.set_current(1)
        self.indicator.set_position(
            self.area_width - self.indicator.get_width(),
            self.area_height + self.get_abs_y(0.01))
        self.add(self.indicator)

        if active:
            self.set_active(active)

        self.set_content(group)

    def set_active(self, boolean):
        """
        Set this widget active / inactive. When inactive indicator is hidden.
        """
        self.active = boolean
        if boolean:
            self.indicator.show()
        else:
            self.indicator.hide()

    def is_active(self):
        """
        Is this widget active.
        """
        return self.active

    def set_content(self, group):
        """
        Set content into scroll area.
        @param group: Group of clutter Actors (clutter.Group)
        """
        if self.content_group is not None:
            self.remove(self.content_group)

        self.content_group = group
        self.add(group)
        self.content_group.set_clip(0, 0, self.area_width, self.area_height)
        tmp = (self.content_group.get_height() -
            self.area_height) / self.step_size
        self.indicator.set_maximum(tmp + 2)

        # Do not show indicator if there is no need for scrolling
        if self.content_group.get_height() < self.area_height:
            self.indicator.hide()

    def set_step_size(self, size_percent):
        """Step size determines how much content is scrolled once."""

        self.step_size = self.get_abs_y(size_percent)
        if self.content_group is not None:
            tmp = (self.content_group.get_height() -
                self.area_height) / self.step_size
            self.indicator.set_maximum(tmp + 2)

    def get_step_size(self):
        """
        Get step size
        @requires: Step size (Integer)
        """
        return self.step_size

    def get_offset(self):
        """
        Get current offset value. This value determines how many pixels current
        view is from the absolute top.
        @return: Integer
        """
        return self.offset

    def scroll_to_top(self):
        """
        Scroll content back to top. This is intial state of the scroll area.
        """
        self.indicator.set_current(1)
        self.offset = 0
        self.content_group.set_clip(0, 0, self.area_width, self.area_height)
        self.content_group.set_position(0, 0)

    def scroll_to_bottom(self):
        """
        Scroll content as much as possible.
        """
        self.indicator.set_current(self.indicator.get_maximum())
        self.offset = self.content_group.get_height() - self.area_height
        self.content_group.set_clip(
            0, self.content_group.get_height() - self.area_height,
            self.area_width, self.area_height)
        self.content_group.set_position(0, - (
            self.content_group.get_height() - self.area_height))

    def scroll_up(self):
        """Scroll up by one step size."""
        if self.offset - self.step_size >= 0:
            self.indicator.set_current(self.indicator.get_current() - 1)
            self.offset = self.offset - self.step_size
            self.content_group.set_clip(
                0, self.offset, self.area_width, self.area_height)
            self.content_group.move_by(0, self.step_size)

        #If there is a smaller gap than step size between current position
        #and the top
        elif self.offset > 0:
            self.indicator.set_current(1)
            self.offset = 0
            self.content_group.set_clip(
                0, 0, self.area_width, self.area_height)
            self.content_group.set_position(0, 0)

    def scroll_down(self):
        """Scroll down by one step size."""
        if (self.offset + self.step_size <=
            self.content_group.get_height() - self.area_height):
            self.indicator.set_current(self.indicator.get_current() + 1)
            self.offset = self.offset + self.step_size
            self.content_group.set_clip(
                0, self.offset, self.area_width, self.area_height)
            self.content_group.move_by(0, -self.step_size)

        #If there is a smaller gap than step size between current position
        #and the bottom
        elif (self.offset <
            self.content_group.get_height() - self.area_height):
            difference = (self.content_group.get_height() -
                self.area_height) - self.offset
            self.indicator.set_current(self.indicator.get_maximum())
            self.offset = (self.content_group.get_height() -
                self.area_height)
            self.content_group.set_clip(0, self.offset, self.area_width,
                self.area_height)
            self.content_group.move_by(0, -difference)

    def scroll_page_up(self):
        """Scroll up by one page. Page is a scroll area height."""
        if self.offset - self.area_height >= 0:
            self.indicator.set_current(self.indicator.get_current() - 1)
            self.offset = self.offset - self.area_height
            self.content_group.set_clip(0, self.offset, self.area_width,
                self.area_height)
            self.content_group.move_by(0, self.area_height)

        #If there is a smaller gap than step size between current position
        #and the top
        elif self.offset > 0:
            self.indicator.set_current(1)
            self.offset = 0
            self.content_group.set_clip(0, 0, self.area_width,
                self.area_height)
            self.content_group.set_position(0, 0)

    def scroll_page_down(self):
        """Scroll down by one page. Page is a scroll area height."""
        if (self.offset + self.area_height <=
            self.content_group.get_height() - self.area_height):
            self.indicator.set_current(self.indicator.get_current() + 1)
            self.offset = self.offset + self.area_height
            self.content_group.set_clip(0, self.offset, self.area_width,
                self.area_height)
            self.content_group.move_by(0, -self.area_height)

        #If there is a smaller gap than step size between current position
        #and the bottom
        elif (self.offset <
            self.content_group.get_height() - self.area_height):
            difference = (self.content_group.get_height() -
                self.area_height) - self.offset
            self.indicator.set_current(self.indicator.get_maximum())
            self.offset = (self.content_group.get_height() -
                self.area_height)
            self.content_group.set_clip(0, self.offset, self.area_width,
                self.area_height)
            self.content_group.move_by(0, -difference)

