'''Menu widget that contains text items'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter

class ScrollMenu(clutter.Group):
    """
    Menu widget that contains text items. Menu can be scrolled up and down.

    - Options: zoom, opacity, display N items, display arrows top+down
    - add Clutter labels
    - go around (how to implement this) (On state_change move items from
        start->back)
    """

    DOWN = 0
    UP = 1

    def __init__(self, item_gap, item_height, animated=False):
        """
        Initialize Menu
        @param item_gap: Gap between menuitems in pixels
        """
        clutter.Group.__init__(self)
        self.__active = True
        self.__clip_size = -1
        self.__items = []
        self.__gap = item_gap
        self.__item_height = item_height
        self.__animated = animated

        self.scroll_timeline = clutter.Timeline(7, 26)
        self.scroll_timeline.connect('new-frame', self.scroll_items_cb)

    def add(self, actor):
        """
        Add new menuitem to the menu.
        @param item: Label
        """
        clutter.Group.add(self, actor)
        actor.connect('notify::y', self.actor_notify_y)

        y = ((len(self.__items) * self.__item_height) +
            (self.__gap * len(self.__items)))

        if y == 0 :
            # need to do that to update opacity of the first item added
            self.actor_notify_y(actor, None)
        actor.set_position(0, y)

        self.__items.append(actor)

    def remove(self, actor):
        """
        Remove actor from the menu.
        """
        clutter.Group.add(self, actor)
        for item in self.__items:
            if item is actor:
                del item

    def remove_all(self):
        """
        Remove all actors from the menu.
        """
        del self.__items
        self.__items = []

    def get_index(self, text):
        """
        Returns index of label with the text as passed, returns -1 if not found
        @author Joshua Scotton
        """
        for item in self.__items:
            if item.get_name() == text:
                return self.__items.index(item)
        return -1

    def select(self, index):
        """
        Selects the item at the passed index, does not support animations
        @author Joshua Scotton
        """
        #split list so that item at the index will now be at position 2
        #which is the selected index in a scroll menu
        items = self.__items[index-2:] + self.__items[0:index-2]

        #Basically remove all then add in the correct order
        self.remove_all()
        for item in items:
            #Remove all doesn't remove the parent object. So in order to avoid
            #adding two parent clutter items to the item we need to remove the
            #old one first
            clutter.Group.remove(self, item)
            self.add(item)

    def get_number_of_items(self):
        """Get number of menuitems in this Menu"""
        return len(self.__items)

    def set_number_of_visible_items(self, number):
        """
        Set how many menuitems are displayed on the screen at the same time.
        @param number: Number of items
        """
        self.__clip_size = number
        self.set_clip(0,
                      0,
                      self.get_width(),
                      (self.__item_height + self.__gap) * number)

    def get_number_of_visible_items(self):
        """
        Get number of menuitems that are displayed on the screen at the same
        time.

        @return: Ingeter
        """
        return self.__clip_size

    def get_selected(self):
        """
        Get currently selected menuitem
        @return Clutter actor object
        """
        return self.__items[2]

    def scroll_down(self):
        """Scroll menu down by one menuitem."""
        if not self.scroll_timeline.is_playing():
            if self.__animated:
                self.__scroll_direction = self.DOWN
                self.scroll_timeline.start()
            else:
                for item in self.__items:
                    item.move_by(0, self.__item_height + self.__gap)

            # Set last item to first (menu scrolls)
            last = self.__items[-1]
            last.set_position(
                0, self.__items[0].get_y() - self.__gap - self.__item_height)
            self.__items = [last] + self.__items
            del self.__items[-1]

    def scroll_up(self):
        """Scroll menu up by one menuitem."""
        if not self.scroll_timeline.is_playing():
            if self.__animated:
                self.__scroll_direction = self.UP
                self.scroll_timeline.start()
            else:
                for item in self.__items:
                    item.move_by(0, -(self.__item_height + self.__gap))

            # Set first item to last (menu scrolls)
            first = self.__items[0]
            first.set_position(
                0, self.__items[-1].get_y() + self.__gap + self.__item_height)
            self.__items.append(first)
            del self.__items[0]

    def scroll_items_cb(self, timeline, frame_num):
        """Callback function of scroll_down_timeline"""
        delta = timeline.get_delta()[0]
        offset = int((self.__item_height + self.__gap) / 7 * delta)
        if self.__scroll_direction == 1:
            offset = - offset
        for item in self.__items:
            item.move_by(0, offset)

    def is_active(self):
        """
        Is this menu active.
        @return boolean, True if menu is currently active, otherwise False
        """
        return self.__active

    def set_active(self, boolean):
        """
        Set this widget active or inactive
        @param boolean: True to set widget active, False to set inactive
        """
        self.__active = boolean
        if boolean:
            #if self.__selector is not None:
            #    self.__selector.show()
            self.set_opacity(255)
        else:
            #if self.__selector is not None:
            #    self.__selector.hide()
            self.set_opacity(128)

    def update(self):
        """Update the whole menu"""
        for item in self.__items:
            opacity = self.get_opacity_for_y(item.get_y())
            item.set_opacity(opacity)

    def get_opacity_for_y(self, y):
        """Calculation of actor's opacity as a function of its y coordinate"""
        opacity_first_item = 30
        opacity_selected_item = 255

        y_medium_item = 2 * (self.__item_height + self.__gap)
        a  = float(opacity_selected_item - opacity_first_item)
        a /= float(y_medium_item)
        if y <= y_medium_item :
            opacity = y * a + opacity_first_item
        else :
            opacity = opacity_selected_item * 2 - opacity_first_item - a * y

        if opacity < 0:
            opacity = 0

        return int(opacity)

    def actor_notify_y(self, actor, stage):
        """Set opacity to actors when they are moving. Opacity is f(y)"""
        opacity = self.get_opacity_for_y(actor.get_y())
        actor.set_opacity(opacity)

