'''TabGroup is a group of tabs. It draws the tab bar.'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import clutter
import pango

from entertainerlib.frontend.gui.widgets.label import Label
from entertainerlib.frontend.gui.user_event import UserEvent
from entertainerlib.utils.configuration import Configuration

class TabGroup(clutter.Group):
    """
    This is a top level container for tabs.

    This widget contains graphics for tab bar, but content of the tabs are
    separated into Tab objects.
    """

    def __init__(self, width, height, color):
        """
        Initialize empty tab group.
        @param width: Width of the tab bar in percents (0-1)
        @param height: height of the tab bar in percents (0-1)
        @param color: color string to call from the theme
        """
        clutter.Group.__init__(self)
        self.config = Configuration()

        self.active = False

        # Size
        self.stage_w = self.config.get_stage_width()
        self.stage_h = self.config.get_stage_height()
        self.width_factor = width
        self.height_factor = height

        # Label text colors
        self.color = color
        self.selected_color = self.config.theme.get_color(color)
        self.unselected_color = (self.selected_color[0],
            self.selected_color[1], self.selected_color[2], 96)

        # Size of the tab bar (This includes label texts)
        self.rect = clutter.Rectangle()
        self.rect.set_size(
            int(self.stage_w * self.width_factor),
            int(self.stage_h * self.height_factor))
        self.rect.set_color(clutter.Color(255, 0, 0, 0))
        self.add(self.rect)

        # Tab object dictionary that stores tab names as the keys.
        self._tabs = {}

        # Tab objects will be held in this list but should only be used for
        # internal calculations in the TabGroup class
        self.tabs_list = []

        self.labels = []
        self.timelines = [] # Contains tuples (timeline, alpha, behaviour)
        self.current_tab = None

    def tab(self, name):
        '''Return the given tab that matches the provided name'''
        return self._tabs[name]

    def __len__(self):
        """
        Get TagGroup length
        @return: Integer, Number of tabs in this TabGroup
        """
        return len(self.tabs_list)

    def resize_widget(self, stage_size):
        """
        Update widget size for new stage size. This is called when stage size
        has changed.
        @param stage_size: Tuple with width and height in pixels (1028x1024)
        """
        stage = clutter.Stage()
        self.stage_w = stage.get_width()
        self.stage_h = stage.get_height()

    def set_active(self, boolean):
        """
        Set widget status active / inactive
        @param boolean: True to set active, False to set inactive
        """
        self.active = boolean
        if boolean:
            self.set_opacity(255)
        else:
            self.set_opacity(128)

    def is_active(self):
        """
        Is this widget active?
        @return: boolean
        """
        return self.active

    def add_tab(self, tab):
        """
        Add new tab to this tab group.
        @param tab: Tab object to be added
        """

        # Add to the dictionary so that the tab can be called by name
        self._tabs[tab.name] = tab

        self.tabs_list.append(tab)

        # Set first tab active
        if self.current_tab == None:
            self.current_tab = 0
            self.tabs_list[0].show_tab()

        tab_title = Label(0.01, self.color, 0, 0, tab.title)
        self.add(tab_title)
        self.labels.append(tab_title)
        self._reposition_labels()

        timeline = clutter.Timeline(25, 60)
        alpha = clutter.Alpha(timeline, clutter.smoothstep_inc_func)
        behaviour = clutter.BehaviourOpacity(255, 96, alpha)
        behaviour.apply(tab_title)
        self.timelines.append((timeline, alpha, behaviour))

    def _reposition_labels(self):
        """
        This is called every time we need to calculate new positions to the
        labels.

        This is when we add or remove tabs from this TabGroup. This method
        calculates label positions and sizes and updates UI accordingly
        """
        self.font_size = self.get_height() * 0.3
        count = len(self.labels)
        width = int(self.stage_w * self.width_factor)

        for i, label in enumerate(self.labels):
            if i != self.current_tab:
                label.set_color(self.unselected_color)
            label.set_font_name("Sans " + str(self.font_size) + "px")
            label.set_anchor_point_from_gravity(clutter.GRAVITY_CENTER)
            label.set_y(int(self.get_height() / 2))
            label.set_x(
                (width / len(self.labels) * (i + 1)) - (width / count / 2))
            label.set_line_wrap(False)
            label.set_ellipsize(pango.ELLIPSIZE_END)

    def _switch_tab_to_left(self):
        """
        Switch one tab to left if possible. Do nothing if there is no tab
        """
        if self.current_tab > 0 and self.current_tab is not None:
            self.current_tab = self.current_tab - 1

            # Animate labels
            self.timelines[self.current_tab + 1][0].set_direction(
                clutter.TIMELINE_FORWARD)
            self.timelines[self.current_tab + 1][0].start()
            self.timelines[self.current_tab][0].set_direction(
                clutter.TIMELINE_BACKWARD)
            self.timelines[self.current_tab][0].start()

            # Change visible tab
            self.tabs_list[self.current_tab + 1].hide_tab()
            self.tabs_list[self.current_tab].show_tab()

    def _switch_tab_to_right(self):
        """
        Switch one tab to right if possible. Do nothing if there is no tab
        """
        if self.current_tab  < len(self.tabs_list) - 1 and (
            self.current_tab is not None):
            self.current_tab = self.current_tab + 1

            # Animate labels
            self.timelines[self.current_tab - 1][0].set_direction(
                clutter.TIMELINE_FORWARD)
            self.timelines[self.current_tab - 1][0].start()
            self.timelines[self.current_tab][0].set_direction(
                clutter.TIMELINE_BACKWARD)
            self.timelines[self.current_tab][0].start()

            # Change visible tab
            self.tabs_list[self.current_tab - 1].hide_tab()
            self.tabs_list[self.current_tab].show_tab()

    def handle_user_event(self, event):
        """
        Handle screen specific user events
        @param event: UserEvent object
        """
        if self.active:
            event_type = event.get_type()
            if event_type == UserEvent.NAVIGATE_LEFT:
                self._switch_tab_to_left()
            elif event_type == UserEvent.NAVIGATE_RIGHT:
                self._switch_tab_to_right()
            elif event_type == UserEvent.NAVIGATE_DOWN and (
                self.tabs_list[self.current_tab].can_activate()):
                self.set_active(False)
                self.tabs_list[self.current_tab].set_active(True)
        else:
            result = self.tabs_list[self.current_tab].handle_user_event(event)
            if result:
                self.set_active(True)
                self.tabs_list[self.current_tab].set_active(False)

