'''Video Library - Interface for Entertainer video library cache'''

__licence__ = "GPLv2"
__copyright__ = "2007-08, Lauri Taimila, Joshua Scotton"
__author__ = "Lauri Taimila <lauri@taimila.com>, "
__author__ += "Joshua Scotton <josh@joshuascotton.com>"

import os
from pysqlite2 import dbapi2 as sqlite

from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger

from entertainerlib.frontend.medialibrary.playable import Playable

class VideoLibrary:
    """
    Video library.

    Interface for Entertainer's video cache.
    """

    def __init__(self, backend_connection):
        """
        Initialize video library
        @param backend_connection: BackendConnection object
        """
        self.config = Configuration()

        if not os.path.exists(self.config.VIDEO_DB):
            raise Exception("Video database doesn't exist!")
        self.backend_connection = backend_connection

    def get_movies(self):
        """
        Get all movies
        @return: List of Movie objects
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT videofile.filename
                          FROM   videofile, metadata
                          WHERE  type='MOVIE'
                          AND    videofile.filename=metadata.filename
                          ORDER BY title""")
        movies = []
        for row in cursor:
            movies.append(Movie(row[0]))
        connection.close()
        return movies

    def get_tv_series(self):
        """
        Get all TV-series.
        @return: List of TV-series titles (strings)
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT DISTINCT series_title
                          FROM   metadata
                          WHERE  type='TV-SERIES'
                          ORDER BY title""")
        series = []
        for row in cursor:
            series.append(TVSeries(row[0]))
        connection.close()
        return series

    def get_video_clips(self):
        """
        Get List of videos that or not movies or tv-series episodes.
        @return: List of VideoClip objects
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT filename
                          FROM   metadata
                          WHERE  type='CLIP'
                          ORDER BY title""")
        clips = []
        for row in cursor:
            clips.append(VideoClip(row[0]))
        connection.close()
        return clips

    def get_number_of_movies(self):
        """
        Get number of movies in the video library
        @return: Integer
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT COUNT(filename)
                          FROM   metadata
                          WHERE  type='MOVIE'""")
        result = cursor.fetchall()
        connection.close()
        return result[0][0]

    def get_number_of_tv_series(self):
        """
        Get number of TV-series in the video library
        @return: Integer
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT DISTINCT COUNT(series_title)
                          FROM   metadata
                          WHERE  type='TV-SERIES'""")
        result = cursor.fetchall()
        connection.close()
        return result[0][0]

    def get_number_of_video_clips(self):
        """
        Get number of misc. VideoClips in the video library
        @return: Integer
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT COUNT(filename)
                          FROM   metadata
                          WHERE  type='CLIP'""")
        result = cursor.fetchall()
        connection.close()
        return result[0][0]


class VideoItem(Playable):
    """
    Objects from this class represents video items
    """

    def __init__(self):
        """Initialize the VideoItem object"""
        self.config = Configuration()

        #Setting default values
        self.logger = Logger().getLogger(
            'frontend.medialibrary.videos.VideoItem')
        self.__title = ""
        self.__filename = ""
        self.__length = 0
        self.__resolution = ""
        self.__art_hash = ""

    def get_cover_art_url(self):
        """
        Get absolute filename of the cover art image file.
        @return: String
        """
        return os.path.join(self.config.MOVIE_ART_DIR,
            self.get_title() + ".jpg")

    def get_filename(self):
        """
        Get filename (abs path) of the VideoItem
        @return: String
        """
        return self.__filename

    def set_filename(self, filename):
        """
        Sets the filename of the VideoItem
        @param filename String
        """
        self.__filename = filename

    def get_title(self):
        """
        Get title of the VideoItem
        @return: String
        """
        if (self.__title == "") or (self.__title == None):
            return self.get_filename()
        else:
            return self.__title

    def set_title(self, title):
        """
        Sets the title of the VideoItem
        @param title String
        """
        self.__title = title

    def get_length(self):
        """
        Get length of the VideoItem
        @return: Integer
        """
        return self.__length

    def set_length(self, length):
        """
        Sets the length of the VideoItem
        @param length Integer
        """
        self.__length = length

    def get_resolution(self):
        """
        Get resolution of the VideoItem
        @return: String
        """
        return self.__resolution

    def set_resolution(self, res):
        """
        Sets the resolution of the VideoItem
        @param res String
        """
        self.__resolution = res

    def get_thumbnail_url(self):
        """
        Get absolute path of the thumbnail image file.
        @return: String
        """
        if self.get_art_hash() is not None:
            thumb = os.path.join(self.config.VIDEO_THUMB_DIR,
                self.get_art_hash() + ".jpg")
            if os.path.exists(thumb):
                return thumb
            else:
                self.logger.error("Thumbnail does not exist for " + \
                    self.get_filename() + ", using default art instead")
                return os.path.join(self.config.get_theme_path(),
                    "images/default_movie_art.png")
        else:
            self.logger.error("Thumbnail does not exist for " + \
                self.get_filename() + ", using default art instead")
            return os.path.join(self.config.get_theme_path(),
                "images/default_movie_art.png")

    def has_thumbnail(self):
        """
        Is there a thumbnail for this VideoItem
        @return Boolean
        """
        thumb_path = os.path.join(self.config.VIDEO_THUMB_DIR,
            self.get_art_hash() + ".jpg")
        return os.path.exists(thumb_path)

    def get_art_hash(self):
        """
        Gets the art hash
        @return String
        """
        return self.__art_hash

    def set_art_hash(self, arthash):
        """
        Sets the art hash
        @param arthash String
        """
        self.__art_hash = arthash

    def has_cover_art(self):
        """
        Is there cover art file for this VideoItem
        @return: Boolean
        """
        art_path =  os.path.join(self.config.MOVIE_ART_DIR,
            self.get_title() + ".jpg")
        return os.path.exists(art_path)

    # Implement playable interface
    def get_type(self):
        """
        Track is an video stream.
        @return: Integer
        """
        return Playable.VIDEO_STREAM

    def get_uri(self):
        """
        Get URI of the video file
        @return: String
        """
        return "file://" + self.get_filename()


class VideoFilm(VideoItem):
    '''Objects from this class represents video films'''
    def __init__(self):
        VideoItem.__init__(self)
        #initialise values
        self.__plot_outline = ""
        self.__plot = ""
        self.__actors = []
        self.__writers = []
        self.__directors = []
        self.__runtime = ""
        self.__genres = []
        self.__rating = 0.0
        self.__year = 2000

    def get_short_plot(self):
        """
        Get plot outline text.
        @return: String
        """
        return self.__plot_outline

    def set_short_plot(self, plot):
        """
        Set plot outline
        @param plot String
        """
        self.__plot_outline = plot

    def get_plot(self):
        """
        Get long plot text.
        @return: String
        """
        return self.__plot

    def set_plot(self, plot):
        """
        Sets the long plot
        @param plot String
        """
        self.__plot = plot

    def get_actors(self):
        """
        Get list of actors
        @return: List of strings
        """
        return self.__actors

    def set_actors(self, actors):
        """
        Sets list of actors
        @param actors List of Strings
        """
        self.__actors = actors

    def get_writers(self):
        """
        Get list of writers
        @return: List of strings
        """
        return self.__writers

    def set_writers(self, writers):
        """
        Sets list of writers
        @param writers List of Strings
        """
        self.__writers = writers

    def get_directors(self):
        """
        Get list of directors
        @return: List of strings
        """
        return self.__directors

    def set_directors(self, directors):
        """
        Sets list of directors
        @param directors List of Strings
        """
        self.__directors = directors

    def get_runtime(self):
        """
        Get runtime of the VideoFilm
        @return: String
        """
        return self.__runtime

    def set_runtime(self, run):
        """
        Set runtime of the VideoFilm
        @param run String
        """
        self.__runtime = run

    def get_genres(self):
        """
        Get list of genre strings
        @return: List of strings
        """
        return self.__genres

    def set_genres(self, genres):
        """
        Sets list of genres
        @param genres List of Strings
        """
        self.__genres = genres

    def get_rating(self):
        """
        Get rating of the VideoFilm (1-5)
        @return: Integer
        """
        return int(self.__rating)

    def set_rating(self, rating):
        """
        Sets rating of the VideoFilm (1-5)
        @param rating Integer
        """
        self.__rating = int(rating)

    def get_year(self):
        """
        Get release year of the VideoFilm.
        @return: String
        """
        return int(self.__year)

    def set_year(self, year):
        """
        Sets the year of the VideoFilm
        @param year String
        """
        try:
            self.__year = int(year)
        except ValueError:
            self.__year = 0
            print "Year for Video should be an integer, not: ", year


class VideoClip(VideoItem):
    """
    Objects from this class represents video files.
    """

    def __init__(self, filename):
        """
        Initialize Clip information
        @param filename: Absolute path of the video clip file
        """
        VideoItem.__init__(self)
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute(
            """SELECT videofile.filename, hash, length, resolution, title
                          FROM   videofile, metadata
                          WHERE  videofile.filename=:fn
                          AND    videofile.filename=metadata.filename
                          ORDER BY title""", { "fn" : filename })
        result = cursor.fetchall()
        self.set_filename(result[0][0])    # Filename of the clip
        self.set_art_hash(result[0][1])   # Cover art hash
        self.set_length(result[0][2])      # Lenght of the movie
        self.set_resolution(result[0][3])  # Video resolution
        self.set_title(result[0][4])       # Clip title
        connection.close()


class Movie(VideoFilm):
    """
    Objects from this class represents movie.
    """


    def __init__(self, filename):
        """
        Initialize Movie information
        @param filename: Absolute path of the Movie file
        """
        VideoFilm.__init__(self)
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT metadata.filename, hash, length, resolution,
                                 title, runtime, genres, rating, year,
                                 plot_outline, plot, actor_1, actor_2,
                                 actor_3, actor_4, actor_5, writer_1,
                                 writer_2, director_1, director_2
                          FROM   videofile, metadata
                          WHERE  metadata.filename=:fn
                          AND    videofile.filename=metadata.filename
                          ORDER BY title""", { "fn" : filename })
        result = cursor.fetchall()
        self.set_filename(result[0][0])
        self.set_art_hash(result[0][1])
        self.set_length(result[0][2])
        self.set_resolution(result[0][3])
        self.set_title(result[0][4])
        self.set_runtime(result[0][5])
        self.set_genres(result[0][6].split(','))
        self.set_rating(result[0][7])
        self.set_year(result[0][8])
        self.set_short_plot(result[0][9])
        self.set_plot(result[0][10])
        self.set_actors([])
        self.set_art_hash([])
        self.set_directors([])

        # Actors
        if len(result[0][11]) > 0:
            self.get_actors().append(result[0][11])
        if len(result[0][12]) > 0:
            self.get_actors().append(result[0][12])
        if len(result[0][13]) > 0:
            self.get_actors().append(result[0][13])
        if len(result[0][14]) > 0:
            self.get_actors().append(result[0][14])
        if len(result[0][15]) > 0:
            self.get_actors().append(result[0][15])

        # Writers
        if len(result[0][16]) > 0:
            self.get_writers().append(result[0][16])
        if len(result[0][17]) > 0:
            self.get_writers().append(result[0][17])

        # Directors
        if len(result[0][18]) > 0:
            self.get_directors().append(result[0][18])
        if len(result[0][19]) > 0:
            self.get_directors().append(result[0][19])

        connection.close()


class TVSeries(object):
    """
    TV Series

    TV Series contains TVEpisodes organized by seasons.
    """

    def __init__(self, title):
        """
        Initialize TV-Series
        @param title: Series title
        """
        self.config = Configuration()

        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT DISTINCT series_title
                          FROM   metadata
                          WHERE  series_title=:title""", { "title" : title })
        result = cursor.fetchall()
        self.__title = None
        if len(result) == 0:
            raise Exception("No such TV-series as " + title)
        else:
            self.__title = title
        connection.close()

    def get_title(self):
        """
        Get title of the series
        @return: String
        """
        return self.__title

    def set_title(self, title):
        """
        Sets title of the series
        @param title String
        """
        self.__title = title

    def has_cover_art(self):
        """
        Is there cover art file for this TV-serie
        @return: String
        """
        art_path =  os.path.join(self.config.MOVIE_ART_DIR,
            self.get_title() + ".jpg")
        return os.path.exists(art_path)

    def get_cover_art_url(self):
        """
        Get absolute filename of the cover art image file.
        @return: String
        """
        return os.path.join(self.config.MOVIE_ART_DIR,
            self.get_title() + ".jpg")

    def get_seasons(self):
        """
        Return list of numbers. For example [1,2,3,5] in case that user
        doesn't have episodes from season 4.
        @return: List of Integers
        """
        seasons = []
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT DISTINCT season
                          FROM   metadata
                          WHERE  series_title=:title""",
                          { "title" : self.get_title() })
        for row in cursor:
            seasons.append(row[0])
        connection.close()
        return seasons

    def get_number_of_episodes(self):
        """
        Get number of episodes available on this series
        @return: Integer
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT COUNT(filename)
                          FROM   metadata
                          WHERE  series_title=:title""",
                          { "title" : self.get_title() })
        result = cursor.fetchall()
        connection.close()
        return result[0][0]

    def get_number_of_seasons(self):
        """
        Get number of seasons available on this series
        @return: Integer
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT COUNT(DISTINCT season)
                          FROM   metadata
                          WHERE  series_title=:title""",
                          { "title" : self.get_title() })
        result = cursor.fetchall()
        connection.close()
        return result[0][0]

    def get_episodes_from_season(self, season):
        """
        Get all episodes of the season.
        @param season: Season number (Integer)
        """
        episodes = []
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT filename, episode
                          FROM   metadata
                          WHERE  series_title=:title
                          AND    season=:season
                          ORDER BY episode""",
                          { "title" : self.get_title(), "season" : season })
        for row in cursor:
            episodes.append(TVEpisode(row[0], self))
        connection.close()
        return episodes

    def get_episode_from_season(self, season, episode):
        """
        Get specific episode from this series.
        @param season: Season number (Integer)
        @param episode: Episode number (Integer)
        @return TVEpisode object
        """
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT filename
                          FROM   metadata
                          WHERE  series_title=:title
                          AND    episode=:episode
                          AND    season=:season""",
                          { "title" : self.get_title(),
                            "season" : season,
                            "episode" : episode })
        result = cursor.fetchall()
        episode = TVEpisode(result[0][0], self)
        connection.close()
        return episode

    def get_episodes(self):
        """
        Get all episodes of this series.
        @return: List of TVEpisode objects
        """
        episodes = []
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT filename, season, episode
                          FROM   metadata
                          WHERE  series_title=:title
                          ORDER BY season, episode""",
                          { "title" : self.get_title() })
        for row in cursor:
            episodes.append(TVEpisode(row[0], self))
        connection.close()
        return episodes


class TVEpisode(VideoFilm):
    """
    Objects from this class represents TV-Episode video file.
    """

    def __init__(self, filename, series):
        """
        Initialize episode information.
        @param filename: Name of the video file
        @param series: TVSeries object
        """
        VideoFilm.__init__(self)
        connection = sqlite.connect(self.config.VIDEO_DB)
        cursor = connection.cursor()
        cursor.execute("""SELECT videofile.filename, hash, length, resolution,
                                 title, runtime, genres, rating, year,
                                 plot_outline, plot, actor_1, actor_2,
                                 actor_3, actor_4, actor_5, writer_1,
                                 writer_2, director_1, director_2,
                                 series_title, season, episode
                          FROM   videofile, metadata
                          WHERE  videofile.filename=:fn
                          AND    videofile.filename=metadata.filename
                          ORDER BY title""", { "fn" : filename })
        result = cursor.fetchall()
        self.set_filename(result[0][0])
        self.set_art_hash(result[0][1])
        self.set_length(result[0][2])
        self.set_resolution(result[0][3])
        self.set_title(result[0][4])
        self.set_runtime(result[0][5])
        self.set_genres(result[0][6].split(','))
        self.set_rating(result[0][7])
        self.set_year(result[0][8])
        self.set_short_plot(result[0][9])
        self.set_plot(result[0][10])

        # Actors
        self.set_actors([])
        if len(result[0][11]) > 0:
            self.get_actors().append(result[0][11])
        if len(result[0][12]) > 0:
            self.get_actors().append(result[0][12])
        if len(result[0][13]) > 0:
            self.get_actors().append(result[0][13])
        if len(result[0][14]) > 0:
            self.get_actors().append(result[0][14])
        if len(result[0][15]) > 0:
            self.get_actors().append(result[0][15])

        # Writers
        self.set_writers([])
        if len(result[0][16]) > 0:
            self.get_writers().append(result[0][16])
        if len(result[0][17]) > 0:
            self.get_writers().append(result[0][17])

        # Directors
        self.set_directors([])
        if len(result[0][18]) > 0:
            self.get_directors().append(result[0][18])
        if len(result[0][19]) > 0:
            self.get_directors().append(result[0][19])

        self.__series = series
        self.__season = result[0][21]
        self.__episode_number = result[0][22]
        connection.close()

    def get_cover_art_url(self):
        """
        Get absolute filename of the cover art image file.
        @return: String
        """
        return os.path.join(self.config.MOVIE_ART_DIR,
            self.get_series() + ".jpg")

    def get_series(self):
        """
        Get TV-series
        @return: TVSeries objects
        """
        return self.__series

    def set_series(self, series):
        """
        Sets TV-Series
        @param series TVSeries
        """
        self.__series = series

    def get_episode_number(self):
        """
        Get number of the episode
        @return: Integer
        """
        return int(self.__episode_number)

    def set_episode_number(self, num):
        """
        Sets the number of the episode
        @param num Integer
        """
        self.__episode_number = int(num)

    def get_season(self):
        """
        Get number of the season
        @return: Interger
        """
        return int(self.__season)

    def set_season(self, season):
        """
        Sets number of the season
        @param season Integer
        """
        self.__season = int(season)

