'''Tests Configuration'''
# pylint: disable-msg=W0212

__license__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import os

from entertainerlib.tests import EntertainerTest
from entertainerlib.utils.configuration import Configuration

class ConfigurationTest(EntertainerTest):
    '''Test for utils.configuration'''

    def setUp(self):
        '''Set up the test'''
        EntertainerTest.setUp(self)

        self.configuration = Configuration()

    def tearDown(self):
        '''Clean up after the test'''
        EntertainerTest.tearDown(self)

    def testCreate(self):
        """Test correct Configuration initialization"""
        self.assertTrue(isinstance(self.configuration, Configuration))

    def test_create_dir(self):
        '''Test Configuration object directory creation'''
        self.assertTrue(os.path.exists(self.test_cfg_dir))

    def testBorg(self):
        '''Test that configuration objects share state'''
        self.second_config = Configuration()
        self.assertTrue(
            self.second_config.__dict__ is self.configuration.__dict__)

    def testGetStageWidth(self):
        '''Test getting the stage width'''
        self.assertEqual(self.configuration.get_stage_width(), 1366)

    def testSetStageWidth(self):
        '''Test setting the stage width'''
        new_width = 1600
        self.configuration.set_stage_width(new_width)
        self.assertEqual(self.configuration.get_stage_width(), new_width)

        # Test that the value is properly tainted
        self.assertEqual(
            self.configuration._tainted_in_memory[
                self.configuration.set_stage_width],
            {'new_width' : 1366, 'sanitize' : True})

    def testGetStageHeight(self):
        '''Test getting the stage height'''
        self.assertEqual(self.configuration.get_stage_height(), 768)

    def testSetStageHeight(self):
        '''Test setting the stage height'''
        new_height = 900
        self.configuration.set_stage_height(new_height)
        self.assertEqual(self.configuration.get_stage_height(), new_height)

        # Test that the value is properly tainted
        self.assertEqual(
            self.configuration._tainted_in_memory[
                self.configuration.set_stage_height],
            {'new_height' : 768, 'sanitize' : True})

    def testGetVerticalOffset(self):
        '''Test getting the vertical offset'''
        self.configuration.set_stage_height(11)
        self.configuration.set_stage_width(16)
        self.assertEqual(self.configuration.get_vertical_offset(), 1)

    def testGetCfgDir(self):
        """Test getting the cfg dir"""
        self.assertEqual(self.configuration.get_cfg_dir(), self.config.TEST_DIR)

    def testTrayIconEnabled(self):
        """Test getting the display_icon boolean"""
        self.assertFalse(self.configuration.tray_icon_enabled())

    def test_write_content_value(self):
        """Test writing a value to the content configuration file"""
        new_value = "Gaithersburg,Maryland"
        self.configuration.write_content_value("Weather", "location", new_value)
        self.configuration.update_configuration()
        self.assertEqual(self.configuration.get_weather_location(), new_value)

        # Test that the value is properly tainted
        self.assertEqual(
            self.configuration._tainted['content Weather location'],
            'Bath,England')

    def test_write_preference_value(self):
        """Test writing a value to the preference configuration file"""
        new_value = True
        self.configuration.write_preference_value(
            "General", "display_icon", new_value)
        self.configuration.update_configuration()
        self.assertEqual(self.configuration.tray_icon_enabled(), new_value)

        # Test that the value is properly tainted
        self.assertEqual(
            self.configuration._tainted['preferences General display_icon'],
            'False')

    def testGetThemeName(self):
        """Test getting the current theme name"""
        current_name = "Default"
        self.assertEqual(self.configuration.get_theme_name(), current_name)

    def testStartAutoServer(self):
        """Test getting the start_auto_server boolean"""
        self.assertTrue(self.configuration.start_auto_server())

    def testGetSlideshowStep(self):
        '''Test getting the slideshow step'''
        self.assertEqual(self.configuration.get_slideshow_step(), 5)

    def test_taint(self):
        '''Test tainting configuration data'''
        # Test tainting preferences data
        kind = 'preferences'
        section = 'General'
        option = 'history_size'
        self.configuration.taint(kind, section, option)
        key = "%s %s %s" % (kind, section, option)
        self.assertEqual(self.configuration._tainted[key], '8')

        # Test tainting content data
        kind = 'content'
        section = 'Videos'
        option = 'download_metadata'
        self.configuration.taint(kind, section, option)
        key = "%s %s %s" % (kind, section, option)
        self.assertEqual(self.configuration._tainted[key], 'True')

        # Test the bad branch
        kind = 'bogus'
        self.assertRaises(Exception, self.configuration.taint, kind, section,
            option)

    def test_taint_in_memory(self):
        '''Test tainting data already in memory'''
        value = 'test'

        def set_value(value):
            '''Dummy method used as a key'''

        kwargs = {'value' : value}
        self.configuration.taint_in_memory(set_value, kwargs)
        self.assertEqual(self.configuration._tainted_in_memory[set_value],
            kwargs)

    def test_sanitize(self):
        '''Test sanitizing configuration data'''
        # Test sanitizing for config file data
        new_value = 15
        self.configuration.write_preference_value(
            "General", "history_size", new_value)
        self.configuration.update_configuration()
        self.configuration.sanitize()
        self.assertEqual(self.configuration.history_size(), 8)

        # Test sanitizing in memory data
        self.configuration.set_stage_height(400)
        self.configuration.sanitize()
        self.assertEqual(self.configuration.get_stage_height(), 768)

