'''Tests feed entries on the frontend'''
# pylint: disable-msg=W0212

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

from pysqlite2 import dbapi2 as sqlite

from entertainerlib.frontend.medialibrary.feeds import Feed, Entry
from entertainerlib.tests import EntertainerTest

class FrontendFeedEntryTest(EntertainerTest):
    '''Tests feed entries on the frontend'''

    entry_title = "Test Entry"
    entry_desc = "This is a test description"
    entry_id = "1"
    entry_time = "13:02"
    entry_date = "02/02/08"

    def setUp(self):
        """Sets up Entry object for the test"""
        EntertainerTest.setUp(self)

        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""CREATE TABLE feed(
                                url TEXT,
                                title TEXT,
                                subtitle TEXT,
                                description TEXT,
                                time TIME,
                                date DATE,
                                PRIMARY KEY(url))""")
        db_cursor.execute("""CREATE TABLE entry(
                                feed_url TEXT,
                                title TEXT,
                                description TEXT,
                                isread TEXT,
                                time TIME,
                                date DATE,
                                id TEXT,
                                PRIMARY KEY(id))""")
        db_conn.commit()
        entry_row = ("url2",
                     "entry1",
                     "desc",
                     "NO",
                     "1",
                     "2008-01-01",
                     "12:00:00")
        db_cursor.execute("""INSERT INTO entry(
            feed_url,title,description,isread,id,date,time)
            VALUES (?,?,?,?,?,?,?)""", entry_row)

        db_conn.commit()
        feed_row = ("url2",
                    "title2",
                    "subtitle2",
                    "description2",
                    "2008-01-01",
                    "12:00:00")
        db_cursor.execute(
            """
            INSERT INTO feed(url,title,subtitle,description,date,time)
            VALUES (?,?,?,?,?,?)""", feed_row)
        db_conn.commit()
        db_conn.close()
        self.entry = Entry(self.entry_title, self.entry_desc, self.entry_id,
            self.entry_time, self.entry_date, False)

    def tearDown(self):
        """
        Removes test database file
        """
        EntertainerTest.tearDown(self)

        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""DROP TABLE IF EXISTS feed""")
        db_cursor.execute("""DROP TABLE IF EXISTS entry""")
        db_conn.commit()
        db_conn.close()

    def testInitReadEntry(self):
        """Tests if Entry can handle the additional read parameter"""
        new_entry = Entry( self.entry_title, self.entry_desc, self.entry_id,
                           self.entry_time, self.entry_date, True)
        self.assertTrue(new_entry.is_read())


    def testGetTitle(self):
        """
        testGetTitle: Tests if the title parameter is set properly
        """
        self.assertEqual(self.entry_title, self.entry.get_title())

    def testGetDescription(self):
        """
        testGetDescription: Tests if the description parameter is set properly
        """
        self.assertEqual(self.entry_desc, self.entry.get_description())

    def testGetId(self):
        """
        testGetId: Tests if the id parameter is set properly
        """
        self.assertEqual(self.entry_id, self.entry.get_id())

    def testGetTime(self):
        """
        testGetTime: Tests if the time parameter is set properly
        """
        self.assertEqual(self.entry_time, self.entry.get_time())

    def testGetDate(self):
        """
        testGetDate: Tests if the date parameter is set properly
        """
        self.assertEqual(self.entry_date, self.entry.get_date())

    def testIsRead(self):
        """
        testIsRead: Tests if the read parameter of Entry defaults to False
        """
        self.assertFalse(self.entry.is_read())

    def testMarkAsRead(self):
        """
        testMarkAsRead: Tests if mark_as_read() changes the read parameter
        """
        self.entry.mark_as_read()
        self.assertTrue(self.entry.is_read())

    def testMarkAsUnread(self):
        """
        testMarkAsUnread: Tests if mark_as_unread() changes the read parameter
        """
        self.entry.mark_as_read()
        self.entry.mark_as_unread()
        self.assertFalse(self.entry.is_read())

    def testSetRead(self):
        """
        testSetRead: Tests if _set_read() changes the read parameter
        """
        self.entry._set_read(True)
        self.assertTrue(self.entry.is_read())

    def testDatabaseConnectionSetRead(self):
        """Tests if _set_read changes database properly"""
        feed = Feed("url2")
        self.entry._set_read(True)
        self.assertEqual(feed.get_number_of_unread(), 0)

