'''Tests feed library from the frontend'''

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

from pysqlite2 import dbapi2 as sqlite

from entertainerlib.frontend.medialibrary.feeds import FeedLibrary
from entertainerlib.tests import EntertainerTest
from entertainerlib.tests.mock import MockBackendConnection

class FrontendFeedLibraryTest(EntertainerTest):
    """
    Tests the frontend.medialibrary.feeds.FeedLibrary object
    @todo comment tests
    @todo the test database doesn't seem to be working
    """

    def setUp(self):
        """Sets up everything for the test"""
        EntertainerTest.setUp(self)

        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""CREATE TABLE feed(
                                url TEXT,
                                title TEXT,
                                subtitle TEXT,
                                description TEXT,
                                time TIME,
                                date DATE,
                                PRIMARY KEY(url))""")

        db_cursor.execute("""CREATE TABLE entry(
                                feed_url TEXT,
                                title TEXT,
                                description TEXT,
                                isread TEXT,
                                time TIME,
                                date DATE,
                                id TEXT,
                                PRIMARY KEY(id))""")
        db_conn.commit()
        for i in range(10):
            feed_row = ("url"+str(i),
                        "title"+str(i),
                        "subtitle"+str(i),
                        "description"+str(i),
                        "2008-01-01",
                        "12:00:00")
            db_cursor.execute("""
                INSERT INTO feed(url,title,subtitle,description,date,time)
                VALUES (?,?,?,?,?,?)""", feed_row)
            db_conn.commit()
        for i in range(9):
            if i % 2 == 0:
                read = "NO"
            else:
                read = "YES"
            entry_row = ("url2",
                         "entry"+str(i),
                         "desc",
                         read,
                         str(i),
                         "2008-01-01",
                        "12:00:0"+str(i))
            db_cursor.execute("""INSERT INTO
                entry(feed_url,title,description,isread,id,date,time)
                VALUES (?,?,?,?,?,?,?)""", entry_row)
            db_conn.commit()
        db_conn.close()

        self.backend_connection = MockBackendConnection()
        self.library = FeedLibrary(self.backend_connection)

    def tearDown(self):
        """
        Removes test database file
        """
        EntertainerTest.tearDown(self)

        db_conn = sqlite.connect(self.config.FEED_DB)
        db_cursor = db_conn.cursor()
        db_cursor.execute("""DROP TABLE IF EXISTS feed""")
        db_cursor.execute("""DROP TABLE IF EXISTS entry""")
        db_conn.commit()
        db_conn.close()

    def testRemoveFeed(self):
        """
        testRemoveFeed: Tests FeedLibrary.remove_feed()
        """
        self.library.remove_feed("url2")
        self.assertEqual(self.library.number_of_feeds(), 9)

    def testIsEmptyWhenFull(self):
        """
        testIsEmptyWhenFull: Tests FeedLibrary.is_empty() when there are feeds
        in database
        """
        self.assertFalse(self.library.is_empty())

    def testIsEmptyWhenEmpty(self):
        """
        testIsEmptyWhenEmpty: Tests FeedLibrary.is_empty() when the feed list
        is empty
        """
        for i in range(10):
            self.library.remove_feed("url"+str(i))
        self.assertTrue(self.library.is_empty())

    def testGetFeeds(self):
        """
        testGetFeeds: Tests FeedLibrary.get_feeds() sorted by TITLE
        """
        feedlist = self.library.get_feeds('TITLE')
        for i in range(10):
            self.assertEqual(feedlist[i].get_feed_url(), "url"+str(i))

    def testGetFeedsByUnread(self):
        """
        testGetFeedsByUnread: Tests FeedLibrary.get_feeds() sorted by unread
        """
        feedlist = self.library.get_feeds('UNREAD')
        self.assertEqual(feedlist[0].get_feed_url(), "url2")

    def testGetLatestEntries(self):
        """
        testGetLatestEntries: Tests FeedLibrary.get_latest_entries()
        """
        latest = self.library.get_latest_entries(1)
        self.assertEqual("12:00:08", str(latest[0][1].get_time()))

    def testNumberOfFeeds(self):
        """
        testNumberOfFeeds: Tests FeedLibrary.number_of_feeds()
        """
        self.assertEqual(self.library.number_of_feeds(), 10)

    def testNumberOfUnreadEntries(self):
        """
        testNumberOfUnreadEntries: Tests FeedLibrary.number_of_unread_entries()
        """
        self.assertEqual(self.library.number_of_unread_entries(), 5)

    def testMarkAllAsRead(self):
        """
        testMarkAllAsRead: tests if all entries are marked as read
        """
        self.library.mark_all_as_read()
        self.assertEqual(self.library.number_of_unread_entries(), 0)

