'''Tests for all models'''

__license__ = 'GPLv2'
__copyright__ = '2008, Paul Hummer'
__author__ = 'Paul Hummer <paul.hummer@canonical.com>'

from storm.locals import Store

from entertainerlib.backend.core.db import models
from entertainerlib.tests import EntertainerTestWithDatabase


class ModelTestCase(EntertainerTestWithDatabase):
    '''Abstract Model Test Case

    The parent class of all model test classes.
    '''

    def setUp(self):
        EntertainerTestWithDatabase.setUp(self)


class TestBaseModel(ModelTestCase):
    '''BaseModel test case.'''

    def test_to_dict(self):
        '''Test that the to_dict function raises a NotImplemented exception.
        '''
        test_model = models.BaseModel()
        self.assertRaises(NotImplementedError, test_model.to_dict)


class TestNewsFeed(ModelTestCase):
    '''NewsFeed test case'''

    def testCreate(self):
        '''Test creation of new NewsFeed'''

        store = Store(self.db)

        newsfeed = models.NewsFeed()
        newsfeed.url = u'http://theironlion.net/blog/feed'
        newsfeed.title = u'The Iron Lion'
        newsfeed.description = u'Paul Hummer\'s Blog'
        store.add(newsfeed)
        store.commit()

        self.assertTrue(Store.of(newsfeed) is store)

        newsfeed_from_database = store.find(models.NewsFeed,
            models.NewsFeed.title == u'The Iron Lion').one()

        self.assertTrue(newsfeed is newsfeed_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        newsfeed = models.NewsFeed()
        newsfeed.url = u'http://theironlion.net/blog/feed'
        newsfeed.title = u'The Iron Lion'
        newsfeed.description = u'Paul Hummer\'s Blog'
        store.add(newsfeed)

        entry = models.NewsEntry()
        entry.title = u'Rockenrolla'
        newsfeed.entries.add(entry)

        store.commit()

        newsfeed_dict = newsfeed.to_dict()
        self.assertEqual(
            newsfeed_dict['url'], u'http://theironlion.net/blog/feed')
        self.assertEqual(
            newsfeed_dict['title'], u'The Iron Lion')
        self.assertEqual(
            newsfeed_dict['description'], u'Paul Hummer\'s Blog')
        self.assertEqual(
            len(newsfeed_dict['entries']), 1)
        self.assertEqual(
            newsfeed_dict['entries'][0]['title'], u'Rockenrolla')


class TestNewsEntry(ModelTestCase):
    '''NewsEntry test case'''

    def testCreate(self):
        '''Test creation of new NewsEntry'''

        store = Store(self.db)
        newsentry = models.NewsEntry()
        newsentry.url = u'http://theironlion.net/blog/foo-bar-baz'
        newsentry.title = u'Foo Bar Baz'
        newsentry.description = u'There once was a girl from Nantucket'
        newsentry.is_read = 0
        store.add(newsentry)

        store.commit()

        self.assertTrue(Store.of(newsentry) is store)

        newsentry_from_database = store.find(models.NewsEntry,
            models.NewsEntry.title == u'Foo Bar Baz').one()
        self.assertTrue(newsentry is newsentry_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        feed = models.NewsFeed()
        store.add(feed)

        newsentry = models.NewsEntry()
        newsentry.url = u'http://theironlion.net/blog/foo-bar-baz'
        newsentry.title = u'Foo Bar Baz'
        newsentry.description = u'There once was a girl from Nantucket'
        newsentry.is_read = 0
        newsentry.feed = feed
        store.add(newsentry)
        store.commit()

        newsentry_dict = newsentry.to_dict()
        self.assertEqual(
            newsentry_dict['url'], u'http://theironlion.net/blog/foo-bar-baz')
        self.assertEqual(
            newsentry_dict['title'], u'Foo Bar Baz')
        self.assertEqual(
            newsentry_dict['description'],
            u'There once was a girl from Nantucket')
        self.assertEqual(
            newsentry_dict['creation_date'], None)
        self.assertEqual(newsentry_dict['is_read'], 0)
        self.assertEqual(newsentry_dict['feed']['id'], feed.id)


class TestPhotoAlbum(ModelTestCase):
    '''PhotoAlbum test case'''

    def testCreate(self):
        '''Test creation of new PhotoAlbum'''

        store = Store(self.db)
        photoalbum = models.PhotoAlbum()
        photoalbum.title = u'Photo Album Title'
        photoalbum.description = u'This is a photo description'
        store.add(photoalbum)
        store.commit()

        self.assertTrue(Store.of(photoalbum) is store)

        photoalbum_from_database = store.find(models.PhotoAlbum,
            models.PhotoAlbum.title == u'Photo Album Title').one()
        self.assertTrue(photoalbum is photoalbum_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        photoalbum = models.PhotoAlbum()
        photoalbum.title = u'Photo Album Title'
        photoalbum.description = u'This is a photo description'
        store.add(photoalbum)

        image = models.PhotoImage()
        image.title = u'Super Fun Photo'
        photoalbum.images.add(image)
        store.commit()

        photoalbum_dict = photoalbum.to_dict()
        self.assertEqual(
            photoalbum_dict['title'], u'Photo Album Title')
        self.assertEqual(
            photoalbum_dict['description'], u'This is a photo description')
        self.assertEqual(
            len(photoalbum_dict['images']), 1)
        self.assertEqual(
            photoalbum_dict['images'][0]['title'], u'Super Fun Photo')


class TestPhotoImage(ModelTestCase):
    '''PhotoImage test case'''

    def testCreate(self):
        '''Test creation of new PhotoImage'''

        store = Store(self.db)
        photoimage = models.PhotoImage()
        photoimage.filename = u'/home/user/photo.jpg'
        store.add(photoimage)
        store.commit()

        self.assertTrue(Store.of(photoimage) is store)

        photoimage_from_database = store.find(models.PhotoImage,
            models.PhotoImage.filename == u'/home/user/photo.jpg').one()
        self.assertTrue(photoimage is photoimage_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        photoalbum = models.PhotoAlbum()
        photoalbum.title = u'Spring Break 1978'
        store.add(photoalbum)

        photoimage = models.PhotoImage()
        photoimage.filename = u'/home/user/photo.jpg'
        photoimage.title = u'Forgot Sunscreen!'
        photoimage.description = u'Look how sunburned I am'
        photoimage.album = photoalbum
        store.add(photoimage)

        store.commit()

        photoimage_dict = photoimage.to_dict()
        self.assertEqual(
            photoimage_dict['filename'], u'/home/user/photo.jpg')
        self.assertEqual(
            photoimage_dict['thumb'], None)
        self.assertEqual(
            photoimage_dict['title'], u'Forgot Sunscreen!')
        self.assertEqual(
            photoimage_dict['description'], u'Look how sunburned I am')
        self.assertEqual(
            photoimage_dict['creation_date'], None)
        self.assertEqual(
            photoimage_dict['album']['title'], u'Spring Break 1978')


class TestMusicAlbum(ModelTestCase):
    '''MusicAlbum test case'''

    def testCreate(self):
        '''Test creation of new MusicAlbum'''

        store = Store(self.db)
        musicalbum = models.MusicAlbum()
        musicalbum.title = u'The Lady Dance'
        store.add(musicalbum)
        store.commit()

        self.assertTrue(Store.of(musicalbum) is store)

        musicalbum_from_database = store.find(models.MusicAlbum,
            models.MusicAlbum.title == u'The Lady Dance').one()
        self.assertTrue(musicalbum is musicalbum_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        musicalbum = models.MusicAlbum()
        musicalbum.title = u'The Lady Dance'
        musicalbum.artist = u'Enter the Dragon'

        musictrack = models.MusicTrack()
        musictrack.title = u'Naked Dance'
        musicalbum.tracks.add(musictrack)
        store.add(musicalbum)
        store.commit()

        musicalbum_dict = musicalbum.to_dict()
        self.assertEqual(
            musicalbum_dict['title'], u'The Lady Dance')
        self.assertEqual(
            musicalbum_dict['artist'], u'Enter the Dragon')
        self.assertEqual(
            musicalbum_dict['year'], None)
        self.assertEqual(
            musicalbum_dict['genre'], None)
        self.assertEqual(
            len(musicalbum_dict['tracks']), 1)
        self.assertEqual(
            musicalbum_dict['tracks'][0]['title'],
            u'Naked Dance')


class TestMusicTrack(ModelTestCase):
    '''MusicTrack test case'''

    def testCreate(self):
        '''Test creation of new MusicTrack'''

        store = Store(self.db)
        musictrack = models.MusicTrack()
        musictrack.title = u'The Beautiful Ones'
        store.add(musictrack)
        store.commit()

        self.assertTrue(Store.of(musictrack) is store)

        musictrack_from_database = store.find(models.MusicTrack,
            models.MusicTrack.title == u'The Beautiful Ones').one()
        self.assertTrue(musictrack is musictrack_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        musictrack = models.MusicTrack()
        musictrack.title = u'The Beautiful Ones'
        musictrack.filename = u'/foo/bar.mp3'

        musicalbum = models.MusicAlbum()
        musicalbum.title = u'Super Best Friends'
        musictrack.album = musicalbum

        store.add(musictrack)
        store.commit()

        musictrack_dict = musictrack.to_dict()
        self.assertEqual(
            musictrack_dict['filename'], u'/foo/bar.mp3')
        self.assertEqual(
            musictrack_dict['title'], u'The Beautiful Ones')
        self.assertEqual(
            musictrack_dict['tracknumber'], None)
        self.assertEqual(
            musictrack_dict['rating'], None)
        self.assertEqual(
            musictrack_dict['length'], None)
        self.assertEqual(
            musictrack_dict['bitrate'], None)
        self.assertEqual(
            musictrack_dict['comment'], None)
        self.assertEqual(
            musictrack_dict['lyrics'], None)
        self.assertEqual(
            musictrack_dict['album']['title'],
            u'Super Best Friends')



class TestMusicPlaylist(ModelTestCase):
    '''MusicPlaylist test case'''

    def testCreate(self):
        '''Test creation of new MusicPlaylist'''

        store = Store(self.db)
        musicplaylist = models.MusicPlaylist()
        musicplaylist.title = u'The Ultimate Heavy Metal Goth Emo'
        store.add(musicplaylist)
        store.commit()

        self.assertTrue(Store.of(musicplaylist) is store)

        musicplaylist_from_database = store.find(models.MusicPlaylist,
            models.MusicPlaylist.title == \
                u'The Ultimate Heavy Metal Goth Emo').one()
        self.assertTrue(musicplaylist is musicplaylist_from_database)


class TestVideoFile(ModelTestCase):
    '''VideoFile test case'''

    def testCreate(self):
        '''Test creation of new VideoFile'''

        store = Store(self.db)
        videofile = models.VideoFile()
        videofile.filename = u'/home/user/foo.mpg'
        store.add(videofile)
        store.commit()

        self.assertTrue(Store.of(videofile) is store)

        videofile_from_database = store.find(models.VideoFile,
            models.VideoFile.filename == u'/home/user/foo.mpg').one()
        self.assertTrue(videofile is videofile_from_database)

    def test_to_dict(self):
        '''Test that to_dict returns an accurate dict.'''
        store = Store(self.db)

        videofile = models.VideoFile()
        videofile.filename = u'/home/user/foo.mpg'
        store.add(videofile)
        store.commit()

        videofile_dict = videofile.to_dict()
        self.assertEqual(
            videofile_dict['filename'], u'/home/user/foo.mpg')
        self.assertEqual(
            videofile_dict['thumb'], None)
        self.assertEqual(
            videofile_dict['type'], None)



