'''Tests Music'''

__licence__ = "GPLv2"
__copyright__ = "2008, Matt Layman"
__author__ = "Matt Layman <laymansterms.dev@gmail.com>"

import os

from pysqlite2 import dbapi2 as sqlite

from entertainerlib.frontend.medialibrary.music import (
    Album, AlbumHasNoTracks, MusicLibrary, Track, TrackRatingOutOfRange,
    TrackTypeError)
from entertainerlib.tests.mock import MockBackendConnection
from entertainerlib.frontend.medialibrary.playable import Playable
from entertainerlib.tests import EntertainerTest

class TestMusic(EntertainerTest):
    """This test class sets up the test database required for all test
    classes that need to access the music cache"""

    def setUp(self):
        """See unittest.TestCase"""
        EntertainerTest.setUp(self)

        self.debug = False
        # The location of album art is required for some tests
        self.art_path = self.config.ALBUM_ART_DIR

        connection = sqlite.connect(self.config.MUSIC_DB)
        self.cursor = connection.cursor()
        self.cursor.execute("""DROP TABLE IF EXISTS track""")
        self.cursor.execute("""CREATE TABLE track(
                            filename TEXT,
                            title VARCHAR(255),
                            artist VARCHAR(255),
                            album VARCHAR(255),
                            tracknumber INTEGER,
                            bitrate INTEGER,
                            year INTEGER,
                            rating INTEGER DEFAULT NULL,
                            length INTEGER,
                            genre VARCHAR(128),
                            comment TEXT,
                            lyrics TEXT DEFAULT "",
                            PRIMARY KEY(filename))""")
        # Populate the test database with enough information to make it useful
        for i in range(2):
            for j in range(2):
                for k in range(2):
                    total = i + j + k # to get unique track numbers for albums
                    db_row = ('/filename/%s' % str(i) + str(j) + str(k),
                              'title%s' % str(i) + str(j),
                              'artist0',
                              'album%d' % j,
                              'genre%d' % i,
                              i + j + k, # length
                              total, # tracknumber
                              i, # bitrate
                              'comment%d' % i,
                              i  # year
                              )
                    self.cursor.execute("""INSERT INTO track(filename,
                                                             title,
                                                             artist,
                                                             album,
                                                             genre,
                                                             length,
                                                             tracknumber,
                                                             bitrate,
                                                             comment,
                                                             year)
                                        VALUES(?,?,?,?,?,?,?,?,?,?)""", db_row)

        connection.commit()

    def tearDown(self):
        """Clean up after the test"""
        EntertainerTest.tearDown(self)

        connection = sqlite.connect(self.config.MUSIC_DB)
        cursor = connection.cursor()
        cursor.execute("""DROP TABLE IF EXISTS track""")
        connection.commit()
        connection.close()


class TestTrack(TestMusic):

    def setUp(self):
        TestMusic.setUp(self)
        self.track = Track('/path/to/track.mp3', # filename
                           'title',
                           1, # tracknumber
                           'artist0',
                           'album0',
                           'genre',
                           '128', # bitrate
                           2008, # year
                           5, # rating
                           240, # length
                           'comment',
                           'lyrics')

    def tearDown(self):
        TestMusic.tearDown(self)

    def testTrackConstructor(self):
        """testTrackConstructor - Ensures that a Track object is created"""
        self.assertTrue(isinstance(self.track, Track))

    def testTrackBadConstructor(self):
        """testTrackBadConstructor - Ensures that bad track construction
        raises an exception for the integer fields"""
        for i in [2, 7, 8, 9]:
            t = ['a', 'b', 1, 'c', 'd', 'e', 'f', 2, 3, 4, 'g', 'h']
            t[i] = str(t[i])
            self.assertRaises(TrackTypeError, Track, t[0], t[1], t[2], t[3],
                              t[4], t[5], t[6], t[7], t[8], t[9], t[10], t[11])

    def testTrackRatingInRange(self):
        """testTrackRatingInRange - Ensures that the ratings in range create
        valid Track objects"""
        for i in range(1, 6):
            self.track = Track('a', 'b', 1, 'c', 'd', 'e', 'f', 2, i, 3, 'g',
                               'h')
            self.assertTrue(isinstance(self.track, Track))
            self.assertEqual(self.track.get_rating(), i)

    def testTrackRatingOutOfRange(self):
        """testTrackRatingOutOfRange - Ensures that the rating raises an
        exception for something out of range in construction"""
        for i in [0, 6]:
            self.assertRaises(TrackRatingOutOfRange, Track, 'a', 'b', 1, 'c',
                              'd', 'e', 'f', 2, i, 3, 'g', 'h')

    def testTrackGetFilename(self):
        """testTrackGetFilename - Ensures that the filename is returned"""
        result = self.track.get_filename()
        self.assertEqual(result, '/path/to/track.mp3')

    def testTrackGetTitle(self):
        """testTrackGetTitle - Ensures that the title is returned"""
        result = self.track.get_title()
        self.assertEqual(result, 'title')

    def testTrackGetTrackNumber(self):
        """testTrackGetTrackNumber - Ensures that the tracknumber is
        returned"""
        result = self.track.get_tracknumber()
        self.assertEqual(result, 1)

    def testTrackGetArtist(self):
        """testTrackGetArtist - Ensures that the artist is returned"""
        result = self.track.get_artist()
        self.assertEqual(result, 'artist0')

    def testTrackGetAlbum(self):
        """testTrackGetAlbum - Ensures that an album object is returned"""
        result = self.track.get_album(self.cursor)
        self.assertTrue(isinstance(result, Album))
        self.assertEqual(result.get_title(), 'album0')

    def testTrackGetAlbumNot(self):
        """testTrackGetAlbumNot - Ensures that a bad album in the track
        returns AlbumHasNoTracks"""
        self.badTrack = Track('path', 'title', 1, 'artist',
                              'foo-bar-baz**', # Here is the bad input
                              'genre', '128', 2008, 5, 240, 'comment',
                              'lyrics')
        self.assertRaises(AlbumHasNoTracks, self.badTrack.get_album,
                          self.cursor)

    def testTrackGetAlbumArtUrlExists(self):
        """testTrackGetAlbumArtUrl - Ensures that the album art url is
        returned"""
        album_artist = "artist0 - album0"
        album_artist = album_artist.encode("base64")
        album_art = os.path.join(self.art_path, album_artist + ".jpg")
        open(album_art, "wb").close()
        result = self.track.get_album_art_url(self.cursor)
        self.assertEqual(result, album_art)
        if os.path.exists(album_art):
            os.remove(album_art)

    def testTrackGetAlbumArtUrlNotExists(self):
        """testTrackGetAlbumArtUrlNotExists - Ensures that when art does
        not exist, None is returned"""
        result = self.track.get_album_art_url(self.cursor)
        self.assertEqual(result, None)

    def testTrackGetGenre(self):
        """testTrackGetGenre - Ensures that the genre is returned"""
        result = self.track.get_genre()
        self.assertEqual(result, 'genre')

    def testTrackGetBitrate(self):
        """testTrackGetBitrate - Ensures that the bitrate is returned"""
        result = self.track.get_bitrate()
        self.assertEqual(result, '128')

    def testTrackGetYear(self):
        """testTrackGetYear - Ensures that the year is returned"""
        result = self.track.get_year()
        self.assertEqual(result, 2008)

    def testTrackGetRating(self):
        """testTrackGetRating - Ensures that the rating is returned"""
        result = self.track.get_rating()
        self.assertEqual(result, 5)

    def testTrackGetLength(self):
        """testTrackGetLength - Ensures that the length is returned"""
        result = self.track.get_length()
        self.assertEqual(result, 240)

    def testTrackGetComment(self):
        """testTrackGetComment - Ensures that the comment is returned"""
        result = self.track.get_comment()
        self.assertEqual(result, 'comment')

    def testTrackGetLyrics(self):
        """testTrackGetLyrics - Ensures that the lyrics are returned"""
        result = self.track.get_lyrics()
        self.assertEqual(result, 'lyrics')

    def testTrackSetLyrics(self):
        """testTrackSetLyrics - Ensures that lyrics are properly set"""
        self.track.set_lyrics('some new lyrics')
        self.assertEqual(self.track.get_lyrics(), 'some new lyrics')

    def testTrackSetNoneLyrics(self):
        """testTrackSetNoneLyrics - Ensures that lyrics are set to empty
        string when None is passed as lyrics input"""
        self.track.set_lyrics(None)
        self.assertEqual(self.track.get_lyrics(), '')

    def testTrackGetType(self):
        """testTrackGetType - Ensures that the type is returned"""
        result = self.track.get_type()
        self.assertEqual(result, Playable.AUDIO_STREAM)

    def testTrackGetUri(self):
        """testTrackGetUri - Ensures that the uri is returned"""
        result = self.track.get_uri()
        self.assertEqual(result, 'file:///path/to/track.mp3')


class TestMusicLibrary(TestMusic):
    '''Tests MusicLibrary'''

    def setUp(self):
        TestMusic.setUp(self)
        self.backend_connection = MockBackendConnection()
        self.musiclibrary = MusicLibrary(self.backend_connection)

    def tearDown(self):
        self.backend_connection.close_connection()
        TestMusic.tearDown(self)

    def testMusicLibraryConstructor(self):
        """testMusicLibraryContructor - Ensures instantiation of MusicLibrary
        class"""
        self.assertTrue(isinstance(self.musiclibrary, MusicLibrary))

    def testGetAllArtists(self):
        """testGetAllArtists - Ensures that all artists are returned from
        the music library"""
        result = self.musiclibrary.get_all_artists()
        self.assertEqual(result, ['artist0'])

    def testGetAllTracks(self):
        """testGetAllTracks - Ensures that all tracks are returned"""
        result = self.musiclibrary.get_all_tracks()
        for i in result:
            self.assertTrue(isinstance(i, Track))
        self.assertEqual(len(result), 8)

    def testGetTracksByGenre(self):
        """testGetTracksByGenre - Ensures tracks of a certain genre are
        returned"""
        result = self.musiclibrary.get_tracks_by_genre('genre0')
        self.assertEqual(len(result), 4)
        for i in result:
            self.assertEqual(i.get_genre(), 'genre0')

    def testGetTracksByMissingGenre(self):
        """testGetTracksByMissingGenre - Ensures proper handling of a missing
        genre"""
        self.assertEquals(
            len(self.musiclibrary.get_tracks_by_genre('foo-bar-baz**')), 0)

    def testGetTracksByArtist(self):
        """testGetTracksByArtist - Ensure tracks by a certain artist are
        returned"""
        result = self.musiclibrary.get_tracks_by_artist('artist0')
        self.assertEqual(len(result), 8)
        for i in result:
            self.assertEqual(i.get_artist(), 'artist0')

    def testGetTracksByUnknownArtist(self):
        """testGetTracksByUnknownArtist - Ensure proper handling of an missing
        artist in the cache"""
        self.assertEquals(
            len(self.musiclibrary.get_tracks_by_artist('foo')), 0)

    def testGetAllAlbums(self):
        """testGetAllAlbums - Ensures all albums are returned"""
        result = self.musiclibrary.get_all_albums()
        for i in result:
            self.assertTrue(isinstance(i, Album))
        self.assertEqual(len(result), 2)

    def testGetAlbumsByArtist(self):
        """testGetAlbumsByArtist - Ensures correct albums by an artist is
        returned"""
        result = self.musiclibrary.get_albums_by_artist('artist0')
        self.assertEqual(len(result), 2)
        for i in result:
            self.assertEqual(i.get_artist(), 'artist0')

    def testGetAlbumsByUnknownArtist(self):
        """testGetAlbumsByUnknownArtist - Ensures proper handling of an
        artist not in the cache"""
        self.assertEquals(
            len(self.musiclibrary.get_albums_by_artist('foo')), 0)

    def testNumberOfTracks(self):
        """testNumberOfTracks - Ensures number of all tracks is returned"""
        result = self.musiclibrary.number_of_tracks()
        self.assertEqual(result, 8)

    def testNumberOfTracksByArtist(self):
        """testNumberOfTracksByArtist - Ensures number of all tracks by one
        artist is returned"""
        result = self.musiclibrary.number_of_tracks_by_artist('artist0')
        self.assertEqual(result, 8)

    def testNumberOfTracksByUnknownArtist(self):
        """testNumberOfTracksByUnknownArtist - Ensures proper handling when
        artist called is not in the cache"""
        self.assertEqual(
            self.musiclibrary.number_of_albums_by_artist('foo'), 0)

    def testNumberOfAlbumsByArtist(self):
        """testNumberOfAlbumsByArtist - Ensures number of all albums by one
        artist is returned"""
        result = self.musiclibrary.number_of_albums_by_artist('artist0')
        self.assertEqual(result, 2)

    def testNumberOfAlbumsByUnknownArtist(self):
        """testNumberOfAlbumsByUnknownArtist - Ensures proper handling when
        artist called is not in the cache"""
        self.assertEqual(
            self.musiclibrary.number_of_albums_by_artist('foo'), 0)

    def testSaveLyrics(self):
        """testSaveLyrics - Ensures lyrics for a track are saved in the
        database"""
        # Only need a filename that matches something in the database, the rest
        # of the track is for construction purposes only
        self.track = Track('/filename/000', '', 0, '', '', '', '', 0, 1, 0, '',
                           '')
        self.musiclibrary.save_lyrics(self.track, 'some lyrics here')
        self.assertEqual(self.track.get_lyrics(), 'some lyrics here')


class TestAlbum(TestMusic):
    '''Tests Album'''

    def setUp(self):
        """See unittest.TestCase"""
        TestMusic.setUp(self)
        self.album = Album('album1', self.cursor)

    def tearDown(self):
        """Clean up after the test"""
        TestMusic.tearDown(self)

    def testAlbumConstructor(self):
        """Test that an Album object is properly constructed"""
        self.assertTrue(isinstance(self.album, Album))

    def testAlbumConstructorNot(self):
        """Test that an AlbumHasNoTracks exception is raised when the created
        album doesn't exist in the cache"""
        self.assertRaises(AlbumHasNoTracks, Album, 'foo', self.cursor)

    def testAlbumStr(self):
        """Test that title is returned in string conversion"""
        result = str(self.album)
        self.assertEqual(result, 'album1')

    def testAlbumGetTitle(self):
        """Test that the album title is returned"""
        result = self.album.get_title()
        self.assertEqual(result, 'album1')

    def testAlbumHasAlbumArt(self):
        """Test that album art exists for the file"""
        album_artist = "artist0 - album1"
        album_artist = album_artist.encode("base64")
        album_art = os.path.join(self.art_path, album_artist + ".jpg")
        open(album_art, "wb").close()
        self.assertTrue(self.album.has_album_art())
        if os.path.exists(album_art):
            os.remove(album_art)

    def testAlbumHasAlbumArtNot(self):
        """Test that missing album art is reported back"""
        otherAlbum = Album('album1', self.cursor)
        self.assertFalse(otherAlbum.has_album_art())

    def testAlbumGetAlbumArtUrl(self):
        """Test that the path to the album's art is returned"""
        result = self.album.get_album_art_url()
        album_artist = "artist0 - album1"
        album_artist = album_artist.encode("base64")
        album_art = os.path.join(self.art_path, album_artist + ".jpg")
        self.assertEqual(result, album_art)

    def testAlbumGetTracks(self):
        """Test that all tracks for an album are returned"""
        result = self.album.get_tracks()
        self.assertEqual(len(result), 4)
        for i in result:
            self.assertTrue(isinstance(i, Track))

    def testAlbumGetNumberOfTracks(self):
        """Test correct number of tracks from album is returned"""
        self.assertEqual(self.album.get_number_of_tracks(), 4)

    def testAlbumGetYear(self):
        """Test that year of album is returned"""
        self.assertEqual(self.album.get_year(), 0)

    def testAlbumGetGenre(self):
        """Test that genre of album is returned"""
        self.assertEqual(self.album.get_genre(), 'genre0')

    def testAlbumGetArtist(self):
        """Test that artist of album is returned"""
        self.assertEqual(self.album.get_artist(), 'artist0')

    def testAlbumGetTotalLength(self):
        """Test that total length of album is returned"""
        self.assertEqual(self.album.get_total_length(), 8)

