'''Tests OPMLParser'''
# pylint: disable-msg=C0301

__licence__ = "GPLv2"
__copyright__ = "2008, Joshua Scotton"
__author__ = "Joshua Scotton <josh@joshuascotton.com>"

import unittest
import os
import xml.parsers.expat

from entertainerlib.utils.feed_utils import OPMLParser

THIS_DIR = os.path.dirname(__file__)


class OPMLParserTest(unittest.TestCase):
    '''Test OPMLParser'''

    def setUp(self):
        '''See unittest.TestCase'''

        #set up lists and paths
        self.testURL = "http://www.scripting.com/feeds/top100.opml"
        self.multipleLifereaFolders = (THIS_DIR +
            '/data/OPMLParser/lifereaMultiple')
        self.singleLifereaFolder = (THIS_DIR +
            '/data/OPMLParser/lifereaSingle')
        self.noLifereaFolder = (THIS_DIR +
            '/data/OPMLParser/lifereaNoFolder')
        self.noLifereaFile = (THIS_DIR +
            '/data/OPMLParser/lifereaNoFile')
        self.okFilePath = (THIS_DIR +
            '/data/OPMLParser/test.opml')
        self.badFilePath = THIS_DIR + '/data/OPMLParser/null.opml'
        self.notXMLFile = THIS_DIR + '/data/OPMLParser/noXML.txt'
        self.insideFilePath = (THIS_DIR +
            '/data/OPMLParser/opmlInOpml.opml')
        self.feedList = [
            "http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml",
            "http://www.hackaday.com/rss.xml",
            "http://edevelop.org/planet/rss20.xml",
            "http://feeds.feedburner.com/BlenderartMagazine"]

    def testGetRssFeeds001(self):
        """Check if OPMLParser can parse a normal file"""
        output = OPMLParser().get_rss_feeds(self.okFilePath)
        self.assertEqual(output, self.feedList)

    def testGetRssFeeds002(self):
        """Check if OPMLParser can handle a non-existant file"""
        self.assertRaises(IOError, OPMLParser().get_rss_feeds,
            self.badFilePath)

    def testGetRssFeeds003(self):
        """Check if OPMLParser discards opml links inside the file
        This is the same file as for testGetRssFeeds001 with an extra link to
        an opml file.

        The OPMLParser should discard the opml file and return the rest of the
        links
        """
        output = OPMLParser().get_rss_feeds(self.insideFilePath)
        self.assertEqual(output, self.feedList)

    def testGetRssFeeds004(self):
        """Check if OPMLParser can handle a file which is not an opml file or
        even an xml file
        """
        self.assertRaises(xml.parsers.expat.ExpatError,
            OPMLParser().get_rss_feeds, self.notXMLFile)

    def testGetRssFeeds005(self):
        """Check if OPMLParser can handle a URL"""
        output = OPMLParser().get_rss_feeds(self.testURL)
        self.assertTrue(output != [])

    def testGetLifereaOpml001(self):
        """Check if OPMLParser can pick the opml file for liferea"""
        output = OPMLParser().get_liferea_opml(self.singleLifereaFolder)
        self.assertEqual(output,
            self.singleLifereaFolder+"/.liferea/feedlist.opml")

    def testGetLifereaOpml002(self):
        """Check if OPMLParser can pick the opml file for the latest version
        of liferea out of a selection of folders
        """
        output = OPMLParser().get_liferea_opml(self.multipleLifereaFolders)
        self.assertEqual(output,
            self.multipleLifereaFolders+"/.liferea_1.4/feedlist.opml")

    def testGetLifereaOpml003(self):
        """Check if OPMLParser returns an empty string if it can't find the
        liferea opml folder
        """
        output = OPMLParser().get_liferea_opml(self.noLifereaFolder)
        self.assertEqual(output, "")

    def testGetLifereaOpml004(self):
        """Check if OPMLParser returns an empty string if it can't find the
        liferea opml file
        """
        output = OPMLParser().get_liferea_opml(self.noLifereaFile)
        self.assertEqual(output, "")

