#!/usr/bin/env python
'''Content management GUI for Entertainer'''

__licence__ = "GPLv2"
__copyright__ = "2008, Lauri Taimila and Joshua Scotton"
__author__ = ("Lauri Taimila <lauri@taimila.com>",
              "Joshua Scotton <josh@joshuascotton.com>")

import os
import socket

import gtk
import gtk.glade

from entertainerlib.utils.weather import Weather
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.open_feed_source_dialog import OpenFeedSourceDialog

from entertainerlib.backend.core.message import Message
from entertainerlib.backend.core.message_bus_proxy import MessageBusProxy
from entertainerlib.backend.core.message_type_priority import MessageType

class ContentManagementDialog:
    """
    This is a content management tool for Entertainer media center application.
    """

    # Temporary storage for entered URL
    url = ""
    GLADE_DIR = os.path.join(os.path.dirname(__file__), "glade")

    def __init__(self, stand_alone):
        """
        Initialize content management dialog
        @param stand_alone: Boolean, Is this dialog running as a stand alone
            process
        """
        self.stand_alone = stand_alone
        self.config = Configuration()
        self.weather = Weather()

        # Load glade UI
        self.gladefile = os.path.join(self.GLADE_DIR,
            "entertainer-content-management.glade")
        self.widgets = gtk.glade.XML(self.gladefile)

        # Get content management dialog and bind signal callbacks
        self.dialog = self.widgets.get_widget("ContentManagementDialog")
        if (self.dialog):
            callback_dic = {
                "on_button_open_list_clicked" :
                    self.on_button_open_list_clicked,
                "on_close_button_clicked" : self.on_close_button_clicked,
                "on_button_remove_videos_clicked" :
                    self.on_button_remove_videos_clicked,
                "on_button_add_videos_clicked" :
                    self.on_button_add_videos_clicked,
                "on_button_edit_videos_clicked" :
                    self.on_button_edit_videos_clicked,
                "on_checkbutton_video_metadata_toggled" :
                    self.on_checkbutton_video_metadata_toggled,
                "on_button_add_music_clicked" :
                    self.on_button_add_music_clicked,
                "on_button_remove_music_clicked" :
                    self.on_button_remove_music_clicked,
                "on_button_edit_music_clicked" :
                    self.on_button_edit_music_clicked,
                "on_lyrics_checkbox_toggled" : self.on_lyrics_checkbox_toggled,
                "on_art_checkbox_toggled" : self.on_art_checkbox_toggled,
                "on_button_add_images_clicked" :
                    self.on_button_add_images_clicked,
                "on_button_remove_images_clicked" :
                    self.on_button_remove_images_clicked,
                "on_button_edit_images_clicked" :
                    self.on_button_edit_images_clicked,
                "on_button_add_feed_clicked" :
                    self.on_button_add_feed_clicked,
                "on_button_remove_feed_clicked" :
                    self.on_button_remove_feed_clicked,
                "on_button_edit_feed_clicked" :
                    self.on_button_edit_feed_clicked,
                "on_fetch_interval_spinbutton_value_changed" :
                    self.on_fetch_interval_spinbutton_value_changed,
                "on_ContentManagementDialog_destroy" : self.on_dialog_closed,
                "on_url_dialog_delete_event" : self.on_url_dialog_delete_event,
                "on_url_dialog_ok_button_clicked" :
                    self.on_url_dialog_ok_button_clicked,
                "on_url_dialog_cancel_button_clicked" :
                    self.on_url_dialog_cancel_button_clicked,
                "on_button_video_rebuild_clicked" :
                    self.on_button_video_rebuild_clicked,
                "on_button_music_rebuild_clicked" :
                    self.on_button_music_rebuild_clicked,
                "on_button_image_rebuild_clicked" :
                    self.on_button_image_rebuild_clicked,
                "on_button_feed_rebuild_clicked" :
                    self.on_button_feed_rebuild_clicked,
                "on_button_add_weather_clicked" :
                    self.on_button_add_weather_clicked,
                "on_button_remove_weather_clicked" :
                    self.on_button_remove_weather_clicked,
                "on_weather_display_checkbox_toggled" :
                    self.on_weather_display_checkbox_toggled,
                "on_location_find_button_clicked" :
                    self.on_location_find_button_clicked,
                "on_location_cancel_button_clicked" :
                    self.on_location_cancel_button_clicked,
                "on_location_add_button_clicked" :
                    self.on_location_add_button_clicked,
                "on_location_entry_activate" : self.on_location_entry_activate}
        self.widgets.signal_autoconnect(callback_dic)

        # Initialize dialog widgets with correct values and show dialog
        self.init_dialog_values_from_configure_file()
        self.dialog.resize(500, 300)
        self.dialog.show()

        # Initialize location list in search dialog
        result_list = self.widgets.get_widget("location_results_treeview")
        store = gtk.ListStore(str)
        result_list.set_model(store)
        cell_renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Location"), cell_renderer, text=0)
        result_list.append_column(column)

# Signal handlers

    def on_dialog_closed(self, widget):
        """Callback function for dialog's close button"""
        try:
            proxy = MessageBusProxy(client_name = "Content Management GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.CONTENT_CONF_UPDATED))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                    None, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                        "Entertainer backend is not running. "
                        "Cache cannot be rebuilt."
                    ))
            error.run()
            error.destroy()


        if(self.stand_alone):
            self.dialog.hide()
            self.dialog.destroy()
            gtk.main_quit()
        else:
            self.dialog.hide()
            self.dialog.destroy()

    def on_close_button_clicked(self, widget):
        """Callback function for dialog's close button"""
        try:
            proxy = MessageBusProxy(client_name = "Content Management GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.CONTENT_CONF_UPDATED))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                    None, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                        "Entertainer backend is not running. "
                        "Cache cannot be rebuilt."
                    ))
            error.run()
            error.destroy()

        if(self.stand_alone):
            self.dialog.hide()
            self.dialog.destroy()
            gtk.main_quit()
        else:
            self.dialog.hide()
            self.dialog.destroy()

    def on_button_add_videos_clicked(self, widget):
        """Opens add URL dialog. """
        widget = self.widgets.get_widget("treeview_videos")
        model = widget.get_model()
        # Open "Select folder" dialog
        dialog =  gtk.FileChooserDialog(_("Select video folder"), None,
            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                gtk.STOCK_OPEN,gtk.RESPONSE_OK),
            None)
        status = dialog.run()
        # If folder was selected we add it to model and update config file
        if(status == gtk.RESPONSE_OK):
            self.add_to_model_and_config(dialog.get_current_folder(), model,
                self.video_folders, "Videos")
        dialog.destroy()

    def on_button_remove_videos_clicked(self, widget):
        """Remove currently selected folder from video folders"""
        widget = self.widgets.get_widget("treeview_videos")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        rm_folder = model.get_value(selection[1], 0)
        self.video_folders.remove(rm_folder)
        str_folders = ";".join(self.video_folders)
        self.config.write_content_value("Videos", "folders", str_folders)
        model.remove(selection[1])

    def on_button_edit_videos_clicked(self, widget):
        """Edit currently selected folder"""
        widget = self.widgets.get_widget("treeview_videos")
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        folder = model.get_value(selection[1], 0)
        url_entry.set_text(folder)
        url_dialog.set_title(_("Edit URL"))
        status = url_dialog.run()
        if status == gtk.RESPONSE_OK and os.path.exists(self.url):
            # Update list model
            model.set_value(selection[1], 0, self.url)
            # Update configure file
            pos = self.video_folders.index(folder)
            self.video_folders.remove(folder)
            self.video_folders.insert(pos, self.url)
            str_folders = ";".join(self.video_folders)
            self.config.write_content_value("Videos", "folders",
                str_folders)

    def on_checkbutton_video_metadata_toggled(self, widget):
        """
        Download video file metadata from internet
        @param widget: GTK-Widget
        """
        self.config.write_content_value("Videos", "download_metadata",
            widget.get_active())

    def on_button_add_music_clicked(self, widget):
        """
        Opens add URL dialog
        @param widget: GTK-Widget
        """
        widget = self.widgets.get_widget("treeview_music")
        model = widget.get_model()
        # Open "Select folder" dialog
        dialog =  gtk.FileChooserDialog(_("Select music folder"), None,
            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN,
                gtk.RESPONSE_OK),
            None)
        status = dialog.run()
        # If folder was selected we add it to model and update config file
        if(status == gtk.RESPONSE_OK):
            self.add_to_model_and_config(dialog.get_current_folder(), model,
                self.music_folders, "Music")
        dialog.destroy()

    def on_button_remove_music_clicked(self, widget):
        """Remove currently selected folder from music folders"""
        widget = self.widgets.get_widget("treeview_music")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        rm_folder = model.get_value(selection[1], 0)
        self.music_folders.remove(rm_folder)
        str_folders = ";".join(self.music_folders)
        self.config.write_content_value("Music", "folders", str_folders)
        model.remove(selection[1])

    def on_button_edit_music_clicked(self, widget):
        """Edit currently selected music folder"""
        widget = self.widgets.get_widget("treeview_music")
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        folder = model.get_value(selection[1], 0)
        url_entry.set_text(folder)
        url_dialog.set_title(_("Edit URL"))
        status = url_dialog.run()
        if status == gtk.RESPONSE_OK and os.path.exists(self.url):
            # Update list model
            model.set_value(selection[1], 0, self.url)
            # Update configure file
            pos = self.music_folders.index(folder)
            self.music_folders.remove(folder)
            self.music_folders.insert(pos, self.url)
            str_folders = ";".join(self.music_folders)
            self.config.write_content_value("Music", "folders", str_folders)

    def on_button_add_images_clicked(self, widget):
        """Opens add URL dialog. """
        widget = self.widgets.get_widget("treeview_images")
        model = widget.get_model()
        # Open "Select folder" dialog
        dialog =  gtk.FileChooserDialog(_("Select image folder"), None,
            gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN,
                gtk.RESPONSE_OK),
            None)
        status = dialog.run()
        # If folder was selected we add it to model and update config file
        if(status == gtk.RESPONSE_OK):
            self.add_to_model_and_config(dialog.get_current_folder(), model,
                self.image_folders, "Images")
        dialog.destroy()

    def on_button_remove_images_clicked(self, widget):
        """Remove currently selected folder from images folders"""
        widget = self.widgets.get_widget("treeview_images")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        rm_folder = model.get_value(selection[1], 0)
        self.image_folders.remove(rm_folder)
        str_folders = ";".join(self.image_folders)
        self.config.write_content_value("Images", "folders", str_folders)
        model.remove(selection[1])

    def on_button_edit_images_clicked(self, widget):
        """Edit currently selected music folder"""
        widget = self.widgets.get_widget("treeview_images")
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        folder = model.get_value(selection[1], 0)
        url_entry.set_text(folder)
        url_dialog.set_title(_("Edit URL"))
        status = url_dialog.run()
        if status == gtk.RESPONSE_OK and os.path.exists(self.url):
            # Update list model
            model.set_value(selection[1], 0, self.url)
            # Update configure file
            pos = self.image_folders.index(folder)
            self.image_folders.remove(folder)
            self.image_folders.insert(pos, self.url)
            str_folders = ";".join(self.image_folders)
            self.config.write_content_value("Images", "folders",
                str_folders)

    def on_button_add_feed_clicked(self, widget):
        """Opens add feed dialog. """
        widget = self.widgets.get_widget("treeview_feeds")
        url_dialog = self.widgets.get_widget("url_dialog")
        model = widget.get_model()
        # Open dialog
        url_dialog.set_title(_("Add RSS-feed"))
        status = url_dialog.run()
        # If folder was selected we add it to model and update config file
        if(status == gtk.RESPONSE_OK):
            model.append([self.url])
            self.feeds.append(self.url)
            str_folders = ";".join(self.feeds)
            self.config.write_content_value("RSS", "feeds", str_folders)

    def on_button_remove_feed_clicked(self, widget):
        """Remove currently selected reed from RSS-feeds"""
        widget = self.widgets.get_widget("treeview_feeds")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        rm_folder = model.get_value(selection[1], 0)
        self.feeds.remove(rm_folder)
        str_folders = ";".join(self.feeds)
        self.config.write_content_value("RSS", "feeds", str_folders)
        model.remove(selection[1])

    def on_button_edit_feed_clicked(self, widget):
        """Edit currently selected feed"""
        widget = self.widgets.get_widget("treeview_feeds")
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        model = widget.get_model()
        selection = widget.get_selection().get_selected()
        feed = model.get_value(selection[1], 0)
        url_entry.set_text(feed)
        url_dialog.set_title(_("Edit feed"))
        status = url_dialog.run()
        if status == gtk.RESPONSE_OK:
            # Update list model
            model.set_value(selection[1], 0, self.url)
            # Update configure file
            pos = self.feeds.index(feed)
            self.feeds.remove(feed)
            self.feeds.insert(pos, self.url)
            str_feeds = ";".join(self.feeds)
            self.config.write_content_value("RSS", "feeds", str_feeds)

    def on_button_open_list_clicked(self, widget):
        """Opens the open feed source dialog"""
        open_dialog = OpenFeedSourceDialog(
            self.widgets.get_widget("treeview_feeds"), self.feeds)
        open_dialog.dialog.connect("destroy", open.on_closeButton_clicked)
        open_dialog.dialog.hide()
        open_dialog.dialog.destroy()

    def on_fetch_interval_spinbutton_value_changed(self, widget):
        self.config.write_content_value("RSS", "fetch_interval",
            widget.get_value_as_int())

    def on_lyrics_checkbox_toggled(self, widget):
        self.config.write_content_value("Music", "download_lyrics",
            widget.get_active())

    def on_art_checkbox_toggled(self, widget):
        self.config.write_content_value("Music", "download_album_art",
            widget.get_active())

    def on_url_dialog_ok_button_clicked(self, widget):
        """URL dialog OK button pressed. Sets self.url"""
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        url_dialog.hide()
        self.url = url_entry.get_text()
        url_entry.set_text("")
        url_dialog.response(gtk.RESPONSE_OK)

    def on_url_dialog_cancel_button_clicked(self, widget):
        """URL dialog cancelled. Hides dialog"""
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        url_dialog.hide()
        url_entry.set_text("")
        url_dialog.response(gtk.RESPONSE_CANCEL)

    def on_url_dialog_delete_event(self, widget, data):
        """Dialog's X clicked. Hides dialog"""
        url_dialog = self.widgets.get_widget("url_dialog")
        url_entry = self.widgets.get_widget("url_entry")
        url_dialog.hide()
        url_entry.set_text("")
        url_dialog.response(gtk.RESPONSE_CANCEL)
        return True

    def on_button_add_weather_clicked(self, widget):
        """
        Open location search dialog
        @param widget: GTK-Widget
        """
        location_dialog = self.widgets.get_widget("weather_search_dialog")
        location_dialog.set_title(_("Add location"))

        # Clear results
        result_list = self.widgets.get_widget("location_results_treeview")
        model = result_list.get_model()
        model.clear()

        status = location_dialog.run()
        if(status == gtk.RESPONSE_OK):
            print "Added"

    def on_button_remove_weather_clicked(self, widget):
        """
        Remove currently selected weather location from the location list
        @param widget: GTK-Widget
        """
        widget = self.widgets.get_widget("treeview_locations")
        model = widget.get_model()
        self.weather_locations = []
        str_folders = ""
        self.config.write_content_value("Weather", "location", str_folders)
        model.clear()

    def on_weather_display_checkbox_toggled(self, widget):
        """
        Checkbox that defines should we use weather conditions
        @param widget: GTK-Widget
        """
        self.config.write_content_value("Weather", "display_in_menu",
            widget.get_active())
        if widget.get_active():
            self.widgets.get_widget("button_add_weather").set_sensitive(True)
            self.widgets.get_widget(
                "button_remove_weather").set_sensitive(True)
            self.widgets.get_widget("treeview_locations").set_sensitive(True)
        else:
            self.widgets.get_widget("button_add_weather").set_sensitive(False)
            self.widgets.get_widget(
                "button_remove_weather").set_sensitive(False)
            self.widgets.get_widget("treeview_locations").set_sensitive(False)

    def on_location_find_button_clicked(self, widget):
        """
        Find location by search string
        @param widget: GTK-Widget
        """
        add_button = self.widgets.get_widget("location_add_button")
        search_term = self.widgets.get_widget("location_entry").get_text()
        result_list = self.widgets.get_widget("location_results_treeview")
        model = result_list.get_model()
        model.clear()
        if search_term != "":
            self.weather.set_location(search_term)
            self.weather.refresh()
            results = self.weather.get_forecasts()
            if len(results) > 0:
                add_button.set_sensitive(True)
                model.append([search_term])
                result_list.set_cursor(0)
            else:
                model.clear()
                model.append([_("Location Not Found!")])
                add_button.set_sensitive(False)

    def on_location_cancel_button_clicked(self, widget):
        """
        Close location search dialog without taking any actions.0
        @param widget: GTK-Widget
        """
        location_dialog = self.widgets.get_widget("weather_search_dialog")
        location_entry = self.widgets.get_widget("location_entry")
        location_dialog.hide()
        location_entry.set_text("")
        location_dialog.response(gtk.RESPONSE_CANCEL)

    def on_location_add_button_clicked(self, widget):
        """
        Add selected location to location list and close search dialog
        @param widget: GTK-Widget
        """
        self.weather_locations = []
        result_list = self.widgets.get_widget("location_results_treeview")
        model = result_list.get_model()
        selection = result_list.get_selection().get_selected()
        location_string = model.get_value(selection[1], 0)

        location_list = self.widgets.get_widget("treeview_locations")
        loc_model = location_list.get_model()
        loc_model.clear()
        loc_model.append([location_string])

        self.weather_locations.append(location_string)
        str_locations = ";".join(self.weather_locations)
        self.config.write_content_value("Weather", "location", str_locations)

        location_dialog = self.widgets.get_widget("weather_search_dialog")
        location_entry = self.widgets.get_widget("location_entry")
        location_dialog.hide()
        location_entry.set_text("")
        location_dialog.response(gtk.RESPONSE_CANCEL)

    def on_location_entry_activate(self, widget):
        """
        User hit enter on location entry to start search
        @param widget: GTK-Widget
        """
        self.on_location_find_button_clicked(widget)

    def on_button_video_rebuild_clicked(self, widget):
        """
        Rebuild video cache requested
        @param widget: GTK-Widget
        """
        try:
            proxy = MessageBusProxy(client_name = "Content Management GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.REBUILD_VIDEO_CACHE))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                None, gtk.DIALOG_MODAL,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                    "Entertainer backend is not running. "
                    "Cache cannot be rebuilt."
                ))
            error.run()
            error.destroy()

    def on_button_music_rebuild_clicked(self, widget):
        """
        Rebuild music cache requested
        @param widget: GTK-Widget
        """
        try:
            proxy = MessageBusProxy(client_name = "Content Management GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.REBUILD_MUSIC_CACHE))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                None, gtk.DIALOG_MODAL,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                    "Entertainer backend is not running. "
                    "Cache cannot be rebuilt."
                ))
            error.run()
            error.destroy()

    def on_button_image_rebuild_clicked(self, widget):
        """
        Rebuild image cache requested
        @param widget: GTK-Widget
        """
        try:
            proxy = MessageBusProxy(client_name = "Content Management GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.REBUILD_IMAGE_CACHE))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                None, gtk.DIALOG_MODAL,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                    "Entertainer backend is not running. "
                    "Cache cannot be rebuilt."
                ))
            error.run()
            error.destroy()

    def on_button_feed_rebuild_clicked(self, widget):
        """
        Rebuild feed cache requested
        @param widget: GTK-Widget
        """
        #We need the user to confirm the rebuild feed cache request
        dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING,
            gtk.BUTTONS_OK_CANCEL,
            _("This will completely remove any feed entries in the cache!"))
        status = dialog.run()
        #If user has ok'd the request send the message to the message bus
        if(status == gtk.RESPONSE_OK):
            try:
                proxy = MessageBusProxy(client_name = "Content Management GUI")
                proxy.connectToMessageBus()
                proxy.sendMessage(Message(MessageType.REBUILD_FEED_CACHE))
                proxy.disconnectFromMessageBus()
            except socket.error:
                error = gtk.MessageDialog(
                    None, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _(
                        "Entertainer backend is not running. "
                        "Cache cannot be rebuilt."
                    ))
                error.run()
                error.destroy()

        dialog.destroy()

    def init_dialog_values_from_configure_file(self):
        """Read configuration and set dialog widget values with read values.
        """
        # == Videos ==
        videolist_widget = self.widgets.get_widget("treeview_videos")
        videostore = gtk.ListStore(str)

        cell_renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Folders"), cell_renderer, text=0)
        videolist_widget.append_column(column)

        self.video_folders = self.config.get_video_folders()

        # Fill model with folders read from config file
        self.init_model(videostore, self.video_folders)

        videolist_widget.set_model(videostore)

        # Checkboxes
        metadata_checkbox = self.widgets.get_widget("video_metadata_checkbox")
        md_val = self.config.download_video_metadata()
        metadata_checkbox.set_active(md_val)

        # == Music ==
        musiclist_widget = self.widgets.get_widget("treeview_music")
        music_model = gtk.ListStore(str)

        music_cell = gtk.CellRendererText()
        music_column = gtk.TreeViewColumn(_("Folders"), music_cell, text=0)
        musiclist_widget.append_column(music_column)

        self.music_folders = self.config.get_music_folders()

        # Fill model with folders read from config file
        self.init_model(music_model, self.music_folders)

        musiclist_widget.set_model(music_model)

        # Checkboxes
        art_checkbox = self.widgets.get_widget("art_checkbox")
        art_val = self.config.download_album_art()
        art_checkbox.set_active(art_val)

        lyrics_checkbox = self.widgets.get_widget("lyrics_checkbox")
        lyrics_val = self.config.download_lyrics()
        lyrics_checkbox.set_active(lyrics_val)

        # == Images ==
        imagelist_widget = self.widgets.get_widget("treeview_images")
        images_model = gtk.ListStore(str)

        img_cell = gtk.CellRendererText()
        img_column = gtk.TreeViewColumn(_("Folders"), img_cell, text=0)
        imagelist_widget.append_column(img_column)

        self.image_folders = self.config.get_image_folders()

        # Fill model with folders read from config file
        self.init_model(images_model, self.image_folders)

        imagelist_widget.set_model(images_model)

        # == RSS-feeds ==
        feedlist_widget = self.widgets.get_widget("treeview_feeds")
        feed_model = gtk.ListStore(str)

        rss_cell = gtk.CellRendererText()
        rss_column = gtk.TreeViewColumn(_("Feeds"), rss_cell, text=0)
        feedlist_widget.append_column(rss_column)

        self.feeds = self.config.get_feeds()

        # Fill model with folders read from config file
        for i in range(len(self.feeds)):
            feed_model.insert(i, [self.feeds[i]])

        feedlist_widget.set_model(feed_model)

        # Interval spinner
        interval_spinner = self.widgets.get_widget("fetch_interval_spinbutton")
        interval_val = self.config.get_feed_fetch_interval()
        if interval_val < 15:
            interval_val = 15
        elif interval_val > 900:
            interval_val = 900
        interval_spinner.set_value(interval_val)

        # == Weather location ==
        locationlist_widget = self.widgets.get_widget("treeview_locations")
        location_model = gtk.ListStore(str)

        loc_cell = gtk.CellRendererText()
        location_column = gtk.TreeViewColumn(_("Location"), loc_cell, text=0)
        locationlist_widget.append_column(location_column)

        self.weather_location = self.config.get_weather_location()

        # Fill model with location read from config file
        location_model.insert(0, [self.weather_location])

        locationlist_widget.set_model(location_model)

        # Checkboxes
        weather_display_checkbox = self.widgets.get_widget(
            "weather_display_checkbox")
        display_val = self.config.display_weather_in_frontend()
        weather_display_checkbox.set_active(display_val)
        if not display_val:
            self.widgets.get_widget("button_add_weather").set_sensitive(False)
            self.widgets.get_widget("button_remove_weather").set_sensitive(
                False)
            self.widgets.get_widget("treeview_locations").set_sensitive(False)

    def add_to_model_and_config(self, selected_folder, model, folders, kind):
        """
        Add selected_folder to the model and the folders list while updating
        the configuration item section specified by type
        """
        if not selected_folder in folders:
            model.append([selected_folder])

            if(folders == None):
                folders = [selected_folder]
            else:
                folders.append(selected_folder)

            if "" in folders:
                folders.remove("")
            str_folders = ";".join(folders)
            self.config.write_content_value(kind, "folders", str_folders)

    def init_model(self, model, items):
        """Fill model with items from supplied list"""
        for i in range(len(items)):
            if not str(items[i]).strip() == "":
                model.insert(i, [items[i]])

