'''Implements dialog that allows users to see logged events'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os
import sys

import gtk
import gtk.glade

from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.logger import Logger


class LogViewer:
    """
    Implements dialog that allows user to see logged events.

    This dialog is used to check Entertainer logfiles. It reads all data from
    selected file and saves rows to self.log_rows. Then it filters unwanted
    rows away by calling self.filterMessages(). This method adds rows to
    ListStore, which is the model of TreeView object.

    Combobox and refresh -button actions read files again
    Checkbox actions just filter current rows again
    """

    GLADE_DIR = os.path.join(os.path.dirname(__file__), "glade")

    # Is this dialog running as a stand alone process
    __STAND_ALONE = None

    widgets = None
    dialog = None
    log_store = None
    log_rows = []

    gladefile = os.path.join(GLADE_DIR, "log_dialog.glade")

    def __init__(self, stand_alone):
        self.logfile_entertainer = Configuration().ENTERTAINER_LOG
        self.logger = Logger().getLogger('utils.log_viewer')

        self.__STAND_ALONE = stand_alone
        try:
            self.widgets = gtk.glade.XML(self.gladefile)
        except RuntimeError:
            self.logger.critical("Couldn't open glade file: " + self.gladefile)
            sys.exit(1)
        callback_dic = {
            "on_close_log_button_clicked" : self.on_close_log_button_clicked,
            "on_log_refresh_button_clicked" : self.update_log_rows,
            "on_checkbutton_debug_toggled" : self.filter_messages,
            "on_checkbutton_critical_toggled" : self.filter_messages,
            "on_checkbutton_error_toggled" : self.filter_messages,
            "on_checkbutton_warning_toggled" : self.filter_messages,
            "on_checkbutton_info_toggled" : self.filter_messages }

        self.widgets.signal_autoconnect(callback_dic)

        # Create log treeview
        treeview = self.widgets.get_widget("treeview_log")
        cell_renderer1 = gtk.CellRendererText()
        cell_renderer2 = gtk.CellRendererText()
        cell_renderer3 = gtk.CellRendererText()
        cell_renderer4 = gtk.CellRendererText()

        column1 = gtk.TreeViewColumn("Date")
        column1.pack_start(cell_renderer1, True)
        column1.set_attributes(cell_renderer1, text = 0)

        column2 = gtk.TreeViewColumn("Time")
        column2.pack_start(cell_renderer2, True)
        column2.set_attributes(cell_renderer2, text = 1)

        column3 = gtk.TreeViewColumn("Type")
        column3.pack_start(cell_renderer3, True)
        column3.set_attributes(cell_renderer3, text = 2)

        column4 = gtk.TreeViewColumn("Message")
        column4.pack_end(cell_renderer4, True)
        column4.set_attributes(cell_renderer4, text = 3)

        treeview.append_column(column1)
        treeview.append_column(column2)
        treeview.append_column(column3)
        treeview.append_column(column4)
        treeview.set_headers_visible(True)

        # Set model to view and read data from logfile
        self.log_store = gtk.ListStore(str, str, str, str)
        treeview.set_model(self.log_store)
        self.update_log_rows()

        # Show Log viewer dialog
        self.dialog = self.widgets.get_widget("LogDialog")
        self.dialog.resize(750, 500)
        self.dialog.connect("destroy", self.on_close_log_button_clicked)
        self.dialog.show()

    def update_log_rows(self, widget=None):
        """Read logfile and udpate treeview"""
        self.log_rows[:] = []

        try:
            for line in open(self.logfile_entertainer, 'r'):
                try:
                    line_table = line.split()
                    message = ' '.join(line_table[3:])
                    row = line_table[:3] + [message]
                    parsed_row = parse_row(row)
                    self.log_rows.append(parsed_row)
                except IndexError:
                    print "Cannot parse log line: ", line
        except IOError:
            print "Cannot find logfile: ", self.logfile_entertainer

        # Reverse so that the latest message is at top
        self.log_rows.reverse()
        # Filter unwated message types
        self.filter_messages()

    def filter_messages(self, widget = None):
        """Checks which message types should be displayed on treeview"""
        if self.log_store:
            self.log_store.clear()

        debug = self.widgets.get_widget("checkbutton_debug").get_active()
        critical = self.widgets.get_widget("checkbutton_critical").get_active()
        error = self.widgets.get_widget("checkbutton_error").get_active()
        warning = self.widgets.get_widget("checkbutton_warning").get_active()
        info = self.widgets.get_widget("checkbutton_info").get_active()

        for element in self.log_rows:
            if element[2] == "DEBUG" and debug:
                self.log_store.append(element)
            elif element[2] == "CRITICAL" and critical:
                self.log_store.append(element)
            elif element[2] == "ERROR" and error:
                self.log_store.append(element)
            elif element[2] == "WARNING" and warning:
                self.log_store.append(element)
            elif element[2] == "INFO" and info:
                self.log_store.append(element)

    # Signal handlers
    def on_close_log_button_clicked(self, widget):
        """
        If running as a stand alone process, quit.
        Otherwise only destroy dialog.
        """
        self.dialog.hide()
        self.dialog.destroy()
        if(self.__STAND_ALONE):
            gtk.main_quit()


def parse_row(row):
    """
    This parses the input list into a list suitable for the logviewer
    @author Joshua Scotton
    @param row The input list [Date, Time, Class, Type + Description]
    """
    if row[3][:5] == "DEBUG":
        return [row[0], row[1], "DEBUG",
            row[2] + ": " + row[3][5:]]
    elif row[3][:8] == "CRITICAL":
        return [row[0], row[1], "CRITICAL",
            row[2] + ": " + row[3][8:]]
    elif row[3][:5] == "ERROR":
        return [row[0], row[1], "ERROR",
            row[2] + ": " + row[3][5:]]
    elif row[3][:7] == "WARNING":
        return [row[0], row[1], "WARNING",
            row[2] + ": " + row[3][7:]]
    elif row[3][:4] == "INFO":
        return [row[0], row[1], "INFO",
            row[2] + ": " + row[3][4:]]

