"""Search and download lyrics from the internet."""

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = ("Lauri Taimila <lauri@taimila.com>",
              "Charclo Michael <charclo.michael@gmail.com>")

import urllib
import socket
import gobject
from xml.dom import minidom

class LyricsDownloader(object):
    """
    Search and download song lyrics from the internet.
    Update music cache if lyrics found.
    """

    # The permanent user ID from Lyricsfly
    # NOTICE: This is the personal user ID for Entertainer, if you want to
    # experiment with the API from lyricsfly you can get an ID here =>
    # http://lyricsfly.com/api/, don't use this one as abuse of our key may
    # invalidate it.
    _LYRICSFLY_KEY = 'YzIxOTM4M2NkNGQ4MmRmODEtZW50ZXJ0YWluZXItcHJvamVjdC5jb20='

    def __init__(self, title, artist, callback):
        """
        Initialize lyrics downloader.
        @param title: Title of the track
        @param artist: Artist of the track
        @param callback: Callback function which is called when search is over.
                   lyrics are given as a parameter to this callback function.
        """
        self.title = title.lower()
        self.artist = artist.lower()
        self.callback = callback

    def search(self):
        """
        Search lyrics and download if found. Search is done asynchronously.
        This method returns immediately and set callback is called when search
        is over.
        """
        gobject.timeout_add(2000, self._async_search)

    def _async_search(self):
        """
        Search lyrics and download if found
        """
        lyrics = ""
        self._clean_up_artist_title()
        lyrics_xml = self._get_lyrics_xml()
        lyrics = self._parse_lyrics_xml(lyrics_xml)
        self.callback(lyrics)

    def _clean_up_artist_title(self):
        """
        Clean up the artist and title.
        """
        # Clean spaces
        self.title = self.title.strip()
        self.artist = self.artist.strip()

        # Convert title and artist to use in url, special symbols have to be
        # replaced by a '%' not '%xx'
        # TODO: Find out what the special symbols are (', &, ...)
        # not letters, digits, spaces and ()$^*=:;|#@}{][!,.-_\
        self.artist = urllib.quote(self.artist.encode('utf-8'),
                                   "'&()$^*=:;|#@}{][!,\\")
        self.title = urllib.quote(self.title.encode('utf-8'),
                                   "'&()$^*=:;|#@}{][!,\\")

        self.artist = self.artist.replace("'", "%").replace("&", "%")
        self.title = self.title.replace("'", "%").replace("&", "%")

    def _get_lyrics_xml(self):
        """
        Download the lyrics XML-file.
        """
        lyrics_xml = None

        # timeout in seconds
        timeout = 5
        socket.setdefaulttimeout(timeout)

        url = "http://lyricsfly.com/api/api.php?i=%s&a=%s&t=%s" \
            % (self._LYRICSFLY_KEY.decode('base64'), self.artist, self.title)
        temp = urllib.urlopen(url)
        lyrics_xml = temp.read()

        return lyrics_xml

    def _parse_lyrics_xml(self, lyrics_xml):
        """Parse lyrics XML and return lyrics string"""
        xmldoc = minidom.parseString(lyrics_xml).documentElement

        # Get the lyric from the XML file
        lyrics = xmldoc.getElementsByTagName('tx')[0].firstChild.nodeValue

        # Clean spaces and enters
        lyrics = lyrics.strip().replace('\n', '')
        lyrics = lyrics.replace('[br]', '\n')

        # Add the artist and title to the top of the lyric
        lyrics = xmldoc.getElementsByTagName('ar')[0].firstChild.nodeValue + \
        ' - ' + xmldoc.getElementsByTagName('tt')[0].firstChild.nodeValue + \
        '\n\n' + lyrics

        xmldoc.unlink()

        return lyrics
