#!/usr/bin/env python
'''Preferences editing tool for Entertainer media center'''

import os
import shutil
import socket
import tarfile

import gtk
import gtk.glade

from entertainerlib.backend.core.message import Message
from entertainerlib.backend.core.message_bus_proxy import MessageBusProxy
from entertainerlib.backend.core.message_type_priority import MessageType
from entertainerlib.utils.configuration import Configuration
from entertainerlib.utils.theme import Theme


class PreferencesDialog:
    """This is a preferences editing tool for Entertainer media center."""

    # Glade file location
    GLADE_DIR = os.path.join(os.path.dirname(__file__), "glade")

    def __init__(self, stand_alone):
        """
        Initialize Preferenced dialog
        @param stand_alone: Is this dialog stand alone process or part of
            frontend
        """
        # Is this dialog running as a stand alone process
        self.STAND_ALONE = stand_alone
        self.themes = []
        self.config = Configuration()

        # Load glade UI
        self.gladefile = os.path.join(self.GLADE_DIR,
            "entertainer-preferences.glade")
        self.widgets = gtk.glade.XML(self.gladefile)

        # Get preferences dialog and bind signal callbacks
        self.pref_dialog = self.widgets.get_widget("PreferencesDialog")
        if (self.pref_dialog):
            callback_dic = {
                "on_close_button_clicked" : self.on_close_button_clicked,
                "on_PreferencesDialog_delete_event" : self.on_dialog_closed,
                "on_theme_list_cursor_changed" :
                    self.on_theme_list_cursor_changed,
                "on_checkbutton_autostart_toggled" :
                    self.on_checkbutton_autostart_toggled,
                "on_checkbutton_systray_icon_toggled" :
                    self.on_checkbutton_systray_icon_toggled,
                "on_effect_checkbox_toggled" : self.on_effect_checkbox_toggled,
                "on_effect_combobox_changed" : self.on_effect_combobox_changed,
                "on_add_button_clicked" : self.on_add_button_clicked,
                "on_remove_button_clicked" : self.on_remove_button_clicked,
                "on_spinbutton_slideshow_step_value_changed":
                    self.on_spinbutton_slideshow_step_value_changed
            }
        self.widgets.signal_autoconnect(callback_dic)

        # Initilize dialog widgets with correct values and show dialog
        self.init_dialog_values_from_configure_file()
        #self.pref_dialog.resize(500,400)
        self.pref_dialog.show()

    def load_themes(self):
        """Load themes"""
        themes = os.listdir(os.path.join(self.config.get_cfg_dir(), 'themes'))
        for element in themes:
            theme = os.path.join(self.config.get_cfg_dir(), 'themes', element)
            if os.path.isdir(theme):
                self.themes.append(element)

    def on_add_button_clicked(self, widget):
        """Add theme button clicked"""
        themelist = self.widgets.get_widget("theme_list")
        model = themelist.get_model()
        # Open "Select folder" dialog
        dialog = gtk.FileChooserDialog(_("Select theme package file"),
            None, gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL,
            gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK), None)
        file_filter = gtk.FileFilter()
        file_filter.set_name(_("Theme package (tar.gz)"))
        file_filter.add_pattern("*.tar.gz")
        dialog.add_filter(file_filter)
        status = dialog.run()

        # If theme was selected with file chooser
        if(status == gtk.RESPONSE_OK):
            package = dialog.get_filename()
            tar = tarfile.open(package, 'r:gz') # Open tar.gz package

            # Make sure that package contains configuration file (is theme)
            content = tar.getnames()
            theme_name = None
            is_theme = False
            for element in content:
                if element[-10:] == "theme.conf":
                    theme_name = element[:-11]
                    is_theme = True

            # Install them
            if is_theme:
                tar.extractall(os.path.join(self.config.get_cfg_dir(),
                    'themes'))
                model.insert(len(model), [theme_name])
            else:
                error = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Invalid theme file!"))
                error.run()
                error.destroy()

        dialog.destroy()

    def on_remove_button_clicked(self, widget):
        """Remove theme button clicked"""
        # Get currently selected theme
        themelist = self.widgets.get_widget("theme_list")
        model = themelist.get_model()
        selection = themelist.get_selection().get_selected()
        name = model.get_value(selection[1], 0)

        confirm = gtk.MessageDialog(None,
            gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
                _("Are you sure you want to delete\ntheme: %(name)s") % \
                {'name': name})
        status = confirm.run()
        confirm.destroy()
        if(status == gtk.RESPONSE_YES):
            themedir = os.path.join(self.config.get_cfg_dir(), 'themes', name)
            shutil.rmtree(themedir)
            model.remove(selection[1])
            themelist.set_cursor(0)
            self.themes.remove(name)

    def on_theme_list_cursor_changed(self, widget):
        """Executed when theme is changed in theme list. Update preview."""
        # Get currently selected theme
        themelist = self.widgets.get_widget("theme_list")
        model = themelist.get_model()
        selection = themelist.get_selection().get_selected()
        name = model.get_value(selection[1], 0)
        themedir = os.path.join(self.config.get_cfg_dir(), 'themes', name)
        theme = Theme(theme_path=themedir)

        # Update preview
        image = self.widgets.get_widget("theme_image")
        image.set_from_file(os.path.join(themedir, "thumbnail.png"))
        name = self.widgets.get_widget("name_label")
        name.set_text(theme.getName())
        author = self.widgets.get_widget("author_label")
        author.set_text(theme.getAuthor())
        licence = self.widgets.get_widget("licence_label")
        licence.set_text(theme.getLicence())
        copyright_label = self.widgets.get_widget("copyright_label")
        copyright_label.set_text(theme.getCopyright())
        comment = self.widgets.get_widget("comment_label")
        comment.set_text(theme.getComment())

        self.config.write_preference_value("General", "theme", name.get_text())

    def on_checkbutton_systray_icon_toggled(self, widget):
        """System Tray Icon checkbox toggled"""
        self.config.write_preference_value("General", "display_icon",
            widget.get_active())

    def on_checkbutton_autostart_toggled(self, widget):
        """Server Autostart checkbox toggled"""
        self.config.write_preference_value("General", "start_server_auto",
            widget.get_active())

    def on_effect_checkbox_toggled(self, widget):
        """Effect checkbox toggled"""
        combobox = self.widgets.get_widget("effect_combobox")
        combobox.set_sensitive(widget.get_active())
        self.config.write_preference_value("General", "show_effects",
            widget.get_active())

    def on_effect_combobox_changed(self, widget):
        """User changed effect for screen transitions"""
        text = widget.get_active_text()
        if text == _("No effect"):
            english_text = "No effect"
        if text == _("Crossfade"):
            english_text = "Crossfade"
        if text == _("Zoom and fade"):
            english_text = "Zoom and fade"
        if text == _("Slide"):
            english_text = "Slide"
        self.config.write_preference_value("General", "transition_effect",
            english_text)

    def on_spinbutton_slideshow_step_value_changed(self, widget):
        """Activation of slideshow effects"""
        self.config.write_preference_value("Photographs", "slideshow_step",
            int(widget.get_value()))

    def on_dialog_closed(self, widget, after):
        """Callback function for dialog's close button"""
        self.close()

    def on_close_button_clicked(self, widget):
        """Callback function for dialog's close button"""
        self.close()

    def close(self):
        """Close Dialog"""
        try:
            proxy = MessageBusProxy(client_name = "Preferences GUI")
            proxy.connectToMessageBus()
            proxy.sendMessage(Message(MessageType.PREFERENCES_CONF_UPDATED))
            proxy.disconnectFromMessageBus()
        except socket.error:
            error = gtk.MessageDialog(
                    None, gtk.DIALOG_MODAL,
                    gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, _(
                        "Entertainer backend is not running. "
                        "Config updates will happen on backend restart."
                    ))
            error.run()
            error.destroy()


        if(self.STAND_ALONE):
            self.pref_dialog.hide()
            self.pref_dialog.destroy()
            gtk.main_quit()
        else:
            self.pref_dialog.hide()
            self.pref_dialog.destroy()

    def init_dialog_values_from_configure_file(self):
        """Read configure file and set dialog widget values with read values.
        """
        self.load_themes()
        current_theme = self.config.get_theme_name()

        themelist_widget = self.widgets.get_widget("theme_list")
        model = gtk.ListStore(str)

        cell_renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Themes", cell_renderer, text=0)
        themelist_widget.append_column(column)

        # Fill model with installed themes
        for i in range(len(self.themes)):
            model.insert(i, [self.themes[i]])

        themelist_widget.set_model(model)

        # Set current theme selected in theme list
        index = model.get_iter_first()
        unselected = True
        index_counter = 0
        while(unselected):
            name = model.get_value(index, 0)
            if name == current_theme:
                unselected = False
                themelist_widget.set_cursor(index_counter)
            index = model.iter_next(index)
            index_counter += 1

        # Checkboxes
        effect_checkbox = self.widgets.get_widget("effect_checkbox")
        effect_combobox = self.widgets.get_widget("effect_combobox")
        checkbutton_systray_icon = self.widgets.get_widget(
            "checkbutton_systray_icon")
        checkbutton_autostart = self.widgets.get_widget(
            "checkbutton_autostart")
        spinbutton_slideshow_step = self.widgets.get_widget(
            "spinbutton_slideshow_step")

        show_effects = self.config.show_effects()
        effect = self.config.transition_effect()
        start_icon = self.config.tray_icon_enabled()
        start_server_auto  = self.config.start_auto_server()
        slideshow_step = self.config.get_slideshow_step()

        if start_icon:
            checkbutton_systray_icon.set_active(True)
        else:
            checkbutton_systray_icon.set_active(False)

        if start_server_auto:
            checkbutton_autostart.set_active(True)
        else:
            checkbutton_autostart.set_active(False)

        if show_effects:
            effect_checkbox.set_active(True)
            effect_combobox.set_sensitive(True)
        else:
            effect_checkbox.set_active(False)
            effect_combobox.set_sensitive(False)

        spinbutton_slideshow_step.set_value(slideshow_step)

        # Set Effect Combobox value (Text values are set in glade file)
        if effect == "No effect":
            effect_combobox.set_active(0)
        if effect == "Crossfade":
            effect_combobox.set_active(1)
        if effect == "Zoom and fade":
            effect_combobox.set_active(2)
        if effect == "Slide":
            effect_combobox.set_active(3)
        if effect == "Slide and fade":
            effect_combobox.set_active(4)

