'''SystemTrayIcon class - Implements system tray icon for Entertainer.'''

__licence__ = "GPLv2"
__copyright__ = "2007, Lauri Taimila"
__author__ = "Lauri Taimila <lauri@taimila.com>"

import os

import gtk
import gtk.glade

from entertainerlib.utils.log_viewer import LogViewer
from entertainerlib.utils.preferences_dialog import PreferencesDialog
from entertainerlib.utils.content_management_dialog import (
    ContentManagementDialog)
from entertainerlib.utils.configuration import Configuration

class SystemTrayIcon:
    """Implements system tray icon for entertainer."""

    FILE_DIR = os.path.dirname(__file__)
    GLADE_DIR = os.path.join(FILE_DIR, "glade")

    def __init__(self, quit_callback, toggle_interface_visibility_callback):
        """
        Create system tray icon and pop-up menu for it.
        @author ["Joshua Scotton", "Lauri Taimila"]
        """
        self.quit_callback = quit_callback
        self.toggle_interface_visibility_callback = \
            toggle_interface_visibility_callback
        self.config = Configuration()

        # Path to the tray icon when using a branch
        self.tray_icon_url = os.path.join(self.FILE_DIR, "..", "..", "icons",
            "hicolor", "24x24", "apps", "entertainer.png")

        self.icon_widget = gtk.StatusIcon()
        self.icon_widget.set_tooltip(_("Entertainer Server"))

        # Load glade files
        self.menu_widgets = gtk.glade.XML(
            os.path.join(self.GLADE_DIR, "system_tray_icon_menu.glade"))

        # Bind menu signals
        callback_dic = {"on_menuitem_frontend_activate"
                        : self.on_menuitem_frontend_activate,
                        "on_menuitem_preferences_activate"
                        : self.on_menuitem_preferences_activate,
                        "on_menuitem_content_management_activate"
                        : self.on_menuitem_content_management_activate,
                        "on_menuitem_log_viewer_activate"
                        : self.on_menuitem_log_viewer_activate,
                        "on_menuitem_quit_activate"
                        : self.on_menuitem_quit_activate
                        }
        self.menu_widgets.signal_autoconnect(callback_dic)
        self.popup = self.menu_widgets.get_widget("SystemTrayIconMenu")

        # Check if running from a branch to set the tray icon
        if (os.path.exists(self.tray_icon_url)):
            self.icon_widget.set_from_file(self.tray_icon_url)
        else:
            # Must be running from a package, therefore available by icon name
            self.icon_widget.set_from_icon_name("entertainer")

        self.icon_widget.connect('activate', self.systray_icon_activated)
        self.icon_widget.connect('popup-menu', self.open_popup_menu)

    def systray_icon_activated(self, widget, data= None):
        """Switch visibility of frontend when system tray icon is clicked"""
        self.toggle_interface_visibility_callback()

    def open_popup_menu(self, widget, button, time, data = None):
        """Display pop-up menu when system tray icon is clicked"""
        self.popup.show_all()
        self.popup.popup(None, None, None, 3, time)

    def on_menuitem_frontend_activate(self, widget):
        """Execute frontend here if not running. Show if running"""
        self.set_frontend_visible(True)

    def on_menuitem_preferences_activate(self, widget):
        """Executes preferences-tool."""
        PreferencesDialog(False)

    def on_menuitem_content_management_activate(self, widget):
        """Executes content-management-tool"""
        ContentManagementDialog(False)

    def on_menuitem_log_viewer_activate(self, widget):
        """Display log viewer dialog"""
        LogViewer(False)

    def on_menuitem_quit_activate(self, widget):
        '''Close the application by calling the quit callback.'''
        self.quit_callback()

