"""
Weather Class
  Uses google weathers api to get a weather forecast for the next
  4 days.
"""
__licence__ = "GPLv2"
__copyright__ = ("2008, Jamie Bennett", "2008, Samuel Buffet")
__author__ = ("Jamie Bennett <jamie@linuxuk.org>", "Samuel Buffet")

import locale
import urllib2
from xml.dom import minidom

from entertainerlib.utils.configuration import Configuration

#The lookup url for the google weather API
WEATHER_LOOKUP_URL = "http://www.google.co.uk/ig/api?weather="

#use the icon instead of condition to make it work with any languages
WEATHER_IMAGES = {'/images/weather/mostly_sunny.gif': 'weather-sun-mostly',
                  '/images/weather/chance_of_rain.gif': 'weather-rain-chance',
                  #'/images/weather/partly_sunny.gif': 'weather-sun-clouds', 
                  # no dedicated gif provided by google for that weather 
                  # condition.
                  # it's a *mostly_sunny* gif wich is given instead :(
                  '/images/weather/rain.gif': 'weather-rain',
                  '/images/weather/storm.gif': 'weather-lightning',
                  '/images/weather/fog.gif': 'weather-fog',
                  '/images/weather/sunny.gif': 'weather-sun',
                  '/images/weather/icy.gif': 'weather-ice',
                  '/images/weather/cloudy.gif': 'weather-clouds',
                  '/images/weather/snow.gif': 'weather-snow'
                  }

class Weather:
    """
    Weather class
    @author: Jamie Bennett <jamie@linuxuk.org>
    """

    def __init__(self, location=""):
        """
        Sets up the weather object
        @param location (String) location of weather to get forecast
        for.
        """
        self.__location = ''
        self.__forecasts = []
        self.theme = Configuration().theme
        self.set_location(location)
        self.locale = self.__get_locale ()
        self.refresh()

    def __get_locale (self):
        '''Get locale information from user\'s machine'''
        default = locale.getdefaultlocale ()[0]
        if default:
            lang = default.split("_")[0]
        else:
            lang = 'en'

        return lang

    def find_forecast(self, search):
        """
        Returns the search results page for the search
        @param search String
        @return array of forecasts
        """
        url = WEATHER_LOOKUP_URL + search + "&hl=" + self.locale
        url = url.replace(" ", "%20")

        raw_data = urllib2.urlopen(url)
        # We get the charset from the content-type of the http answer
        conent_type = raw_data.info()["Content-Type"]
        encoding = conent_type.split("=")[1]

        # we read data according to the relevant charset and then we
        # prepare it to be parsed by minidom
        data = unicode(raw_data.read(), encoding)
        data = data.encode("utf8")
        dom = minidom.parseString(data)

        in_forecast = 0
        day = ''
        low = ''
        high = ''
        condition = ''
        imagename = ''
        image = ''

        for node in dom.getElementsByTagNameNS('*', '*'):
            # a problem occured so bail out
            if (node.nodeName == 'problem_cause'):
                print _('Location not found or network problem')
                break

            if (node.nodeName == 'forecast_conditions'):
                in_forecast = 1

            if (in_forecast):
                if node.nodeName == 'day_of_week':
                    day = node.getAttribute('data')
                if node.nodeName == 'low':
                    low = node.getAttribute('data')
                    converted_low = int(low)
                if node.nodeName == 'high':
                    high = node.getAttribute('data')
                    converted_high = int(high)
                if node.nodeName == 'condition':
                    condition = node.getAttribute('data')
                if node.nodeName == 'icon':
                    imagename = node.getAttribute('data')
                    image = self.set_image (imagename)

            # condition is the last element of the forecast we are
            # interested in so write out the forecast if this is set.
                if (condition):
                    forecast = {"Day":day,
                                "Low":str(converted_low),
                                "High":str(converted_high),
                                "Condition":condition,
                                "Image":image}
                    self.add_forecast(forecast)
                    in_forecast = 0
                    day = ''
                    low = ''
                    high = ''
                    condition = ''
                    imagename = ''
                    image = ''

        return self.__forecasts

    def set_location(self, location):
        """
        Sets the current location
        @param type String should be a city, country name or
        post/zip code
        """
        self.__location = location

    def get_location(self):
        """
        Gets the current location
        @return type String of city, country name or post/zip code
        """
        return self.__location

    def clear_forecasts(self):
        """
        Resets the forecast array
        """
        self.__forecasts = []

    def add_forecast(self, forecast):
        """
        Appends a forecast to the forecast array
        @param forecast dict
        """
        self.__forecasts.append(forecast)

    def get_forecasts(self):
        """
        Returns an array of forecasts
        @return array of forecasts
        """
        return self.__forecasts

    def set_image(self, condition):
        """
        Returns an image for a given weather condition.
        @return type String of weather image
        """
        try:
            image_name = WEATHER_IMAGES[condition]
        except KeyError:
            image_name = 'weather-na'

        image = self.theme.getImage(image_name)

        return image

    def refresh(self):
        """
        Clear current weather and forecasts and then loads new data
        from xml or rss feed
        """
        if (self.__location):
            self.clear_forecasts()
            self.find_forecast(self.__location)

